@@ -1,11 +1,16 @@
 #include "pdfapp.h"
-#include "curl_stream.h"
+#include "mupdf/helpers/pkcs7-check.h"
+#include "mupdf/helpers/pkcs7-openssl.h"
 
 #include <string.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
 
+#ifndef _WIN32
+#include <unistd.h> /* for getcwd */
+#endif
+
 #define BEYOND_THRESHHOLD 40
 
 #ifndef PATH_MAX
@@ -23,16 +28,13 @@ enum panning
 	PAN_TO_BOTTOM
 };
 
-enum
-{
-	PDFAPP_OUTLINE_DEFERRED = 1,
-	PDFAPP_OUTLINE_LOAD_NOW = 2
-};
-
 static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repaint, int transition, int searching);
-static void pdfapp_updatepage(pdfapp_t *app);
 
-static const int zoomlist[] = { 18, 24, 36, 54, 72, 96, 120, 144, 180, 216, 288 };
+static const int zoomlist[] = {
+	18, 24, 36, 54, 72, 96, 120, 144, 180,
+	216, 288, 360, 432, 504, 576, 648, 720,
+	792, 864, 936, 1008, 1080, 1152
+};
 
 static int zoom_in(int oldres)
 {
@@ -89,6 +91,7 @@ char *pdfapp_usage(pdfapp_t *app)
 		"W\t\t-- zoom to fit window width\n"
 		"H\t\t-- zoom to fit window height\n"
 		"Z\t\t-- zoom to fit page\n"
+		"z\t\t-- reset zoom\n"
 		"[\t\t-- decrease font size (EPUB only)\n"
 		"]\t\t-- increase font size (EPUB only)\n"
 		"w\t\t-- shrinkwrap\n"
@@ -110,6 +113,8 @@ char *pdfapp_usage(pdfapp_t *app)
 		"N\t\t-- find previous search result\n"
 		"c\t\t-- toggle between color and grayscale\n"
 		"i\t\t-- toggle inverted color mode\n"
+		"C\t\t-- toggle tinted color mode\n"
+		"e\t\t-- enable/disable spot color mode\n"
 		"q\t\t-- quit\n"
 	;
 }
@@ -130,25 +135,23 @@ void pdfapp_init(fz_context *ctx, pdfapp_t *app)
 
 	app->transition.duration = 0.25f;
 	app->transition.type = FZ_TRANSITION_FADE;
-#if defined(_WIN32) || defined(_WIN64)
+#ifdef _WIN32
 	app->colorspace = fz_device_bgr(ctx);
 #else
 	app->colorspace = fz_device_rgb(ctx);
 #endif
-	app->tint_r = 255;
-	app->tint_g = 250;
-	app->tint_b = 240;
+	app->tint_white = 0xFFFAF0;
 }
 
 void pdfapp_setresolution(pdfapp_t *app, int res)
 {
+	app->default_resolution = res;
 	app->resolution = res;
 }
 
-void pdfapp_invert(pdfapp_t *app, const fz_rect *rect)
+void pdfapp_invert(pdfapp_t *app, fz_rect rect)
 {
-	fz_irect b;
-	fz_invert_pixmap_rect(app->ctx, app->image, fz_round_rect(&b, rect));
+	fz_invert_pixmap_rect(app->ctx, app->image, fz_round_rect(rect));
 }
 
 void pdfapp_reloadfile(pdfapp_t *app)
@@ -187,10 +190,6 @@ static void event_cb(fz_context *ctx, pdf_document *doc, pdf_doc_event *event, v
 		}
 		break;
 
-	case PDF_DOCUMENT_EVENT_EXEC_DIALOG:
-		pdfapp_warn(app, "The document attempted to open a dialog box. (Not supported)");
-		break;
-
 	case PDF_DOCUMENT_EVENT_LAUNCH_URL:
 		{
 			pdf_launch_url_event *launch_url = pdf_access_launch_url_event(ctx, event);
@@ -213,27 +212,6 @@ static void event_cb(fz_context *ctx, pdf_document *doc, pdf_doc_event *event, v
 	}
 }
 
-void pdfapp_open(pdfapp_t *app, char *filename, int reload)
-{
-	pdfapp_open_progressive(app, filename, reload, 0);
-}
-
-#ifdef HAVE_CURL
-static void
-pdfapp_more_data(void *app_, int complete)
-{
-	pdfapp_t *app = (pdfapp_t *)app_;
-
-	if (complete && app->outline_deferred == PDFAPP_OUTLINE_DEFERRED)
-	{
-		app->outline_deferred = PDFAPP_OUTLINE_LOAD_NOW;
-		winreloadpage(app);
-	}
-	else if (app->incomplete)
-		winreloadpage(app);
-}
-#endif
-
 static int make_fake_doc(pdfapp_t *app)
 {
 	fz_context *ctx = app->ctx;
@@ -251,7 +229,6 @@ static int make_fake_doc(pdfapp_t *app)
 
 		pdf = pdf_create_document(ctx);
 
-
 		contents = fz_new_buffer(ctx, 100);
 		fz_append_printf(ctx, contents, "1 0 0 RG %g w 0 0 m %g %g l 0 %g m %g 0 l s\n",
 			fz_min(mediabox.x1, mediabox.y1) / 20,
@@ -261,7 +238,7 @@ static int make_fake_doc(pdfapp_t *app)
 		/* Create enough copies of our blank(ish) page so that the
 		 * page number is preserved if and when a subsequent load
 		 * works. */
-		page_obj = pdf_add_page(ctx, pdf, &mediabox, 0, NULL, contents);
+		page_obj = pdf_add_page(ctx, pdf, mediabox, 0, NULL, contents);
 		for (i = 0; i < app->pagecount; i++)
 			pdf_insert_page(ctx, pdf, -1, page_obj);
 	}
@@ -280,7 +257,7 @@ static int make_fake_doc(pdfapp_t *app)
 	return 0;
 }
 
-void pdfapp_open_progressive(pdfapp_t *app, char *filename, int reload, int bps)
+void pdfapp_open(pdfapp_t *app, char *filename, int reload)
 {
 	fz_context *ctx = app->ctx;
 	char *password = "";
@@ -299,57 +276,7 @@ void pdfapp_open_progressive(pdfapp_t *app, char *filename, int reload, int bps)
 
 		fz_set_use_document_css(ctx, app->layout_use_doc_css);
 
-#ifdef HAVE_CURL
-		if (!strncmp(filename, "http://", 7) || !strncmp(filename, "https://", 8))
-		{
-			app->stream = fz_stream_from_curl(ctx, filename, pdfapp_more_data, app);
-			while (1)
-			{
-				fz_try(ctx)
-				{
-					fz_seek(ctx, app->stream, 0, SEEK_SET);
-					app->doc = fz_open_document_with_stream(ctx, filename, app->stream);
-				}
-				fz_catch(ctx)
-				{
-					if (fz_caught(ctx) == FZ_ERROR_TRYLATER)
-					{
-						pdfapp_warn(app, "not enough data to open yet");
-						continue;
-					}
-					fz_rethrow(ctx);
-				}
-				break;
-			}
-		}
-		else
-#endif
-		if (bps == 0)
-		{
-			app->doc = fz_open_document(ctx, filename);
-		}
-		else
-		{
-			fz_stream *stream = fz_open_file_progressive(ctx, filename, bps);
-			while (1)
-			{
-				fz_try(ctx)
-				{
-					fz_seek(ctx, stream, 0, SEEK_SET);
-					app->doc = fz_open_document_with_stream(ctx, filename, stream);
-				}
-				fz_catch(ctx)
-				{
-					if (fz_caught(ctx) == FZ_ERROR_TRYLATER)
-					{
-						pdfapp_warn(app, "not enough data to open yet");
-						continue;
-					}
-					fz_rethrow(ctx);
-				}
-				break;
-			}
-		}
+		app->doc = fz_open_document(ctx, filename);
 	}
 	fz_catch(ctx)
 	{
@@ -398,39 +325,18 @@ void pdfapp_open_progressive(pdfapp_t *app, char *filename, int reload, int bps)
 
 		fz_layout_document(app->ctx, app->doc, app->layout_w, app->layout_h, app->layout_em);
 
-		while (1)
+		app->pagecount = fz_count_pages(app->ctx, app->doc);
+		if (app->pagecount <= 0)
+			fz_throw(ctx, FZ_ERROR_GENERIC, "No pages in document");
+
+		fz_try(ctx)
 		{
-			fz_try(ctx)
-			{
-				app->pagecount = fz_count_pages(app->ctx, app->doc);
-				if (app->pagecount <= 0)
-					fz_throw(ctx, FZ_ERROR_GENERIC, "No pages in document");
-			}
-			fz_catch(ctx)
-			{
-				if (fz_caught(ctx) == FZ_ERROR_TRYLATER)
-				{
-					pdfapp_warn(app, "not enough data to count pages yet");
-					continue;
-				}
-				fz_rethrow(ctx);
-			}
-			break;
+			app->outline = fz_load_outline(app->ctx, app->doc);
 		}
-		while (1)
+		fz_catch(ctx)
 		{
-			fz_try(ctx)
-			{
-				app->outline = fz_load_outline(app->ctx, app->doc);
-			}
-			fz_catch(ctx)
-			{
-				app->outline = NULL;
-				if (fz_caught(ctx) == FZ_ERROR_TRYLATER)
-					app->outline_deferred = PDFAPP_OUTLINE_DEFERRED;
-				else
-					pdfapp_warn(app, "failed to load outline");
-			}
+			app->outline = NULL;
+			pdfapp_warn(app, "failed to load outline");
 			break;
 		}
 	}
@@ -467,6 +373,9 @@ void pdfapp_close(pdfapp_t *app)
 	fz_drop_display_list(app->ctx, app->annotations_list);
 	app->annotations_list = NULL;
 
+	fz_drop_separations(app->ctx, app->seps);
+	app->seps = NULL;
+
 	fz_drop_stext_page(app->ctx, app->page_text);
 	app->page_text = NULL;
 
@@ -497,10 +406,6 @@ void pdfapp_close(pdfapp_t *app)
 	fz_drop_document(app->ctx, app->doc);
 	app->doc = NULL;
 
-#ifdef HAVE_CURL
-	fz_drop_stream(app->ctx, app->stream);
-#endif
-
 	fz_flush_warnings(app->ctx);
 }
 
@@ -548,6 +453,8 @@ static int pdfapp_save(pdfapp_t *app)
 		{
 			wincopyfile(app->docpath, buf);
 			pdf_save_document(app->ctx, idoc, buf, &opts);
+			pdfapp_close(app);
+			pdfapp_open(app, buf, 1);
 			return 1;
 		}
 
@@ -605,7 +512,7 @@ int pdfapp_preclose(pdfapp_t *app)
 
 static void pdfapp_viewctm(fz_matrix *mat, pdfapp_t *app)
 {
-	fz_pre_rotate(fz_scale(mat, app->resolution/72.0f, app->resolution/72.0f), app->rotate);
+	*mat = fz_transform_page(app->page_bbox, app->resolution, app->rotate);
 }
 
 static void pdfapp_panview(pdfapp_t *app, int newx, int newy)
@@ -651,12 +558,14 @@ static void pdfapp_loadpage(pdfapp_t *app, int no_cache)
 
 	fz_drop_display_list(app->ctx, app->page_list);
 	fz_drop_display_list(app->ctx, app->annotations_list);
+	fz_drop_separations(app->ctx, app->seps);
 	fz_drop_stext_page(app->ctx, app->page_text);
 	fz_drop_link(app->ctx, app->page_links);
 	fz_drop_page(app->ctx, app->page);
 
 	app->page_list = NULL;
 	app->annotations_list = NULL;
+	app->seps = NULL;
 	app->page_text = NULL;
 	app->page_links = NULL;
 	app->page = NULL;
@@ -665,107 +574,65 @@ static void pdfapp_loadpage(pdfapp_t *app, int no_cache)
 	app->page_bbox.x1 = 100;
 	app->page_bbox.y1 = 100;
 
-	app->incomplete = 0;
-
 	fz_try(app->ctx)
 	{
 		app->page = fz_load_page(app->ctx, app->doc, app->pageno - 1);
-
-		fz_bound_page(app->ctx, app->page, &app->page_bbox);
+		app->page_bbox = fz_bound_page(app->ctx, app->page);
 	}
 	fz_catch(app->ctx)
 	{
-		if (fz_caught(app->ctx) == FZ_ERROR_TRYLATER)
-			app->incomplete = 1;
-		else
-			pdfapp_warn(app, "Cannot load page");
+		pdfapp_warn(app, "Cannot load page");
 		return;
 	}
 
+	if (app->useseparations)
+	{
+		fz_try(app->ctx)
+		{
+			app->seps = fz_page_separations(app->ctx, app->page);
+			if (app->seps)
+			{
+				int i, n = fz_count_separations(app->ctx, app->seps);
+				for (i = 0; i < n; i++)
+					fz_set_separation_behavior(app->ctx, app->seps, i, FZ_SEPARATION_COMPOSITE);
+			}
+			else if (fz_page_uses_overprint(app->ctx, app->page))
+			{
+				/* This page uses overprint, so we need an empty
+				 * sep object to force the overprint simulation on. */
+				app->seps = fz_new_separations(app->ctx, 0);
+			}
+			else if (fz_document_output_intent(app->ctx, app->doc))
+			{
+				/* We have an output intent. Force the overprint
+				 *simulation on, because this ensures that
+				 * we 'simulate' the output intent too. */
+				app->seps = fz_new_separations(app->ctx, 0);
+			}
+		}
+		fz_catch(app->ctx)
+		{
+			pdfapp_warn(app, "Cannot load page");
+			errored = 1;
+		}
+	}
+
 	fz_try(app->ctx)
 	{
-		fz_annot *annot;
 		/* Create display lists */
-		app->page_list = fz_new_display_list(app->ctx, NULL);
+		app->page_list = fz_new_display_list(app->ctx, fz_infinite_rect);
 		mdev = fz_new_list_device(app->ctx, app->page_list);
 		if (no_cache)
 			fz_enable_device_hints(app->ctx, mdev, FZ_NO_CACHE);
-		cookie.incomplete_ok = 1;
-		fz_run_page_contents(app->ctx, app->page, mdev, &fz_identity, &cookie);
+		fz_run_page_contents(app->ctx, app->page, mdev, fz_identity, &cookie);
 		fz_close_device(app->ctx, mdev);
 		fz_drop_device(app->ctx, mdev);
 		mdev = NULL;
-		app->annotations_list = fz_new_display_list(app->ctx, NULL);
+		app->annotations_list = fz_new_display_list(app->ctx, fz_infinite_rect);
 		mdev = fz_new_list_device(app->ctx, app->annotations_list);
-		for (annot = fz_first_annot(app->ctx, app->page); annot; annot = fz_next_annot(app->ctx, annot))
-			fz_run_annot(app->ctx, annot, mdev, &fz_identity, &cookie);
-		if (cookie.incomplete)
-		{
-			app->incomplete = 1;
-			//pdfapp_warn(app, "Incomplete page rendering");
-		}
-		else if (cookie.errors)
-		{
-			pdfapp_warn(app, "Errors found on page");
-			errored = 1;
-		}
-		fz_close_device(app->ctx, mdev);
-	}
-	fz_always(app->ctx)
-	{
-		fz_drop_device(app->ctx, mdev);
-	}
-	fz_catch(app->ctx)
-	{
-		if (fz_caught(app->ctx) == FZ_ERROR_TRYLATER)
-			app->incomplete = 1;
-		else
-		{
-			pdfapp_warn(app, "Cannot load page");
-			errored = 1;
-		}
-	}
-
-	fz_try(app->ctx)
-	{
-		app->page_links = fz_load_links(app->ctx, app->page);
-	}
-	fz_catch(app->ctx)
-	{
-		if (fz_caught(app->ctx) == FZ_ERROR_TRYLATER)
-			app->incomplete = 1;
-		else if (!errored)
-			pdfapp_warn(app, "Cannot load page");
-	}
-
-	app->errored = errored;
-}
-
-static void pdfapp_recreate_annotationslist(pdfapp_t *app)
-{
-	fz_device *mdev = NULL;
-	int errored = 0;
-	fz_cookie cookie = { 0 };
-
-	fz_var(mdev);
-
-	fz_drop_display_list(app->ctx, app->annotations_list);
-	app->annotations_list = NULL;
-
-	fz_try(app->ctx)
-	{
-		fz_annot *annot;
-		/* Create display list */
-		app->annotations_list = fz_new_display_list(app->ctx, NULL);
-		mdev = fz_new_list_device(app->ctx, app->annotations_list);
-		for (annot = fz_first_annot(app->ctx, app->page); annot; annot = fz_next_annot(app->ctx, annot))
-			fz_run_annot(app->ctx, annot, mdev, &fz_identity, &cookie);
-		if (cookie.incomplete)
-		{
-			app->incomplete = 1;
-			//pdfapp_warn(app, "Incomplete page rendering");
-		}
-		else if (cookie.errors)
+		fz_run_page_annots(app->ctx, app->page, mdev, fz_identity, &cookie);
+		fz_run_page_widgets(app->ctx, app->page, mdev, fz_identity, &cookie);
+		if (cookie.errors)
 		{
 			pdfapp_warn(app, "Errors found on page");
 			errored = 1;
@@ -782,65 +649,31 @@ static void pdfapp_recreate_annotationslist(pdfapp_t *app)
 		errored = 1;
 	}
 
+	fz_try(app->ctx)
+	{
+		app->page_links = fz_load_links(app->ctx, app->page);
+	}
+	fz_catch(app->ctx)
+	{
+		if (!errored)
+			pdfapp_warn(app, "Cannot load page");
+	}
+
 	app->errored = errored;
 }
 
-static void pdfapp_runpage(pdfapp_t *app, fz_device *dev, const fz_matrix *ctm, const fz_rect *rect, fz_cookie *cookie)
+static void pdfapp_runpage(pdfapp_t *app, fz_device *dev, const fz_matrix ctm, fz_rect scissor, fz_cookie *cookie)
 {
 	if (app->page_list)
-		fz_run_display_list(app->ctx, app->page_list, dev, ctm, rect, cookie);
+		fz_run_display_list(app->ctx, app->page_list, dev, ctm, scissor, cookie);
 	if (app->annotations_list)
-		fz_run_display_list(app->ctx, app->annotations_list, dev, ctm, rect, cookie);
+		fz_run_display_list(app->ctx, app->annotations_list, dev, ctm, scissor, cookie);
 }
 
 #define MAX_TITLE 256
 
-static void pdfapp_updatepage(pdfapp_t *app)
-{
-	fz_device *idev;
-	fz_matrix ctm;
-	pdf_annot *pannot;
-
-	pdfapp_viewctm(&ctm, app);
-	pdf_update_page(app->ctx, (pdf_page *)app->page);
-	pdfapp_recreate_annotationslist(app);
-
-	for (pannot = pdf_first_annot(app->ctx, (pdf_page*)app->page); pannot; pannot = pdf_next_annot(app->ctx, pannot))
-	{
-		if (pdf_annot_is_dirty(app->ctx, pannot))
-		{
-			fz_annot *annot = (fz_annot*)pannot;
-			fz_rect bounds;
-			fz_irect ibounds;
-			fz_transform_rect(fz_bound_annot(app->ctx, annot, &bounds), &ctm);
-			fz_rect_from_irect(&bounds, fz_round_rect(&ibounds, &bounds));
-			fz_clear_pixmap_rect_with_value(app->ctx, app->image, 255, &ibounds);
-			idev = fz_new_draw_device_with_bbox(app->ctx, NULL, app->image, &ibounds);
-			fz_try(app->ctx)
-			{
-				pdfapp_runpage(app, idev, &ctm, &bounds, NULL);
-				fz_close_device(app->ctx, idev);
-			}
-			fz_always(app->ctx)
-				fz_drop_device(app->ctx, idev);
-			fz_catch(app->ctx)
-				fz_rethrow(app->ctx);
-		}
-	}
-
-	pdfapp_showpage(app, 0, 0, 1, 0, 0);
-}
-
 void pdfapp_reloadpage(pdfapp_t *app)
 {
-	if (app->outline_deferred == PDFAPP_OUTLINE_LOAD_NOW)
-	{
-		fz_try(app->ctx)
-			app->outline = fz_load_outline(app->ctx, app->doc);
-		fz_catch(app->ctx)
-			app->outline = NULL;
-		app->outline_deferred = 0;
-	}
 	pdfapp_showpage(app, 1, 1, 1, 0, 0);
 }
 
@@ -876,14 +709,15 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 		app->hit_count = 0;
 
 		/* Extract text */
-		app->page_text = fz_new_stext_page(app->ctx, fz_bound_page(app->ctx, app->page, &mediabox));
+		mediabox = fz_bound_page(app->ctx, app->page);
+		app->page_text = fz_new_stext_page(app->ctx, mediabox);
 
 		if (app->page_list || app->annotations_list)
 		{
 			tdev = fz_new_stext_device(app->ctx, app->page_text, NULL);
 			fz_try(app->ctx)
 			{
-				pdfapp_runpage(app, tdev, &fz_identity, &fz_infinite_rect, &cookie);
+				pdfapp_runpage(app, tdev, fz_identity, fz_infinite_rect, &cookie);
 				fz_close_device(app->ctx, tdev);
 			}
 			fz_always(app->ctx)
@@ -898,7 +732,7 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 		char buf2[64];
 		size_t len;
 
-		sprintf(buf2, " - %d/%d (%d dpi)",
+		sprintf(buf2, " - %d/%d (%g dpi)",
 				app->pageno, app->pagecount, app->resolution);
 		len = MAX_TITLE-strlen(buf2);
 		if (strlen(app->doctitle) > len)
@@ -912,9 +746,9 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 		wintitle(app, buf);
 
 		pdfapp_viewctm(&ctm, app);
-		bounds = app->page_bbox;
-		fz_round_rect(&ibounds, fz_transform_rect(&bounds, &ctm));
-		fz_rect_from_irect(&bounds, &ibounds);
+		bounds = fz_transform_rect(app->page_bbox, ctm);
+		ibounds = fz_round_rect(bounds);
+		bounds = fz_rect_from_irect(ibounds);
 
 		/* Draw */
 		fz_drop_pixmap(app->ctx, app->image);
@@ -929,18 +763,18 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 
 		fz_try(app->ctx)
 		{
-			app->image = fz_new_pixmap_with_bbox(app->ctx, colorspace, &ibounds, NULL, 1);
+			app->image = fz_new_pixmap_with_bbox(app->ctx, colorspace, ibounds, app->seps, 1);
 			fz_clear_pixmap_with_value(app->ctx, app->image, 255);
 			if (app->page_list || app->annotations_list)
 			{
-				idev = fz_new_draw_device(app->ctx, NULL, app->image);
-				pdfapp_runpage(app, idev, &ctm, &bounds, &cookie);
+				idev = fz_new_draw_device(app->ctx, fz_identity, app->image);
+				pdfapp_runpage(app, idev, ctm, bounds, &cookie);
 				fz_close_device(app->ctx, idev);
 			}
 			if (app->invert)
 				fz_invert_pixmap(app->ctx, app->image);
 			if (app->tint)
-				fz_tint_pixmap(app->ctx, app->image, app->tint_r, app->tint_g, app->tint_b);
+				fz_tint_pixmap(app->ctx, app->image, 0, app->tint_white);
 		}
 		fz_always(app->ctx)
 			fz_drop_device(app->ctx, idev);
@@ -956,7 +790,7 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 			colorspace = fz_device_gray(app->ctx);
 		else
 			colorspace = app->colorspace;
-		app->image = fz_new_pixmap_with_bbox(app->ctx, colorspace, &ibounds, NULL, 1);
+		app->image = fz_new_pixmap_with_bbox(app->ctx, colorspace, ibounds, app->seps, 1);
 		app->duration = 0;
 		fz_page_presentation(app->ctx, app->page, &app->transition, &app->duration);
 		if (app->duration == 0)
@@ -1012,6 +846,20 @@ static void pdfapp_showpage(pdfapp_t *app, int loadpage, int drawpage, int repai
 
 static void pdfapp_gotouri(pdfapp_t *app, char *uri)
 {
+	char buf[PATH_MAX];
+
+	/* Relative file:// URI, make it absolute! */
+	if (!strncmp(uri, "file://", 7) && uri[7] != '/')
+	{
+		char buf_base[PATH_MAX];
+		char buf_cwd[PATH_MAX];
+		fz_dirname(buf_base, app->docpath, sizeof buf_base);
+		getcwd(buf_cwd, sizeof buf_cwd);
+		fz_snprintf(buf, sizeof buf, "file://%s/%s/%s", buf_cwd, buf_base, uri+7);
+		fz_cleanname(buf+7);
+		uri = buf;
+	}
+
 	winopenuri(app, uri);
 }
 
@@ -1048,8 +896,9 @@ void pdfapp_inverthit(pdfapp_t *app)
 
 	for (i = 0; i < app->hit_count; i++)
 	{
-		bbox = app->hit_bbox[i];
-		pdfapp_invert(app, fz_transform_rect(&bbox, &ctm));
+		bbox = fz_rect_from_quad(app->hit_bbox[i]);
+		bbox = fz_transform_rect(bbox, ctm);
+		pdfapp_invert(app, bbox);
 	}
 }
 
@@ -1264,6 +1113,10 @@ void pdfapp_onkey(pdfapp_t *app, int c, int modifiers)
 	case 'Z':
 		pdfapp_autozoom(app);
 		break;
+	case 'z':
+		app->resolution = app->default_resolution;
+		pdfapp_showpage(app, 0, 1, 1, 0, 0);
+		break;
 
 	case 'L':
 		app->rotate -= 90;
@@ -1289,6 +1142,15 @@ void pdfapp_onkey(pdfapp_t *app, int c, int modifiers)
 		pdfapp_showpage(app, 0, 1, 1, 0, 0);
 		break;
 
+	case 'e':
+		app->useseparations ^= 1;
+		if (app->useseparations)
+			pdfapp_warn(app, "Using separations");
+		else
+			pdfapp_warn(app, "Not using separations");
+		pdfapp_showpage(app, 1, 1, 1, 0, 0);
+		break;
+
 #ifndef NDEBUG
 	case 'a':
 		app->rotate -= 15;
@@ -1558,7 +1420,7 @@ static void handlescroll(pdfapp_t *app, int modifiers, int dir)
 	if (modifiers & (1<<2))
 	{
 		/* zoom in/out if ctrl is pressed */
-		if (dir < 0)
+		if (dir > 0)
 			app->resolution = zoom_in(app->resolution);
 		else
 			app->resolution = zoom_out(app->resolution);
@@ -1622,122 +1484,14 @@ void pdfapp_onmouse(pdfapp_t *app, int x, int y, int btn, int modifiers, int sta
 	int processed = 0;
 
 	if (app->image)
-		fz_pixmap_bbox(app->ctx, app->image, &irect);
+		irect = fz_pixmap_bbox(app->ctx, app->image);
 	p.x = x - app->panx + irect.x0;
 	p.y = y - app->pany + irect.y0;
 
 	pdfapp_viewctm(&ctm, app);
-	fz_invert_matrix(&ctm, &ctm);
+	ctm = fz_invert_matrix(ctm);
 
-	fz_transform_point(&p, &ctm);
-
-	if (btn == 1 && (state == 1 || state == -1))
-	{
-		pdf_ui_event event;
-		pdf_document *idoc = pdf_specifics(app->ctx, app->doc);
-
-		event.etype = PDF_EVENT_TYPE_POINTER;
-		event.event.pointer.pt = p;
-		if (state == 1)
-			event.event.pointer.ptype = PDF_POINTER_DOWN;
-		else /* state == -1 */
-			event.event.pointer.ptype = PDF_POINTER_UP;
-
-		if (idoc && pdf_pass_event(ctx, idoc, (pdf_page *)app->page, &event))
-		{
-			pdf_widget *widget;
-
-			widget = pdf_focused_widget(ctx, idoc);
-
-			app->nowaitcursor = 1;
-			pdfapp_updatepage(app);
-
-			if (widget)
-			{
-				switch (pdf_widget_type(ctx, widget))
-				{
-				case PDF_WIDGET_TYPE_TEXT:
-					{
-						char *text = pdf_text_widget_text(ctx, idoc, widget);
-						char *current_text = text;
-						int retry = 0;
-
-						do
-						{
-							current_text = wintextinput(app, current_text, retry);
-							retry = 1;
-						}
-						while (current_text && !pdf_text_widget_set_text(ctx, idoc, widget, current_text));
-
-						fz_free(app->ctx, text);
-						pdfapp_updatepage(app);
-					}
-					break;
-				case PDF_WIDGET_TYPE_LISTBOX:
-				case PDF_WIDGET_TYPE_COMBOBOX:
-					{
-						int nopts;
-						int nvals;
-						char **opts = NULL;
-						char **vals = NULL;
-
-						fz_var(opts);
-						fz_var(vals);
-
-						fz_try(ctx)
-						{
-							nopts = pdf_choice_widget_options(ctx, idoc, widget, 0, NULL);
-							opts = fz_malloc(ctx, nopts * sizeof(*opts));
-							(void)pdf_choice_widget_options(ctx, idoc, widget, 0, opts);
-
-							nvals = pdf_choice_widget_value(ctx, idoc, widget, NULL);
-							vals = fz_malloc(ctx, MAX(nvals,nopts) * sizeof(*vals));
-							(void)pdf_choice_widget_value(ctx, idoc, widget, vals);
-
-							if (winchoiceinput(app, nopts, opts, &nvals, vals))
-							{
-								pdf_choice_widget_set_value(ctx, idoc, widget, nvals, vals);
-								pdfapp_updatepage(app);
-							}
-						}
-						fz_always(ctx)
-						{
-							fz_free(ctx, opts);
-							fz_free(ctx, vals);
-						}
-						fz_catch(ctx)
-						{
-							pdfapp_warn(app, "setting of choice failed");
-						}
-					}
-					break;
-
-				case PDF_WIDGET_TYPE_SIGNATURE:
-					if (state == -1)
-					{
-						char ebuf[256];
-
-						ebuf[0] = 0;
-						if (pdf_check_signature(ctx, idoc, widget, app->docpath, ebuf, sizeof(ebuf)))
-						{
-							winwarn(app, "Signature is valid");
-						}
-						else
-						{
-							if (ebuf[0] == 0)
-								winwarn(app, "Signature check failed for unknown reason");
-							else
-								winwarn(app, ebuf);
-						}
-					}
-					break;
-				}
-			}
-
-			app->nowaitcursor = 0;
-			processed = 1;
-		}
-	}
+	p = fz_transform_point(p, ctm);
 
 	for (link = app->page_links; link; link = link->next)
 	{
@@ -1758,22 +1512,6 @@ void pdfapp_onmouse(pdfapp_t *app, int x, int y, int btn, int modifiers, int sta
 			return;
 		}
 	}
-	else
-	{
-		fz_annot *annot;
-		for (annot = fz_first_annot(app->ctx, app->page); annot; annot = fz_next_annot(app->ctx, annot))
-		{
-			fz_rect rect;
-			fz_bound_annot(app->ctx, annot, &rect);
-			if (x >= rect.x0 && x < rect.x1)
-				if (y >= rect.y0 && y < rect.y1)
-					break;
-		}
-		if (annot)
-			wincursor(app, CARET);
-		else
-			wincursor(app, ARROW);
-	}
 
 	if (state == 1 && !processed)
 	{
@@ -1914,13 +1652,11 @@ void pdfapp_oncopy(pdfapp_t *app, unsigned short *ucsbuf, int ucslen)
 	fz_stext_block *block;
 	fz_stext_line *line;
 	fz_stext_char *ch;
-
-	int x0 = app->selr.x0;
-	int x1 = app->selr.x1;
-	int y0 = app->selr.y0;
-	int y1 = app->selr.y1;
+	fz_rect sel;
 
 	pdfapp_viewctm(&ctm, app);
+	ctm = fz_invert_matrix(ctm);
+	sel = fz_transform_rect(app->selr, ctm);
 
 	p = 0;
 	need_newline = 0;
@@ -1935,15 +1671,16 @@ void pdfapp_oncopy(pdfapp_t *app, unsigned short *ucsbuf, int ucslen)
 			int saw_text = 0;
 			for (ch = line->first_char; ch; ch = ch->next)
 			{
+				fz_rect bbox = fz_rect_from_quad(ch->quad);
 				int c = ch->c;
 				if (c < 32)
 					c = 0xFFFD;
-				if (ch->bbox.x1 >= x0 && ch->bbox.x0 <= x1 && ch->bbox.y1 >= y0 && ch->bbox.y0 <= y1)
+				if (bbox.x1 >= sel.x0 && bbox.x0 <= sel.x1 && bbox.y1 >= sel.y0 && bbox.y0 <= sel.y1)
 				{
 					saw_text = 1;
 					if (need_newline)
 					{
-#if defined(_WIN32) || defined(_WIN64)
+#ifdef _WIN32
 						if (p < ucslen - 1)
 							ucsbuf[p++] = '\r';
 #endif
