@@ -59,7 +59,6 @@ struct pdfapp_s
 	char *docpath;
 	char *doctitle;
 	fz_outline *outline;
-	int outline_deferred;
 
 	float layout_w;
 	float layout_h;
@@ -70,13 +69,15 @@ struct pdfapp_s
 	int pagecount;
 
 	/* current view params */
-	int resolution;
+	float default_resolution;
+	float resolution;
 	int rotate;
 	fz_pixmap *image;
 	int grayscale;
 	fz_colorspace *colorspace;
 	int invert;
-	int tint, tint_r, tint_g, tint_b;
+	int tint, tint_white;
+	int useseparations;
 
 	/* presentation mode */
 	int presentation_mode;
@@ -97,7 +98,9 @@ struct pdfapp_s
 	fz_stext_page *page_text;
 	fz_link *page_links;
 	int errored;
-	int incomplete;
+
+	/* separations */
+	fz_separations *seps;
 
 	/* snapback history */
 	int hist[256];
@@ -142,15 +145,11 @@ struct pdfapp_s
 	void *userdata;
 
 	fz_context *ctx;
-#ifdef HAVE_CURL
-	fz_stream *stream;
-#endif
 };
 
 void pdfapp_init(fz_context *ctx, pdfapp_t *app);
 void pdfapp_setresolution(pdfapp_t *app, int res);
 void pdfapp_open(pdfapp_t *app, char *filename, int reload);
-void pdfapp_open_progressive(pdfapp_t *app, char *filename, int reload, int bps);
 void pdfapp_close(pdfapp_t *app);
 int pdfapp_preclose(pdfapp_t *app);
 void pdfapp_reloadfile(pdfapp_t *app);
