@@ -84,6 +84,14 @@ fz_new_path(fz_context *ctx)
 	return path;
 }
 
+/*
+	Take an additional reference to
+	a path.
+
+	No modifications should be carried out on a path
+	to which more than one reference is held, as
+	this can cause race conditions.
+*/
 fz_path *
 fz_keep_path(fz_context *ctx, const fz_path *pathc)
 {
@@ -113,6 +121,10 @@ fz_drop_path(fz_context *ctx, const fz_path *pathc)
 	}
 }
 
+/*
+	Return the number of
+	bytes required to pack a path.
+*/
 int fz_packed_path_size(const fz_path *path)
 {
 	switch (path->packed)
@@ -134,6 +146,46 @@ int fz_packed_path_size(const fz_path *path)
 	}
 }
 
+/*
+	Pack a path into the given block.
+	To minimise the size of paths, this function allows them to be
+	packed into a buffer with other information. Paths can be used
+	interchangeably regardless of how they are packed.
+
+	pack: Pointer to a block of data to pack the path into. Should
+	be aligned by the caller to the same alignment as required for
+	a fz_path pointer.
+
+	max: The number of bytes available in the block.
+	If max < sizeof(fz_path) then an exception will
+	be thrown. If max >= the value returned by
+	fz_packed_path_size, then this call will never
+	fail, except in low memory situations with large
+	paths.
+
+	path: The path to pack.
+
+	Returns the number of bytes within the block used. Callers can
+	access the packed path data by casting the value of pack on
+	entry to be a fz_path *.
+
+	Throws exceptions on failure to allocate, or if
+	max < sizeof(fz_path).
+
+	Implementation details: Paths can be 'unpacked', 'flat', or
+	'open'. Standard paths, as created are 'unpacked'. Paths that
+	will pack into less than max bytes will be packed as 'flat',
+	unless they are too large (where large indicates that they
+	exceed some private implementation defined limits, currently
+	including having more than 256 coordinates or commands).
+
+	Large paths are 'open' packed as a header into the given block,
+	plus pointers to other data blocks.
+
+	Users should not have to care about whether paths are 'open'
+	or 'flat' packed. Simply pack a path (if required), and then
+	forget about the details.
+*/
 int
 fz_pack_path(fz_context *ctx, uint8_t *pack_, int max, const fz_path *path)
 {
@@ -251,12 +303,28 @@ push_ord(fz_context *ctx, fz_path *path, float xy, int isx)
 		path->current.y = xy;
 }
 
+/*
+	Return the current point that a path has
+	reached or (0,0) if empty.
+
+	path: path to return the current point of.
+*/
 fz_point
 fz_currentpoint(fz_context *ctx, fz_path *path)
 {
 	return path->current;
 }
 
+/*
+	Append a 'moveto' command to a path.
+	This 'opens' a path.
+
+	path: The path to modify.
+
+	x, y: The coordinate to move to.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_moveto(fz_context *ctx, fz_path *path, float x, float y)
 {
@@ -280,6 +348,15 @@ fz_moveto(fz_context *ctx, fz_path *path, float x, float y)
 	path->begin = path->current;
 }
 
+/*
+	Append a 'lineto' command to an open path.
+
+	path: The path to modify.
+
+	x, y: The coordinate to line to.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_lineto(fz_context *ctx, fz_path *path, float x, float y)
 {
@@ -327,6 +404,22 @@ fz_lineto(fz_context *ctx, fz_path *path, float x, float y)
 	}
 }
 
+/*
+	Append a 'curveto' command to an open path. (For a
+	cubic bezier).
+
+	path: The path to modify.
+
+	x0, y0: The coordinates of the first control point for the
+	curve.
+
+	x1, y1: The coordinates of the second control point for the
+	curve.
+
+	x2, y2: The end coordinates for the curve.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_curveto(fz_context *ctx, fz_path *path,
 	float x1, float y1,
@@ -385,6 +478,18 @@ fz_curveto(fz_context *ctx, fz_path *path,
 	push_coord(ctx, path, x3, y3);
 }
 
+/*
+	Append a 'quadto' command to an open path. (For a
+	quadratic bezier).
+
+	path: The path to modify.
+
+	x0, y0: The control coordinates for the quadratic curve.
+
+	x1, y1: The end coordinates for the quadratic curve.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_quadto(fz_context *ctx, fz_path *path,
 	float x1, float y1,
@@ -419,6 +524,20 @@ fz_quadto(fz_context *ctx, fz_path *path,
 	push_coord(ctx, path, x2, y2);
 }
 
+/*
+	Append a 'curvetov' command to an open path. (For a
+	cubic bezier with the first control coordinate equal to
+	the start point).
+
+	path: The path to modify.
+
+	x1, y1: The coordinates of the second control point for the
+	curve.
+
+	x2, y2: The end coordinates for the curve.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_curvetov(fz_context *ctx, fz_path *path, float x2, float y2, float x3, float y3)
 {
@@ -456,6 +575,21 @@ fz_curvetov(fz_context *ctx, fz_path *path, float x2, float y2, float x3, float
 	push_coord(ctx, path, x3, y3);
 }
 
+/*
+	Append a 'curvetoy' command to an open path. (For a
+	cubic bezier with the second control coordinate equal to
+	the end point).
+
+	path: The path to modify.
+
+	x0, y0: The coordinates of the first control point for the
+	curve.
+
+	x2, y2: The end coordinates for the curve (and the second
+	control coordinate).
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_curvetoy(fz_context *ctx, fz_path *path, float x1, float y1, float x3, float y3)
 {
@@ -488,6 +622,14 @@ fz_curvetoy(fz_context *ctx, fz_path *path, float x1, float y1, float x3, float
 	push_coord(ctx, path, x3, y3);
 }
 
+/*
+	Close the current subpath.
+
+	path: The path to modify.
+
+	Throws exceptions on failure to allocate, and illegal
+	path closes (i.e. closing a non open path).
+*/
 void
 fz_closepath(fz_context *ctx, fz_path *path)
 {
@@ -556,6 +698,24 @@ fz_closepath(fz_context *ctx, fz_path *path)
 	path->current = path->begin;
 }
 
+/*
+	Append a 'rectto' command to an open path.
+
+	The rectangle is equivalent to:
+		moveto x0 y0
+		lineto x1 y0
+		lineto x1 y1
+		lineto x0 y1
+		closepath
+
+	path: The path to modify.
+
+	x0, y0: First corner of the rectangle.
+
+	x1, y1: Second corner of the rectangle.
+
+	Throws exceptions on failure to allocate.
+*/
 void
 fz_rectto(fz_context *ctx, fz_path *path, float x1, float y1, float x2, float y2)
 {
@@ -576,15 +736,32 @@ fz_rectto(fz_context *ctx, fz_path *path, float x1, float y1, float x2, float y2
 	path->current = path->begin;
 }
 
-static inline fz_rect *bound_expand(fz_rect *r, const fz_point *p)
+static inline void bound_expand(fz_rect *r, fz_point p)
 {
-	if (p->x < r->x0) r->x0 = p->x;
-	if (p->y < r->y0) r->y0 = p->y;
-	if (p->x > r->x1) r->x1 = p->x;
-	if (p->y > r->y1) r->y1 = p->y;
-	return r;
+	if (p.x < r->x0) r->x0 = p.x;
+	if (p.y < r->y0) r->y0 = p.y;
+	if (p.x > r->x1) r->x1 = p.x;
+	if (p.y > r->y1) r->y1 = p.y;
 }
 
+/*
+	Walk the segments of a path, calling the
+	appropriate callback function from a given set for each
+	segment of the path.
+
+	path: The path to walk.
+
+	walker: The set of callback functions to use. The first
+	4 callback pointers in the set must be non-NULL. The
+	subsequent ones can either be supplied, or can be left
+	as NULL, in which case the top 4 functions will be
+	called as appropriate to simulate them.
+
+	arg: An opaque argument passed in to each callback.
+
+	Exceptions will only be thrown if the underlying callback
+	functions throw them.
+*/
 void fz_walk_path(fz_context *ctx, const fz_path *path, const fz_path_walker *proc, void *arg)
 {
 	int i, k, cmd_len;
@@ -834,7 +1011,7 @@ void fz_walk_path(fz_context *ctx, const fz_path *path, const fz_path_walker *pr
 
 typedef struct
 {
-	const fz_matrix *ctm;
+	fz_matrix ctm;
 	fz_rect rect;
 	fz_point move;
 	int trailing_move;
@@ -845,10 +1022,7 @@ static void
 bound_moveto(fz_context *ctx, void *arg_, float x, float y)
 {
 	bound_path_arg *arg = (bound_path_arg *)arg_;
-
-	arg->move.x = x;
-	arg->move.y = y;
-	fz_transform_point(&arg->move, arg->ctm);
+	arg->move = fz_transform_point_xy(x, y, arg->ctm);
 	arg->trailing_move = 1;
 }
 
@@ -856,11 +1030,7 @@ static void
 bound_lineto(fz_context *ctx, void *arg_, float x, float y)
 {
 	bound_path_arg *arg = (bound_path_arg *)arg_;
-	fz_point p;
-
-	p.x = x;
-	p.y = y;
-	fz_transform_point(&p, arg->ctm);
+	fz_point p = fz_transform_point_xy(x, y, arg->ctm);
 	if (arg->first)
 	{
 		arg->rect.x0 = arg->rect.x1 = p.x;
@@ -868,12 +1038,11 @@ bound_lineto(fz_context *ctx, void *arg_, float x, float y)
 		arg->first = 0;
 	}
 	else
-		bound_expand(&arg->rect, &p);
-
+		bound_expand(&arg->rect, p);
 	if (arg->trailing_move)
 	{
 		arg->trailing_move = 0;
-		bound_expand(&arg->rect, &arg->move);
+		bound_expand(&arg->rect, arg->move);
 	}
 }
 
@@ -881,11 +1050,7 @@ static void
 bound_curveto(fz_context *ctx, void *arg_, float x1, float y1, float x2, float y2, float x3, float y3)
 {
 	bound_path_arg *arg = (bound_path_arg *)arg_;
-	fz_point p;
-
-	p.x = x1;
-	p.y = y1;
-	fz_transform_point(&p, arg->ctm);
+	fz_point p = fz_transform_point_xy(x1, y1, arg->ctm);
 	if (arg->first)
 	{
 		arg->rect.x0 = arg->rect.x1 = p.x;
@@ -893,17 +1058,13 @@ bound_curveto(fz_context *ctx, void *arg_, float x1, float y1, float x2, float y
 		arg->first = 0;
 	}
 	else
-		bound_expand(&arg->rect, &p);
-	p.x = x2;
-	p.y = y2;
-	bound_expand(&arg->rect, fz_transform_point(&p, arg->ctm));
-	p.x = x3;
-	p.y = y3;
-	bound_expand(&arg->rect, fz_transform_point(&p, arg->ctm));
+		bound_expand(&arg->rect, p);
+	bound_expand(&arg->rect, fz_transform_point_xy(x2, y2, arg->ctm));
+	bound_expand(&arg->rect, fz_transform_point_xy(x3, y3, arg->ctm));
 	if (arg->trailing_move)
 	{
 		arg->trailing_move = 0;
-		bound_expand(&arg->rect, &arg->move);
+		bound_expand(&arg->rect, arg->move);
 	}
 }
 
@@ -915,8 +1076,24 @@ static const fz_path_walker bound_path_walker =
 	NULL
 };
 
-fz_rect *
-fz_bound_path(fz_context *ctx, const fz_path *path, const fz_stroke_state *stroke, const fz_matrix *ctm, fz_rect *r)
+/*
+	Return a bounding rectangle for a path.
+
+	path: The path to bound.
+
+	stroke: If NULL, the bounding rectangle given is for
+	the filled path. If non-NULL the bounding rectangle
+	given is for the path stroked with the given attributes.
+
+	ctm: The matrix to apply to the path during stroking.
+
+	r: Pointer to a fz_rect which will be used to hold
+	the result.
+
+	Returns r, updated to contain the bounding rectangle.
+*/
+fz_rect
+fz_bound_path(fz_context *ctx, const fz_path *path, const fz_stroke_state *stroke, fz_matrix ctm)
 {
 	bound_path_arg arg;
 
@@ -929,15 +1106,14 @@ fz_bound_path(fz_context *ctx, const fz_path *path, const fz_stroke_state *strok
 
 	if (!arg.first && stroke)
 	{
-		fz_adjust_rect_for_stroke(ctx, &arg.rect, stroke, ctm);
+		arg.rect = fz_adjust_rect_for_stroke(ctx, arg.rect, stroke, ctm);
 	}
 
-	*r = arg.rect;
-	return r;
+	return arg.rect;
 }
 
-fz_rect *
-fz_adjust_rect_for_stroke(fz_context *ctx, fz_rect *r, const fz_stroke_state *stroke, const fz_matrix *ctm)
+fz_rect
+fz_adjust_rect_for_stroke(fz_context *ctx, fz_rect r, const fz_stroke_state *stroke, fz_matrix ctm)
 {
 	float expand;
 
@@ -951,15 +1127,26 @@ fz_adjust_rect_for_stroke(fz_context *ctx, fz_rect *r, const fz_stroke_state *st
 	if ((stroke->linejoin == FZ_LINEJOIN_MITER || stroke->linejoin == FZ_LINEJOIN_MITER_XPS) && stroke->miterlimit > 1)
 		expand *= stroke->miterlimit;
 
-	r->x0 -= expand;
-	r->y0 -= expand;
-	r->x1 += expand;
-	r->y1 += expand;
+	r.x0 -= expand;
+	r.y0 -= expand;
+	r.x1 += expand;
+	r.y1 += expand;
 	return r;
 }
 
+/*
+	Transform a path by a given
+	matrix.
+
+	path: The path to modify (must not be a packed path).
+
+	transform: The transform to apply.
+
+	Throws exceptions if the path is packed, or on failure
+	to allocate.
+*/
 void
-fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
+fz_transform_path(fz_context *ctx, fz_path *path, fz_matrix ctm)
 {
 	int i, k, n;
 	fz_point p, p1, p2, p3, q, s;
@@ -967,7 +1154,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 	if (path->packed)
 		fz_throw(ctx, FZ_ERROR_GENERIC, "Cannot transform a packed path");
 
-	if (ctm->b == 0 && ctm->c == 0)
+	if (ctm.b == 0 && ctm.c == 0)
 	{
 		/* Simple, in place transform */
 		i = 0;
@@ -1008,16 +1195,14 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			case FZ_HORIZTO:
 			case FZ_HORIZTOCLOSE:
 				q.x = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.x;
 				n = 0;
 				break;
 			case FZ_VERTTO:
 			case FZ_VERTTOCLOSE:
 				q.y = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.y;
 				n = 0;
 				break;
@@ -1028,8 +1213,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			{
 				q.x = path->coords[k];
 				q.y = path->coords[k+1];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.x;
 				path->coords[k++] = p.y;
 				n--;
@@ -1055,7 +1239,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			i++;
 		}
 	}
-	else if (ctm->a == 0 && ctm->d == 0)
+	else if (ctm.a == 0 && ctm.d == 0)
 	{
 		/* In place transform with command rewriting */
 		i = 0;
@@ -1095,32 +1279,28 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 				break;
 			case FZ_HORIZTO:
 				q.x = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.y;
 				path->cmds[i] = FZ_VERTTO;
 				n = 0;
 				break;
 			case FZ_HORIZTOCLOSE:
 				q.x = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.y;
 				path->cmds[i] = FZ_VERTTOCLOSE;
 				n = 0;
 				break;
 			case FZ_VERTTO:
 				q.y = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.x;
 				path->cmds[i] = FZ_HORIZTO;
 				n = 0;
 				break;
 			case FZ_VERTTOCLOSE:
 				q.y = path->coords[k];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.x;
 				path->cmds[i] = FZ_HORIZTOCLOSE;
 				n = 0;
@@ -1130,10 +1310,9 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			}
 			while (n > 0)
 			{
-				p.x = path->coords[k];
-				p.y = path->coords[k+1];
-				q = p;
-				fz_transform_point(&p, ctm);
+				q.x = path->coords[k];
+				q.y = path->coords[k+1];
+				p = fz_transform_point(q, ctm);
 				path->coords[k++] = p.x;
 				path->coords[k++] = p.y;
 				n--;
@@ -1240,10 +1419,10 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 				p3.x = p.x;
 				p3.y = p2.y;
 				s = p;
-				fz_transform_point(&p, ctm);
-				fz_transform_point(&p1, ctm);
-				fz_transform_point(&p2, ctm);
-				fz_transform_point(&p3, ctm);
+				p = fz_transform_point(p, ctm);
+				p1 = fz_transform_point(p1, ctm);
+				p2 = fz_transform_point(p2, ctm);
+				p3 = fz_transform_point(p3, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				path->coords[coord_write++] = p1.x;
@@ -1260,8 +1439,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 				break;
 			case FZ_HORIZTO:
 				q.x = path->coords[coord_read++];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				path->cmds[cmd_write-1] = FZ_LINETO;
@@ -1270,7 +1448,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			case FZ_HORIZTOCLOSE:
 				p.x = path->coords[coord_read++];
 				p.y = q.y;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(p, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				path->cmds[cmd_write-1] = FZ_LINETOCLOSE;
@@ -1279,8 +1457,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 				break;
 			case FZ_VERTTO:
 				q.y = path->coords[coord_read++];
-				p = q;
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(q, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				path->cmds[cmd_write-1] = FZ_LINETO;
@@ -1289,7 +1466,7 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			case FZ_VERTTOCLOSE:
 				p.x = q.x;
 				p.y = path->coords[coord_read++];
-				fz_transform_point(&p, ctm);
+				p = fz_transform_point(p, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				path->cmds[cmd_write-1] = FZ_LINETOCLOSE;
@@ -1301,10 +1478,9 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 			}
 			while (n > 0)
 			{
-				p.x = path->coords[coord_read++];
-				p.y = path->coords[coord_read++];
-				q = p;
-				fz_transform_point(&p, ctm);
+				q.x = path->coords[coord_read++];
+				q.y = path->coords[coord_read++];
+				p = fz_transform_point(q, ctm);
 				path->coords[coord_write++] = p.x;
 				path->coords[coord_write++] = p.y;
 				n--;
@@ -1331,6 +1507,16 @@ fz_transform_path(fz_context *ctx, fz_path *path, const fz_matrix *ctm)
 	}
 }
 
+/*
+	Minimise the internal storage
+	used by a path.
+
+	As paths are constructed, the internal buffers
+	grow. To avoid repeated reallocations they
+	grow with some spare space. Once a path has
+	been fully constructed, this call allows the
+	excess space to be trimmed.
+*/
 void fz_trim_path(fz_context *ctx, fz_path *path)
 {
 	if (path->packed)
@@ -1355,6 +1541,14 @@ const fz_stroke_state fz_default_stroke_state = {
 	0, 0, { 0 }
 };
 
+/*
+	Take an additional reference to
+	a stroke state structure.
+
+	No modifications should be carried out on a stroke
+	state to which more than one reference is held, as
+	this can cause race conditions.
+*/
 fz_stroke_state *
 fz_keep_stroke_state(fz_context *ctx, const fz_stroke_state *strokec)
 {
@@ -1370,6 +1564,11 @@ fz_keep_stroke_state(fz_context *ctx, const fz_stroke_state *strokec)
 	return fz_keep_imp(ctx, stroke, &stroke->refs);
 }
 
+/*
+	Drop a reference to a stroke
+	state structure, destroying the structure if it is
+	the last reference.
+*/
 void
 fz_drop_stroke_state(fz_context *ctx, const fz_stroke_state *strokec)
 {
@@ -1379,6 +1578,15 @@ fz_drop_stroke_state(fz_context *ctx, const fz_stroke_state *strokec)
 		fz_free(ctx, stroke);
 }
 
+/*
+	Create a new (empty)
+	stroke state structure, with room for dash data of the
+	given length, and return a reference to it.
+
+	len: The number of dash elements to allow room for.
+
+	Throws exception on failure to allocate.
+*/
 fz_stroke_state *
 fz_new_stroke_state_with_dash_len(fz_context *ctx, int len)
 {
@@ -1403,12 +1611,27 @@ fz_new_stroke_state_with_dash_len(fz_context *ctx, int len)
 	return state;
 }
 
+/*
+	Create a new (empty) stroke state
+	structure (with no dash data) and return a reference to it.
+
+	Throws exception on failure to allocate.
+*/
 fz_stroke_state *
 fz_new_stroke_state(fz_context *ctx)
 {
 	return fz_new_stroke_state_with_dash_len(ctx, 0);
 }
 
+/*
+	Create an identical stroke_state
+	structure and return a reference to it.
+
+	stroke: The stroke state reference to clone.
+
+	Exceptions may be thrown in the event of a failure to
+	allocate.
+*/
 fz_stroke_state *
 fz_clone_stroke_state(fz_context *ctx, fz_stroke_state *stroke)
 {
@@ -1420,6 +1643,21 @@ fz_clone_stroke_state(fz_context *ctx, fz_stroke_state *stroke)
 	return clone;
 }
 
+/*
+	Given a reference to a
+	(possibly) shared stroke_state structure, return a reference
+	to a stroke_state structure (with room for a given amount of
+	dash data) that is guaranteed to be unshared (i.e. one that
+	can safely be modified).
+
+	shared: The reference to a (possibly) shared structure
+	to unshare. Ownership of this reference is passed in
+	to this function, even in the case of exceptions being
+	thrown.
+
+	Exceptions may be thrown in the event of failure to
+	allocate if required.
+*/
 fz_stroke_state *
 fz_unshare_stroke_state_with_dash_len(fz_context *ctx, fz_stroke_state *shared, int len)
 {
@@ -1450,6 +1688,21 @@ fz_unshare_stroke_state_with_dash_len(fz_context *ctx, fz_stroke_state *shared,
 	return unshared;
 }
 
+/*
+	Given a reference to a
+	(possibly) shared stroke_state structure, return
+	a reference to an equivalent stroke_state structure
+	that is guaranteed to be unshared (i.e. one that can
+	safely be modified).
+
+	shared: The reference to a (possibly) shared structure
+	to unshare. Ownership of this reference is passed in
+	to this function, even in the case of exceptions being
+	thrown.
+
+	Exceptions may be thrown in the event of failure to
+	allocate if required.
+*/
 fz_stroke_state *
 fz_unshare_stroke_state(fz_context *ctx, fz_stroke_state *shared)
 {
@@ -1469,6 +1722,18 @@ clone_block(fz_context *ctx, void *block, size_t len)
 	return target;
 }
 
+/*
+	Clone the data for a path.
+
+	This is used in preference to fz_keep_path when a whole
+	new copy of a path is required, rather than just a shared
+	pointer. This probably indicates that the path is about to
+	be modified.
+
+	path: path to clone.
+
+	Throws exceptions on failure to allocate.
+*/
 fz_path *
 fz_clone_path(fz_context *ctx, fz_path *path)
 {
@@ -1573,6 +1838,7 @@ fz_clone_path(fz_context *ctx, fz_path *path)
 	}
 	fz_catch(ctx)
 	{
+		fz_free(ctx, new_path->coords);
 		fz_free(ctx, new_path->cmds);
 		fz_free(ctx, new_path);
 		fz_rethrow(ctx);
