@@ -9,7 +9,6 @@ enum
 	FZ_SEPARATION_DISABLED_RENDER = 3
 };
 
-
 struct fz_separations_s
 {
 	int refs;
@@ -23,6 +22,7 @@ struct fz_separations_s
 	char *name[FZ_MAX_SEPARATIONS];
 };
 
+/* Create a new separations structure (initially empty) */
 fz_separations *fz_new_separations(fz_context *ctx, int controllable)
 {
 	fz_separations *sep;
@@ -53,6 +53,7 @@ void fz_drop_separations(fz_context *ctx, fz_separations *sep)
 	}
 }
 
+/* Add a separation (null terminated name, colorspace) */
 void fz_add_separation(fz_context *ctx, fz_separations *sep, const char *name, fz_colorspace *cs, int colorant)
 {
 	int n;
@@ -71,6 +72,7 @@ void fz_add_separation(fz_context *ctx, fz_separations *sep, const char *name, f
 	sep->num_separations++;
 }
 
+/* Add a separation with equivalents (null terminated name, colorspace) (old, deprecated) */
 void fz_add_separation_equivalents(fz_context *ctx, fz_separations *sep, uint32_t rgba, uint32_t cmyk, const char *name)
 {
 	int n;
@@ -89,11 +91,7 @@ void fz_add_separation_equivalents(fz_context *ctx, fz_separations *sep, uint32_
 	sep->num_separations++;
 }
 
-int fz_separations_controllable(fz_context *ctx, const fz_separations *sep)
-{
-	return (!sep || sep->controllable);
-}
-
+/* Control the rendering of a given separation */
 void fz_set_separation_behavior(fz_context *ctx, fz_separations *sep, int separation, fz_separation_behavior beh)
 {
 	int shift;
@@ -138,6 +136,7 @@ fz_separation_behavior fz_separation_current_behavior_internal(fz_context *ctx,
 	return sep_state(sep, separation);
 }
 
+/* Test for the current behavior of a separation */
 fz_separation_behavior fz_separation_current_behavior(fz_context *ctx, const fz_separations *sep, int separation)
 {
 	int beh = fz_separation_current_behavior_internal(ctx, sep, separation);
@@ -147,20 +146,6 @@ fz_separation_behavior fz_separation_current_behavior(fz_context *ctx, const fz_
 	return beh;
 }
 
-int fz_separations_all_composite(fz_context *ctx, const fz_separations *sep)
-{
-	int i;
-
-	if (!sep)
-		return 1;
-
-	for (i = 0; i < (FZ_MAX_SEPARATIONS + 31) / 32; i++)
-		if (sep_state(sep, i) != FZ_SEPARATION_COMPOSITE)
-			return 0;
-
-	return 1;
-}
-
 const char *fz_separation_name(fz_context *ctx, const fz_separations *sep, int separation)
 {
 	if (!sep || separation < 0 || separation >= sep->num_separations)
@@ -176,6 +161,7 @@ int fz_count_separations(fz_context *ctx, const fz_separations *sep)
 	return sep->num_separations;
 }
 
+/* Return the number of active separations. */
 int fz_count_active_separations(fz_context *ctx, const fz_separations *sep)
 {
 	int i, n, c;
@@ -190,6 +176,11 @@ int fz_count_active_separations(fz_context *ctx, const fz_separations *sep)
 	return c;
 }
 
+/* Return a separations object with all the spots in the input
+ * separations object that are set to composite, reset to be
+ * enabled. If there ARE no spots in the object, this returns
+ * NULL. If the object already has all its spots enabled, then
+ * just returns another handle on the same object. */
 fz_separations *fz_clone_separations_for_overprint(fz_context *ctx, fz_separations *sep)
 {
 	int i, j, n, c;
@@ -217,11 +208,11 @@ fz_separations *fz_clone_separations_for_overprint(fz_context *ctx, fz_separatio
 	/* We need to clone us a separation structure, with all
 	 * the composite separations marked as enabled. */
 	clone = fz_malloc_struct(ctx, fz_separations);
+	clone->refs = 1;
+	clone->controllable = 0;
 
 	fz_try(ctx)
 	{
-		clone->refs = 1;
-		clone->controllable = 0;
 		for (i = 0; i < n; i++)
 		{
 			fz_separation_behavior beh = sep_state(sep, i);
@@ -245,11 +236,15 @@ fz_separations *fz_clone_separations_for_overprint(fz_context *ctx, fz_separatio
 	return clone;
 }
 
+/*
+	Convert between
+	different separation results.
+*/
 fz_pixmap *
 fz_clone_pixmap_area_with_different_seps(fz_context *ctx, fz_pixmap *src, const fz_irect *bbox, fz_colorspace *dcs, fz_separations *dseps, const fz_color_params *color_params, fz_default_colorspaces *default_cs)
 {
 	fz_irect local_bbox;
-	fz_pixmap *dst;
+	fz_pixmap *dst, *pix;
 
 	if (bbox == NULL)
 	{
@@ -260,13 +255,21 @@ fz_clone_pixmap_area_with_different_seps(fz_context *ctx, fz_pixmap *src, const
 		bbox = &local_bbox;
 	}
 
-	dst = fz_new_pixmap_with_bbox(ctx, dcs, bbox, dseps, src->alpha);
+	dst = fz_new_pixmap_with_bbox(ctx, dcs, *bbox, dseps, src->alpha);
 	if (src->flags & FZ_PIXMAP_FLAG_INTERPOLATE)
 		dst->flags |= FZ_PIXMAP_FLAG_INTERPOLATE;
 	else
 		dst->flags &= ~FZ_PIXMAP_FLAG_INTERPOLATE;
 
-	return fz_copy_pixmap_area_converting_seps(ctx, dst, src, color_params, NULL, default_cs);
+	fz_try(ctx)
+		pix = fz_copy_pixmap_area_converting_seps(ctx, dst, src, color_params, NULL, default_cs);
+	fz_catch(ctx)
+	{
+		fz_drop_pixmap(ctx, dst);
+		fz_rethrow(ctx);
+	}
+
+	return pix;
 }
 
 /*
@@ -401,8 +404,10 @@ fz_copy_pixmap_area_converting_seps(fz_context *ctx, fz_pixmap *dst, fz_pixmap *
 
 				if (name)
 				{
-					if (!strcmp(name, "None"))
+					if (!strcmp(name, "None")) {
+						mapped[i] = 0;
 						continue;
+					}
 					if (!strcmp(name, "All"))
 					{
 						int n1 = dn - da;
@@ -695,8 +700,7 @@ fz_copy_pixmap_area_converting_seps(fz_context *ctx, fz_pixmap *dst, fz_pixmap *
 	else
 	{
 		/* Use a standard pixmap converter to convert the process + alpha. */
-		fz_pixmap_converter *pc = fz_lookup_pixmap_converter(ctx, dst->colorspace, src->colorspace);
-		pc(ctx, dst, src, proof_cs, default_cs, NULL, 0);
+		fz_convert_pixmap_samples(ctx, dst, src, proof_cs, default_cs, NULL, 0);
 
 		/* And handle the spots ourselves. First make a map of what spots go where. */
 		/* We want to set it up so that:
@@ -959,7 +963,9 @@ fz_copy_pixmap_area_converting_seps(fz_context *ctx, fz_pixmap *dst, fz_pixmap *
 	return dst;
 }
 
-void fz_convert_separation_colors(fz_context *ctx, const fz_color_params *color_params, const fz_colorspace *dst_cs, const fz_separations *dst_seps, float *dst_color, const fz_colorspace *src_cs, const float *src_color)
+/* Convert a color given in terms of one colorspace,
+ * to a color in terms of another colorspace/separations. */
+void fz_convert_separation_colors(fz_context *ctx, const fz_color_params *color_params, fz_colorspace *dst_cs, const fz_separations *dst_seps, float *dst_color, fz_colorspace *src_cs, const float *src_color)
 {
 	int i, j, n, dc, ds, dn, pred;
 	float remainders[FZ_MAX_COLORS];
@@ -1051,7 +1057,8 @@ found_process:
 	}
 }
 
-void fz_separation_equivalent(fz_context *ctx, const fz_separations *seps, int i, const fz_color_params *color_params, const fz_colorspace *dst_cs, const fz_colorspace *prf, float *convert)
+/* Get the equivalent separation color in a given colorspace. */
+void fz_separation_equivalent(fz_context *ctx, const fz_separations *seps, int i, const fz_color_params *color_params, fz_colorspace *dst_cs, fz_colorspace *prf, float *convert)
 {
 	float colors[FZ_MAX_COLORS];
 
