@@ -6,15 +6,15 @@ fz_display_list *
 fz_new_display_list_from_page(fz_context *ctx, fz_page *page)
 {
 	fz_display_list *list;
-	fz_rect bounds;
 	fz_device *dev = NULL;
 
-	list = fz_new_display_list(ctx, fz_bound_page(ctx, page, &bounds));
+	fz_var(dev);
 
+	list = fz_new_display_list(ctx, fz_bound_page(ctx, page));
 	fz_try(ctx)
 	{
 		dev = fz_new_list_device(ctx, list);
-		fz_run_page(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -50,15 +50,15 @@ fz_display_list *
 fz_new_display_list_from_page_contents(fz_context *ctx, fz_page *page)
 {
 	fz_display_list *list;
-	fz_rect bounds;
 	fz_device *dev = NULL;
 
-	list = fz_new_display_list(ctx, fz_bound_page(ctx, page, &bounds));
+	fz_var(dev);
 
+	list = fz_new_display_list(ctx, fz_bound_page(ctx, page));
 	fz_try(ctx)
 	{
 		dev = fz_new_list_device(ctx, list);
-		fz_run_page_contents(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -74,47 +74,30 @@ fz_new_display_list_from_page_contents(fz_context *ctx, fz_page *page)
 	return list;
 }
 
-fz_display_list *
-fz_new_display_list_from_annot(fz_context *ctx, fz_annot *annot)
+/*
+	Render the page to a pixmap using the transform and colorspace.
+*/
+fz_pixmap *
+fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
-	fz_display_list *list;
-	fz_rect bounds;
-	fz_device *dev = NULL;
-
-	list = fz_new_display_list(ctx, fz_bound_annot(ctx, annot, &bounds));
-
-	fz_try(ctx)
-	{
-		dev = fz_new_list_device(ctx, list);
-		fz_run_annot(ctx, annot, dev, &fz_identity, NULL);
-		fz_close_device(ctx, dev);
-	}
-	fz_always(ctx)
-	{
-		fz_drop_device(ctx, dev);
-	}
-	fz_catch(ctx)
-	{
-		fz_drop_display_list(ctx, list);
-		fz_rethrow(ctx);
-	}
-
-	return list;
+	return fz_new_pixmap_from_display_list_with_separations(ctx, list, ctm, cs, NULL, alpha);
 }
 
 fz_pixmap *
-fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_display_list_with_separations(fz_context *ctx, fz_display_list *list, fz_matrix ctm, fz_colorspace *cs, fz_separations *seps, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_display_list(ctx, list, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
+	rect = fz_bound_display_list(ctx, list);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, seps, alpha);
 	if (alpha)
 		fz_clear_pixmap(ctx, pix);
 	else
@@ -123,7 +106,55 @@ fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz
 	fz_try(ctx)
 	{
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_display_list(ctx, list, dev, &fz_identity, NULL, NULL);
+		fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, NULL);
+		fz_close_device(ctx, dev);
+	}
+	fz_always(ctx)
+	{
+		fz_drop_device(ctx, dev);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_pixmap(ctx, pix);
+		fz_rethrow(ctx);
+	}
+
+	return pix;
+}
+
+/*
+	Render the page contents without annotations.
+*/
+fz_pixmap *
+fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, int alpha)
+{
+	return fz_new_pixmap_from_page_contents_with_separations(ctx, page, ctm, cs, NULL, alpha);
+}
+
+fz_pixmap *
+fz_new_pixmap_from_page_contents_with_separations(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, fz_separations *seps, int alpha)
+{
+	fz_rect rect;
+	fz_irect bbox;
+	fz_pixmap *pix;
+	fz_device *dev = NULL;
+
+	fz_var(dev);
+
+	rect = fz_bound_page(ctx, page);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, seps, alpha);
+	if (alpha)
+		fz_clear_pixmap(ctx, pix);
+	else
+		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
+
+	fz_try(ctx)
+	{
+		dev = fz_new_draw_device(ctx, ctm, pix);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -140,27 +171,36 @@ fz_new_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, const fz
 }
 
 fz_pixmap *
-fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, int alpha)
+{
+	return fz_new_pixmap_from_page_with_separations(ctx, page, ctm, cs, NULL, alpha);
+}
+
+fz_pixmap *
+fz_new_pixmap_from_page_with_separations(fz_context *ctx, fz_page *page, fz_matrix ctm, fz_colorspace *cs, fz_separations *seps, int alpha)
 {
 	fz_rect rect;
-	fz_irect irect;
+	fz_irect bbox;
 	fz_pixmap *pix;
 	fz_device *dev = NULL;
 
-	fz_bound_page(ctx, page, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
+	fz_var(dev);
 
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
-	if (alpha)
-		fz_clear_pixmap(ctx, pix);
-	else
-		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
+	rect = fz_bound_page(ctx, page);
+	rect = fz_transform_rect(rect, ctm);
+	bbox = fz_round_rect(rect);
+
+	pix = fz_new_pixmap_with_bbox(ctx, cs, bbox, seps, alpha);
 
 	fz_try(ctx)
 	{
+		if (alpha)
+			fz_clear_pixmap(ctx, pix);
+		else
+			fz_clear_pixmap_with_value(ctx, pix, 0xFF);
+
 		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_page_contents(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -177,88 +217,20 @@ fz_new_pixmap_from_page_contents(fz_context *ctx, fz_page *page, const fz_matrix
 }
 
 fz_pixmap *
-fz_new_pixmap_from_annot(fz_context *ctx, fz_annot *annot, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page_number(fz_context *ctx, fz_document *doc, int number, fz_matrix ctm, fz_colorspace *cs, int alpha)
 {
-	fz_rect rect;
-	fz_irect irect;
-	fz_pixmap *pix;
-	fz_device *dev = NULL;
-
-	fz_bound_annot(ctx, annot, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
-
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
-	if (alpha)
-		fz_clear_pixmap(ctx, pix);
-	else
-		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
-
-	fz_try(ctx)
-	{
-		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_annot(ctx, annot, dev, &fz_identity, NULL);
-		fz_close_device(ctx, dev);
-	}
-	fz_always(ctx)
-	{
-		fz_drop_device(ctx, dev);
-	}
-	fz_catch(ctx)
-	{
-		fz_drop_pixmap(ctx, pix);
-		fz_rethrow(ctx);
-	}
-
-	return pix;
+	return fz_new_pixmap_from_page_number_with_separations(ctx, doc, number, ctm, cs, NULL, alpha);
 }
 
 fz_pixmap *
-fz_new_pixmap_from_page(fz_context *ctx, fz_page *page, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
-{
-	fz_rect rect;
-	fz_irect irect;
-	fz_pixmap *pix;
-	fz_device *dev = NULL;
-
-	fz_bound_page(ctx, page, &rect);
-	fz_transform_rect(&rect, ctm);
-	fz_round_rect(&irect, &rect);
-
-	pix = fz_new_pixmap_with_bbox(ctx, cs, &irect, 0, alpha);
-	if (alpha)
-		fz_clear_pixmap(ctx, pix);
-	else
-		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
-
-	fz_try(ctx)
-	{
-		dev = fz_new_draw_device(ctx, ctm, pix);
-		fz_run_page(ctx, page, dev, &fz_identity, NULL);
-		fz_close_device(ctx, dev);
-	}
-	fz_always(ctx)
-	{
-		fz_drop_device(ctx, dev);
-	}
-	fz_catch(ctx)
-	{
-		fz_drop_pixmap(ctx, pix);
-		fz_rethrow(ctx);
-	}
-
-	return pix;
-}
-
-fz_pixmap *
-fz_new_pixmap_from_page_number(fz_context *ctx, fz_document *doc, int number, const fz_matrix *ctm, fz_colorspace *cs, int alpha)
+fz_new_pixmap_from_page_number_with_separations(fz_context *ctx, fz_document *doc, int number, fz_matrix ctm, fz_colorspace *cs, fz_separations *seps, int alpha)
 {
 	fz_page *page;
 	fz_pixmap *pix = NULL;
 
 	page = fz_load_page(ctx, doc, number);
 	fz_try(ctx)
-		pix = fz_new_pixmap_from_page(ctx, page, ctm, cs, alpha);
+		pix = fz_new_pixmap_from_page_with_separations(ctx, page, ctm, cs, seps, alpha);
 	fz_always(ctx)
 		fz_drop_page(ctx, page);
 	fz_catch(ctx)
@@ -271,16 +243,17 @@ fz_new_stext_page_from_display_list(fz_context *ctx, fz_display_list *list, cons
 {
 	fz_stext_page *text;
 	fz_device *dev = NULL;
-	fz_rect mediabox;
+
+	fz_var(dev);
 
 	if (list == NULL)
 		return NULL;
 
-	text = fz_new_stext_page(ctx, fz_bound_display_list(ctx, list, &mediabox));
+	text = fz_new_stext_page(ctx, fz_bound_display_list(ctx, list));
 	fz_try(ctx)
 	{
 		dev = fz_new_stext_device(ctx, text, options);
-		fz_run_display_list(ctx, list, dev, &fz_identity, NULL, NULL);
+		fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -301,16 +274,17 @@ fz_new_stext_page_from_page(fz_context *ctx, fz_page *page, const fz_stext_optio
 {
 	fz_stext_page *text;
 	fz_device *dev = NULL;
-	fz_rect mediabox;
+
+	fz_var(dev);
 
 	if (page == NULL)
 		return NULL;
 
-	text = fz_new_stext_page(ctx, fz_bound_page(ctx, page, &mediabox));
+	text = fz_new_stext_page(ctx, fz_bound_page(ctx, page));
 	fz_try(ctx)
 	{
 		dev = fz_new_stext_device(ctx, text, options);
-		fz_run_page(ctx, page, dev, &fz_identity, NULL);
+		fz_run_page_contents(ctx, page, dev, fz_identity, NULL);
 		fz_close_device(ctx, dev);
 	}
 	fz_always(ctx)
@@ -343,7 +317,7 @@ fz_new_stext_page_from_page_number(fz_context *ctx, fz_document *doc, int number
 }
 
 int
-fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_stext_page *text;
 	int count = 0;
@@ -358,8 +332,13 @@ fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needl
 	return count;
 }
 
+/*
+	Search for the 'needle' text on the page.
+	Record the hits in the hit_bbox array and return the number of hits.
+	Will stop looking once it has filled hit_max rectangles.
+*/
 int
-fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_stext_page *text;
 	int count = 0;
@@ -375,7 +354,7 @@ fz_search_page(fz_context *ctx, fz_page *page, const char *needle, fz_rect *hit_
 }
 
 int
-fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char *needle, fz_rect *hit_bbox, int hit_max)
+fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char *needle, fz_quad *hit_bbox, int hit_max)
 {
 	fz_page *page;
 	int count = 0;
@@ -390,6 +369,9 @@ fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char
 	return count;
 }
 
+/*
+	Convert structured text into plain text.
+*/
 fz_buffer *
 fz_new_buffer_from_stext_page(fz_context *ctx, fz_stext_page *page)
 {
@@ -472,25 +454,30 @@ fz_new_buffer_from_page_number(fz_context *ctx, fz_document *doc, int number, co
 	return buf;
 }
 
+/*
+	Write image as a data URI (for HTML and SVG output).
+*/
 void
 fz_write_image_as_data_uri(fz_context *ctx, fz_output *out, fz_image *image)
 {
 	fz_compressed_buffer *cbuf;
 	fz_buffer *buf;
-	int n;
 
-	n = fz_colorspace_n(ctx, image->colorspace);
 	cbuf = fz_compressed_image_buffer(ctx, image);
 
-	if (cbuf && cbuf->params.type == FZ_IMAGE_JPEG && (n == 1 || n == 3))
+	if (cbuf && cbuf->params.type == FZ_IMAGE_JPEG)
 	{
-		fz_write_string(ctx, out, "image/jpeg;base64,");
-		fz_write_base64_buffer(ctx, out, cbuf->buffer, 1);
-		return;
+		int type = fz_colorspace_type(ctx, image->colorspace);
+		if (type == FZ_COLORSPACE_GRAY || type == FZ_COLORSPACE_RGB)
+		{
+			fz_write_string(ctx, out, "data:image/jpeg;base64,");
+			fz_write_base64_buffer(ctx, out, cbuf->buffer, 1);
+			return;
+		}
 	}
 	if (cbuf && cbuf->params.type == FZ_IMAGE_PNG)
 	{
-		fz_write_string(ctx, out, "image/png;base64,");
+		fz_write_string(ctx, out, "data:image/png;base64,");
 		fz_write_base64_buffer(ctx, out, cbuf->buffer, 1);
 		return;
 	}
@@ -498,7 +485,22 @@ fz_write_image_as_data_uri(fz_context *ctx, fz_output *out, fz_image *image)
 	buf = fz_new_buffer_from_image_as_png(ctx, image, NULL);
 	fz_try(ctx)
 	{
-		fz_write_string(ctx, out, "image/png;base64,");
+		fz_write_string(ctx, out, "data:image/png;base64,");
+		fz_write_base64_buffer(ctx, out, buf, 1);
+	}
+	fz_always(ctx)
+		fz_drop_buffer(ctx, buf);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+}
+
+void
+fz_write_pixmap_as_data_uri(fz_context *ctx, fz_output *out, fz_pixmap *pixmap)
+{
+	fz_buffer *buf = fz_new_buffer_from_pixmap_as_png(ctx, pixmap, NULL);
+	fz_try(ctx)
+	{
+		fz_write_string(ctx, out, "data:image/png;base64,");
 		fz_write_base64_buffer(ctx, out, buf, 1);
 	}
 	fz_always(ctx)
