@@ -1,4 +1,9 @@
 #include "mupdf/fitz.h"
+#include "../fitz/fitz-imp.h"
+#include <string.h>
+#include <assert.h>
+#include <stdio.h>
+#include <stdlib.h>
 
 #if FZ_ENABLE_GPRF
 /* Choose whether to call gs via an exe or via an API */
@@ -10,8 +15,6 @@
 #define GS_API_NULL_STDIO
 #endif
 
-#include "mupdf/fitz.h"
-
 #if defined(USE_GS_API)
 
 /* We are assumed to be using the DLL here */
@@ -99,7 +102,7 @@ fz_drop_gprf_file(fz_context *ctx, gprf_file *file)
 {
 	if (fz_drop_imp(ctx, file, &file->refs))
 	{
-		unlink(file->filename);
+		remove(file->filename);
 		fz_free(ctx, file->filename);
 		fz_free(ctx, file);
 	}
@@ -153,17 +156,18 @@ gprf_drop_page_imp(fz_context *ctx, fz_page *page_)
 	fz_drop_gprf_file(ctx, page->file);
 }
 
-static fz_rect *
-gprf_bound_page(fz_context *ctx, fz_page *page_, fz_rect *bbox)
+static fz_rect
+gprf_bound_page(fz_context *ctx, fz_page *page_)
 {
 	gprf_page *page = (gprf_page*)page_;
 	gprf_document *doc = page->doc;
+	fz_rect bbox;
 
 	/* BBox is in points, not pixels */
-	bbox->x0 = 0;
-	bbox->y0 = 0;
-	bbox->x1 = 72.0f * page->width / doc->res;
-	bbox->y1 = 72.0f * page->height / doc->res;
+	bbox.x0 = 0;
+	bbox.y0 = 0;
+	bbox.x1 = 72.0f * page->width / doc->res;
+	bbox.y1 = 72.0f * page->height / doc->res;
 
 	return bbox;
 }
@@ -353,8 +357,9 @@ gprf_get_pixmap(fz_context *ctx, fz_image *image_, fz_irect *area, int w, int h,
 	unsigned char equiv[3 + FZ_MAX_SEPARATIONS][4];
 	int bytes_per_channel = image->super.w * image->super.h;
 	unsigned char *out = fz_pixmap_samples(ctx, pix);
+	fz_stream *tmp_file = NULL;
 
-	fz_var(file);
+	fz_var(tmp_file);
 
 	if (area)
 	{
@@ -404,9 +409,11 @@ gprf_get_pixmap(fz_context *ctx, fz_image *image_, fz_irect *area, int w, int h,
 				read_sep[i] = 2;
 				memset(&data[i * decode_chunk_size], 0, decode_chunk_size);
 			}
-			file[i] = fz_open_file(ctx, image->file->filename);
-			fz_seek(ctx, file[i], image->offset[i], SEEK_SET);
-			file[i] = fz_open_flated(ctx, file[i], 15);
+			tmp_file = fz_open_file(ctx, image->file->filename);
+			fz_seek(ctx, tmp_file, image->offset[i], SEEK_SET);
+			file[i] = fz_open_flated(ctx, tmp_file, 15);
+			fz_drop_stream(ctx, tmp_file);
+			tmp_file = NULL;
 		}
 
 		/* Now actually do the decode */
@@ -463,6 +470,7 @@ gprf_get_pixmap(fz_context *ctx, fz_image *image_, fz_irect *area, int w, int h,
 	}
 	fz_always(ctx)
 	{
+		fz_drop_stream(ctx, tmp_file);
 		for (i = 0; i < 3+num_seps; i++)
 			fz_drop_stream(ctx, file[i]);
 	}
@@ -579,7 +587,6 @@ generate_page(fz_context *ctx, gprf_page *page)
 	char *filename;
 	char *disp_profile = NULL;
 	char *print_profile = NULL;
-	int len;
 
 	/* put the page file in the same directory as the gproof file */
 	fz_snprintf(nameroot, sizeof(nameroot), "gprf_%d_", page->number);
@@ -598,30 +605,14 @@ generate_page(fz_context *ctx, gprf_page *page)
 
 	/* Set up the icc profiles */
 	if (strlen(doc->display_profile) == 0)
-	{
-		len = sizeof("-sPostRenderProfile=srgb.icc");
-		disp_profile = (char*)fz_malloc(ctx, len + 1);
-		fz_snprintf(disp_profile, sizeof(disp_profile), "-sPostRenderProfile=srgb.icc");
-	}
+		disp_profile = fz_asprintf(ctx, "-sPostRenderProfile=srgb.icc");
 	else
-	{
-		len = sizeof("-sPostRenderProfile=" QUOTE QUOTE); /* with quotes */
-		disp_profile = (char*)fz_malloc(ctx, len + strlen(doc->display_profile) + 1);
-		fz_snprintf(disp_profile, sizeof(disp_profile), "-sPostRenderProfile=" QUOTE "%s" QUOTE, doc->display_profile);
-	}
+		disp_profile = fz_asprintf(ctx, "-sPostRenderProfile=" QUOTE "%s" QUOTE, doc->display_profile);
 
 	if (strlen(doc->print_profile) == 0)
-	{
-		len = sizeof("-sOutputICCProfile=default_cmyk.icc");
-		print_profile = (char*)fz_malloc(ctx, len + 1);
-		fz_snprintf(print_profile, sizeof(print_profile), "-sOutputICCProfile=default_cmyk.icc");
-	}
+		print_profile = fz_asprintf(ctx, "-sOutputICCProfile=default_cmyk.icc");
 	else if (strcmp(doc->print_profile, "<EMBEDDED>") != 0)
-	{
-		len = sizeof("-sOutputICCProfile=" QUOTE QUOTE); /* with quotes */
-		print_profile = (char*)fz_malloc(ctx, len + strlen(doc->print_profile) + 1);
-		fz_snprintf(print_profile, sizeof(print_profile), "-sOutputICCProfile=" QUOTE "%s" QUOTE, doc->print_profile);
-	}
+		print_profile = fz_asprintf(ctx, "-sOutputICCProfile=" QUOTE "%s" QUOTE, doc->print_profile);
 
 	fz_try(ctx)
 	{
@@ -649,6 +640,7 @@ generate_page(fz_context *ctx, gprf_page *page)
 		fz_snprintf(arg_lp, sizeof(arg_lp), "-dLastPage=%d", page->number+1);
 		argv[argc++] = arg_lp;
 		argv[argc++] = "-I%rom%Resource/Init/";
+		argv[argc++] = "-dSAFER";
 		fz_snprintf(arg_g, sizeof(arg_g), "-g%dx%d", page->width, page->height);
 		argv[argc++] = arg_g;
 		argv[argc++] = doc->pdf_filename;
@@ -676,7 +668,7 @@ generate_page(fz_context *ctx, gprf_page *page)
 #else
 		char gs_command[1024];
 		/* Invoke gs to convert to a temp file. */
-		fz_snprintf(gs_command, sizeof(gs_command), "gswin32c.exe -sDEVICE=gprf %s %s -dFitPage -o \"%s\" -dFirstPage=%d -dLastPage=%d -I%%rom%%Resource/Init/ -g%dx%d \"%s\"",
+		fz_snprintf(gs_command, sizeof(gs_command), "gswin32c.exe -sDEVICE=gprf %s %s -dSAFER -dFitPage -o \"%s\" -dFirstPage=%d -dLastPage=%d -I%%rom%%Resource/Init/ -g%dx%d \"%s\"",
 			print_profile == NULL ? "-dUsePDFX3Profile" : print_profile, disp_profile,
 			filename, page->number+1, page->number+1, page->width, page->height, doc->pdf_filename);
 		fz_system(ctx, gs_command);
@@ -691,7 +683,7 @@ generate_page(fz_context *ctx, gprf_page *page)
 	}
 	fz_catch(ctx)
 	{
-		unlink(filename);
+		remove(filename);
 		fz_free(ctx, filename);
 		fz_rethrow(ctx);
 	}
@@ -806,7 +798,7 @@ read_tiles(fz_context *ctx, gprf_page *page)
 }
 
 static void
-gprf_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, const fz_matrix *ctm, fz_cookie *cookie)
+gprf_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, fz_matrix ctm, fz_cookie *cookie)
 {
 	gprf_page *page = (gprf_page*)page_;
 	gprf_document *doc = page->doc;
@@ -840,8 +832,8 @@ gprf_run_page(fz_context *ctx, fz_page *page_, fz_device *dev, const fz_matrix *
 			local.d = scale_y;
 			local.e = x * scale;
 			local.f = y * scale;
-			fz_concat(&local, &local, ctm);
-			fz_fill_image(ctx, dev, page->tiles[i++], &local, 1.0f, NULL);
+			local = fz_concat(local, ctm);
+			fz_fill_image(ctx, dev, page->tiles[i++], local, 1.0f, NULL);
 		}
 	}
 	fz_render_flags(ctx, dev, 0, FZ_DEVFLAG_GRIDFIT_AS_TILED);
@@ -885,7 +877,7 @@ gprf_load_page(fz_context *ctx, fz_document *doc_, int number)
 }
 
 static void
-gprf_close_document(fz_context *ctx, fz_document *doc_)
+gprf_drop_document(fz_context *ctx, fz_document *doc_)
 {
 	gprf_document *doc = (gprf_document*)doc_;
 
@@ -911,7 +903,7 @@ gprf_open_document_with_stream(fz_context *ctx, fz_stream *file)
 	gprf_document *doc;
 
 	doc = fz_new_derived_document(ctx, gprf_document);
-	doc->super.drop_document = gprf_close_document;
+	doc->super.drop_document = gprf_drop_document;
 	doc->super.count_pages = gprf_count_pages;
 	doc->super.load_page = gprf_load_page;
 	doc->super.lookup_metadata = gprf_lookup_metadata;
