@@ -94,7 +94,6 @@ pdf_add_codespace(fz_context *ctx, pdf_cmap *cmap, unsigned int low, unsigned in
 	cmap->codespace_len ++;
 }
 
-
 struct cmap_splay_s {
 	unsigned int low;
 	unsigned int high;
@@ -362,7 +361,7 @@ dump_splay(cmap_splay *tree, unsigned int node, int depth, const char *pre)
 {
 	int i;
 
-	if (node == EMPTY)
+	if (tree == NULL || node == EMPTY)
 		return;
 
 	for (i = 0; i < depth; i++)
@@ -440,13 +439,31 @@ static void walk_splay(cmap_splay *tree, unsigned int node, void (*fn)(cmap_spla
 }
 
 #ifdef CHECK_SPLAY
+
+static int
+tree_has_overlap(cmap_splay *tree, int node, int low, int high)
+{
+	if (tree[node].left != EMPTY)
+		if (tree_has_overlap(tree, tree[node].left, low, high))
+			return 1;
+	if (tree[node].right != EMPTY)
+		if (tree_has_overlap(tree, tree[node].right, low, high))
+			return 1;
+	return (tree[node].low < low && low < tree[node].high) || (tree[node].low < high && high < tree[node].high);
+}
+
 static void
 do_check(cmap_splay *node, void *arg)
 {
 	cmap_splay *tree = arg;
 	unsigned int num = node - tree;
-	assert(node->left == EMPTY || tree[node->left].parent == num);
-	assert(node->right == EMPTY || tree[node->right].parent == num);
+	assert(!node->many || node->low == node->high);
+	assert(node->low <= node->high);
+	assert((node->left == EMPTY) || (tree[node->left].parent == num &&
+		tree[node->left].high < node->low));
+	assert(node->right == EMPTY || (tree[node->right].parent == num &&
+		node->high < tree[node->right].low));
+	assert(!tree_has_overlap(tree, num, node->low, node->high));
 }
 
 static void
@@ -467,6 +484,9 @@ add_range(fz_context *ctx, pdf_cmap *cmap, unsigned int low, unsigned int high,
 {
 	int current;
 	cmap_splay *tree;
+	int i;
+	int inrange = 0;
+	unsigned int k, count;
 
 	if (low > high)
 	{
@@ -474,6 +494,22 @@ add_range(fz_context *ctx, pdf_cmap *cmap, unsigned int low, unsigned int high,
 		return;
 	}
 
+	count = high - low + 1;
+	for (k = 0; k < count; k++) {
+		unsigned int c = low + k;
+
+		inrange = 0;
+		for (i = 0; i < cmap->codespace_len; i++) {
+			if (cmap->codespace[i].low <= c && c <= cmap->codespace[i].high)
+				inrange = 1;
+		}
+		if (!inrange)
+		{
+			fz_warn(ctx, "ignoring CMap range (%u-%u) that is outside of the codespace", low, high);
+			return;
+		}
+	}
+
 	tree = cmap->tree;
 
 	if (cmap->tlen)
@@ -504,6 +540,12 @@ add_range(fz_context *ctx, pdf_cmap *cmap, unsigned int low, unsigned int high,
 					tree[current].low = high + 1;
 					if (tree[current].low > tree[current].high)
 					{
+						/* update lt/gt references that will be moved/stale after deleting current */
+						if (gt == cmap->tlen - 1)
+							gt = current;
+						if (lt == cmap->tlen - 1)
+							lt = current;
+						/* delete_node() moves the element at cmap->tlen-1 into current */
 						move = delete_node(cmap, current);
 						current = EMPTY;
 						continue;
@@ -520,7 +562,8 @@ add_range(fz_context *ctx, pdf_cmap *cmap, unsigned int low, unsigned int high,
 					/* case 3, reduces to case 5 */
 					int new_high = tree[current].high;
 					tree[current].high = low-1;
-					add_range(ctx, cmap, high+1, new_high, tree[current].out + high + 1 - tree[current].low, 0, many);
+					add_range(ctx, cmap, high+1, new_high, tree[current].out + high + 1 - tree[current].low, 0, tree[current].many);
+					tree = cmap->tree;
 				}
 				/* Now look for where to move to next (left for case 0, right for case 5) */
 				if (tree[current].low > high) {
