@@ -141,37 +141,6 @@ struct ps_stack_s
 	int sp;
 };
 
-void
-pdf_print_ps_stack(fz_context *ctx, fz_output *out, ps_stack *st)
-{
-	int i;
-
-	fz_write_printf(ctx, out, "stack:");
-
-	for (i = 0; i < st->sp; i++)
-	{
-		switch (st->stack[i].type)
-		{
-		case PS_BOOL:
-			if (st->stack[i].u.b)
-				fz_write_printf(ctx, out, " true");
-			else
-				fz_write_printf(ctx, out, " false");
-			break;
-
-		case PS_INT:
-			fz_write_printf(ctx, out, " %d", st->stack[i].u.i);
-			break;
-
-		case PS_REAL:
-			fz_write_printf(ctx, out, " %g", st->stack[i].u.f);
-			break;
-		}
-	}
-
-	fz_write_printf(ctx, out, "\n");
-}
-
 static void
 ps_init_stack(ps_stack *st)
 {
@@ -186,7 +155,7 @@ static inline int ps_overflow(ps_stack *st, int n)
 
 static inline int ps_underflow(ps_stack *st, int n)
 {
-	return n < 0 || st->sp - n < 0;
+	return n < 0 || n > st->sp;
 }
 
 static inline int ps_is_type(ps_stack *st, int t)
@@ -316,7 +285,7 @@ ps_roll(ps_stack *st, int n, int j)
 static void
 ps_index(ps_stack *st, int n)
 {
-	if (!ps_overflow(st, 1) && !ps_underflow(st, n))
+	if (!ps_overflow(st, 1) && !ps_underflow(st, n + 1))
 	{
 		st->stack[st->sp] = st->stack[st->sp - n - 1];
 		st->sp++;
@@ -870,12 +839,10 @@ load_postscript_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 	int codeptr;
 	pdf_lexbuf buf;
 	pdf_token tok;
-	int locked = 0;
 
 	pdf_lexbuf_init(ctx, &buf, PDF_LEXBUF_SMALL);
 
 	fz_var(stream);
-	fz_var(locked);
 
 	fz_try(ctx)
 	{
@@ -945,18 +912,16 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 	int bps;
 	int i;
 
-	fz_var(stream);
-
 	func->u.sa.samples = NULL;
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Size);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Size));
 	if (pdf_array_len(ctx, obj) < func->m)
 		fz_throw(ctx, FZ_ERROR_SYNTAX, "too few sample function dimension sizes");
 	if (pdf_array_len(ctx, obj) > func->m)
 		fz_warn(ctx, "too many sample function dimension sizes");
 	for (i = 0; i < func->m; i++)
 	{
-		func->u.sa.size[i] = pdf_to_int(ctx, pdf_array_get(ctx, obj, i));
+		func->u.sa.size[i] = pdf_array_get_int(ctx, obj, i);
 		if (func->u.sa.size[i] <= 0)
 		{
 			fz_warn(ctx, "non-positive sample function dimension size");
@@ -964,7 +929,7 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		}
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_BitsPerSample);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(BitsPerSample));
 	func->u.sa.bps = bps = pdf_to_int(ctx, obj);
 
 	for (i = 0; i < func->m; i++)
@@ -972,7 +937,7 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		func->u.sa.encode[i][0] = 0;
 		func->u.sa.encode[i][1] = func->u.sa.size[i] - 1;
 	}
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Encode);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Encode));
 	if (pdf_is_array(ctx, obj))
 	{
 		int ranges = fz_mini(func->m, pdf_array_len(ctx, obj) / 2);
@@ -981,8 +946,8 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 
 		for (i = 0; i < ranges; i++)
 		{
-			func->u.sa.encode[i][0] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 0));
-			func->u.sa.encode[i][1] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 1));
+			func->u.sa.encode[i][0] = pdf_array_get_real(ctx, obj, i * 2 + 0);
+			func->u.sa.encode[i][1] = pdf_array_get_real(ctx, obj, i * 2 + 1);
 		}
 	}
 
@@ -992,7 +957,7 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		func->u.sa.decode[i][1] = func->range[i][1];
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Decode);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Decode));
 	if (pdf_is_array(ctx, obj))
 	{
 		int ranges = fz_mini(func->n, pdf_array_len(ctx, obj) / 2);
@@ -1001,8 +966,8 @@ load_sample_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 
 		for (i = 0; i < ranges; i++)
 		{
-			func->u.sa.decode[i][0] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 0));
-			func->u.sa.decode[i][1] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 1));
+			func->u.sa.decode[i][0] = pdf_array_get_real(ctx, obj, i * 2 + 0);
+			func->u.sa.decode[i][1] = pdf_array_get_real(ctx, obj, i * 2 + 1);
 		}
 	}
 
@@ -1093,7 +1058,7 @@ eval_sample_func(fz_context *ctx, pdf_function *func, const float *in, float *ou
 		x = fz_clamp(x, 0, func->u.sa.size[i] - 1);
 		e0[i] = floorf(x);
 		e1[i] = ceilf(x);
-		efrac[i] = x - floorf(x);
+		efrac[i] = x - e0[i];
 	}
 
 	scale[0] = func->n;
@@ -1154,7 +1119,7 @@ load_exponential_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		fz_warn(ctx, "exponential functions have at most one input");
 	func->m = 1;
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_N);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(N));
 	func->u.e.n = pdf_to_real(ctx, obj);
 
 	/* See exponential functions (PDF 1.7 section 3.9.2) */
@@ -1180,7 +1145,7 @@ load_exponential_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		func->u.e.c1[i] = 1;
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_C0);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(C0));
 	if (pdf_is_array(ctx, obj))
 	{
 		int ranges = fz_mini(func->n, pdf_array_len(ctx, obj));
@@ -1188,10 +1153,10 @@ load_exponential_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 			fz_warn(ctx, "wrong number of C0 constants for exponential function");
 
 		for (i = 0; i < ranges; i++)
-			func->u.e.c0[i] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i));
+			func->u.e.c0[i] = pdf_array_get_real(ctx, obj, i);
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_C1);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(C1));
 	if (pdf_is_array(ctx, obj))
 	{
 		int ranges = fz_mini(func->n, pdf_array_len(ctx, obj));
@@ -1199,7 +1164,7 @@ load_exponential_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 			fz_warn(ctx, "wrong number of C1 constants for exponential function");
 
 		for (i = 0; i < ranges; i++)
-			func->u.e.c1[i] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i));
+			func->u.e.c1[i] = pdf_array_get_real(ctx, obj, i);
 	}
 }
 
@@ -1245,7 +1210,7 @@ load_stitching_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		fz_warn(ctx, "stitching functions have at most one input");
 	func->m = 1;
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Functions);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Functions));
 	if (!pdf_is_array(ctx, obj))
 		fz_throw(ctx, FZ_ERROR_SYNTAX, "stitching function has no input functions");
 
@@ -1283,7 +1248,7 @@ load_stitching_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		fz_rethrow(ctx);
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Bounds);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Bounds));
 	if (!pdf_is_array(ctx, obj))
 		fz_throw(ctx, FZ_ERROR_SYNTAX, "stitching function has no bounds");
 	{
@@ -1311,7 +1276,7 @@ load_stitching_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 		func->u.st.encode[i * 2 + 1] = 0;
 	}
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Encode);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Encode));
 	if (pdf_is_array(ctx, obj))
 	{
 		int ranges = fz_mini(k, pdf_array_len(ctx, obj) / 2);
@@ -1320,8 +1285,8 @@ load_stitching_func(fz_context *ctx, pdf_function *func, pdf_obj *dict)
 
 		for (i = 0; i < ranges; i++)
 		{
-			func->u.st.encode[i * 2 + 0] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 0));
-			func->u.st.encode[i * 2 + 1] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 1));
+			func->u.st.encode[i * 2 + 0] = pdf_array_get_real(ctx, obj, i * 2 + 0);
+			func->u.st.encode[i * 2 + 1] = pdf_array_get_real(ctx, obj, i * 2 + 1);
 		}
 	}
 }
@@ -1365,7 +1330,7 @@ eval_stitching_func(fz_context *ctx, pdf_function *func, float in, float *out)
 
 	in = lerp(in, low, high, func->u.st.encode[i * 2 + 0], func->u.st.encode[i * 2 + 1]);
 
-	pdf_eval_function(ctx, func->u.st.funcs[i], &in, 1, out, func->u.st.funcs[i]->n);
+	pdf_eval_function(ctx, func->u.st.funcs[i], &in, 1, out, func->n);
 }
 
 /*
@@ -1420,9 +1385,9 @@ pdf_eval_function(fz_context *ctx, pdf_function *func, const float *in, int inle
 
 	if (inlen < func->m)
 	{
-		for (i = 0; i < func->m; ++i)
+		for (i = 0; i < inlen; ++i)
 			fakein[i] = in[i];
-		for (; i < inlen; ++i)
+		for (; i < func->m; ++i)
 			fakein[i] = 0;
 		in = fakein;
 	}
@@ -1458,28 +1423,28 @@ pdf_load_function(fz_context *ctx, pdf_obj *dict, int in, int out)
 	FZ_INIT_STORABLE(func, 1, pdf_drop_function_imp);
 	func->size = sizeof(*func);
 
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_FunctionType);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(FunctionType));
 	func->type = pdf_to_int(ctx, obj);
 
 	/* required for all */
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Domain);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Domain));
 	func->m = fz_clampi(pdf_array_len(ctx, obj) / 2, 1, MAX_M);
 	for (i = 0; i < func->m; i++)
 	{
-		func->domain[i][0] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 0));
-		func->domain[i][1] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 1));
+		func->domain[i][0] = pdf_array_get_real(ctx, obj, i * 2 + 0);
+		func->domain[i][1] = pdf_array_get_real(ctx, obj, i * 2 + 1);
 	}
 
 	/* required for type0 and type4, optional otherwise */
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Range);
+	obj = pdf_dict_get(ctx, dict, PDF_NAME(Range));
 	if (pdf_is_array(ctx, obj))
 	{
 		func->has_range = 1;
 		func->n = fz_clampi(pdf_array_len(ctx, obj) / 2, 1, MAX_N);
 		for (i = 0; i < func->n; i++)
 		{
-			func->range[i][0] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 0));
-			func->range[i][1] = pdf_to_real(ctx, pdf_array_get(ctx, obj, i * 2 + 1));
+			func->range[i][0] = pdf_array_get_real(ctx, obj, i * 2 + 0);
+			func->range[i][1] = pdf_array_get_real(ctx, obj, i * 2 + 1);
 		}
 	}
 	else
