@@ -13,7 +13,6 @@ struct pdf_js_s
 	fz_context *ctx;
 	pdf_document *doc;
 	pdf_obj *form;
-	pdf_js_event event;
 	js_State *imp;
 };
 
@@ -49,39 +48,6 @@ static pdf_js *unpack_arguments(js_State *J, ...)
 	return js_getcontext(J);
 }
 
-static char *pdf_from_utf8(fz_context *ctx, const char *utf8)
-{
-	char *pdf = fz_malloc(ctx, strlen(utf8)+1);
-	int i = 0;
-	unsigned char c;
-
-	while ((c = *utf8) != 0)
-	{
-		if ((c & 0x80) == 0 && pdf_doc_encoding[c] == c)
-		{
-			pdf[i++] = c;
-			utf8++ ;
-		}
-		else
-		{
-			int rune;
-			int j;
-
-			utf8 += fz_chartorune(&rune, utf8);
-
-			for (j = 0; j < sizeof(pdf_doc_encoding) && pdf_doc_encoding[j] != rune; j++)
-				;
-
-			if (j < sizeof(pdf_doc_encoding))
-				pdf[i++] = j;
-		}
-	}
-
-	pdf[i] = 0;
-
-	return pdf;
-}
-
 static void app_alert(js_State *J)
 {
 	pdf_js *js = unpack_arguments(J, "cMsg", "nIcon", "nType", "cTitle", 0);
@@ -101,22 +67,6 @@ static void app_alert(js_State *J)
 	js_pushnumber(J, event.button_pressed);
 }
 
-static void app_execDialog(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	// monitor
-	// inheritDialog
-	// parentDoc
-
-	fz_try(js->ctx)
-		pdf_event_issue_exec_dialog(js->ctx, js->doc);
-	fz_catch(js->ctx)
-		rethrow(js);
-
-	// return "ok" or "cancel"
-	js_pushstring(J, "cancel");
-}
-
 static void app_execMenuItem(js_State *J)
 {
 	pdf_js *js = js_getcontext(J);
@@ -149,11 +99,8 @@ static void field_buttonSetCaption(js_State *J)
 	pdf_js *js = js_getcontext(J);
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	const char *cCaption = js_tostring(J, 1);
-	char *caption = pdf_from_utf8(js->ctx, cCaption);
 	fz_try(js->ctx)
-		pdf_field_set_button_caption(js->ctx, js->doc, field, caption);
-	fz_always(js->ctx)
-		fz_free(js->ctx, caption);
+		pdf_field_set_button_caption(js->ctx, field, cCaption);
 	fz_catch(js->ctx)
 		rethrow(js);
 }
@@ -164,10 +111,10 @@ static void field_getName(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	char *name = NULL;
 	fz_try(js->ctx)
-		name = pdf_field_name(js->ctx, js->doc, field);
+		name = pdf_field_name(js->ctx, field);
 	fz_catch(js->ctx)
 		rethrow(js);
-	js_pushstring(J, name); /* to utf8? */
+	js_pushstring(J, name);
 }
 
 static void field_setName(js_State *J)
@@ -182,7 +129,7 @@ static void field_getDisplay(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	int display = 0;
 	fz_try(js->ctx)
-		display = pdf_field_display(js->ctx, js->doc, field);
+		display = pdf_field_display(js->ctx, field);
 	fz_catch(js->ctx)
 		rethrow(js);
 	js_pushnumber(J, display);
@@ -194,7 +141,7 @@ static void field_setDisplay(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	int display = js_tonumber(J, 1);
 	fz_try(js->ctx)
-		pdf_field_set_display(js->ctx, js->doc, field, display);
+		pdf_field_set_display(js->ctx, field, display);
 	fz_catch(js->ctx)
 		rethrow(js);
 }
@@ -251,7 +198,7 @@ static void field_setFillColor(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	pdf_obj *color = load_color(js, 1);
 	fz_try(js->ctx)
-		pdf_field_set_fill_color(js->ctx, js->doc, field, color);
+		pdf_field_set_fill_color(js->ctx, field, color);
 	fz_always(js->ctx)
 		pdf_drop_obj(js->ctx, color);
 	fz_catch(js->ctx)
@@ -269,7 +216,7 @@ static void field_setTextColor(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	pdf_obj *color = load_color(js, 1);
 	fz_try(js->ctx)
-		pdf_field_set_text_color(js->ctx, js->doc, field, color);
+		pdf_field_set_text_color(js->ctx, field, color);
 	fz_always(js->ctx)
 		pdf_drop_obj(js->ctx, color);
 	fz_catch(js->ctx)
@@ -282,7 +229,7 @@ static void field_getBorderStyle(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	const char *border_style = NULL;
 	fz_try(js->ctx)
-		border_style = pdf_field_border_style(js->ctx, js->doc, field);
+		border_style = pdf_field_border_style(js->ctx, field);
 	fz_catch(js->ctx)
 		rethrow(js);
 	js_pushstring(J, border_style);
@@ -294,7 +241,7 @@ static void field_setBorderStyle(js_State *J)
 	pdf_obj *field = js_touserdata(J, 0, "Field");
 	const char *border_style = js_tostring(J, 1);
 	fz_try(js->ctx)
-		pdf_field_set_border_style(js->ctx, js->doc, field, border_style);
+		pdf_field_set_border_style(js->ctx, field, border_style);
 	fz_catch(js->ctx)
 		rethrow(js);
 }
@@ -303,16 +250,20 @@ static void field_getValue(js_State *J)
 {
 	pdf_js *js = js_getcontext(J);
 	pdf_obj *field = js_touserdata(J, 0, "Field");
-	char *val = NULL;
+	const char *str = NULL;
+	char *end;
+	double num;
 
 	fz_try(js->ctx)
-		val = pdf_field_value(js->ctx, js->doc, field);
+		str = pdf_field_value(js->ctx, field);
 	fz_catch(js->ctx)
 		rethrow(js);
 
-	js_pushstring(J, val ? val : "");
-
-	fz_free(js->ctx, val);
+	num = strtod(str, &end);
+	if (*str && *end == 0)
+		js_pushnumber(J, num);
+	else
+		js_pushstring(J, str);
 }
 
 static void field_setValue(js_State *J)
@@ -322,74 +273,20 @@ static void field_setValue(js_State *J)
 	const char *value = js_tostring(J, 1);
 
 	fz_try(js->ctx)
-		(void)pdf_field_set_value(js->ctx, js->doc, field, value);
+		(void)pdf_set_field_value(js->ctx, js->doc, field, value, 0);
 	fz_catch(js->ctx)
 		rethrow(js);
 }
 
-static void event_getTarget(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	js_getregistry(J, "Field");
-	js_newuserdata(J, "Field", pdf_keep_obj(js->ctx, js->event.target), field_finalize);
-}
-
-static void event_setTarget(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	fz_warn(js->ctx, "Unexpected call to event_setTarget");
-}
-
-static void event_getValue(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	const char *v = js->event.value;
-	js_pushstring(J, v ? v : "");
-}
-
-static void event_setValue(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	const char *value = js_tostring(J, 1);
-	fz_free(js->ctx, js->event.value);
-	js->event.value = fz_strdup(js->ctx, value);
-}
-
-static void event_getWillCommit(js_State *J)
-{
-	js_pushnumber(J, 1);
-}
-
-static void event_setWillCommit(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	fz_warn(js->ctx, "Unexpected call to event_setWillCommit");
-}
-
-static void event_getRC(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	js_pushnumber(J, js->event.rc);
-}
-
-static void event_setRC(js_State *J)
-{
-	pdf_js *js = js_getcontext(J);
-	js->event.rc = js_tointeger(js->imp, 1);
-}
-
 static void doc_getField(js_State *J)
 {
 	pdf_js *js = js_getcontext(J);
 	fz_context *ctx = js->ctx;
 	const char *cName = js_tostring(J, 1);
-	char *name = pdf_from_utf8(ctx, cName);
 	pdf_obj *dict = NULL;
 
 	fz_try(ctx)
-		dict = pdf_lookup_field(ctx, js->form, name);
-	fz_always(ctx)
-		fz_free(ctx, name);
+		dict = pdf_lookup_field(ctx, js->form, cName);
 	fz_catch(ctx)
 		rethrow(js);
 
@@ -404,28 +301,23 @@ static void doc_getField(js_State *J)
 	}
 }
 
-static void reset_field(pdf_js *js, const char *cName)
+static void doc_getNumPages(js_State *J)
 {
-	fz_context *ctx = js->ctx;
-	if (cName)
-	{
-		char *name = pdf_from_utf8(ctx, cName);
-		fz_try(ctx)
-		{
-			pdf_obj *field = js_touserdata(js->imp, 0, "Field");
-			if (field)
-				pdf_field_reset(ctx, js->doc, field);
-		}
-		fz_always(ctx)
-			fz_free(ctx, name);
-		fz_catch(ctx)
-			rethrow(js);
-	}
+	pdf_js *js = js_getcontext(J);
+	int pages = pdf_count_pages(js->ctx, js->doc);
+	js_pushnumber(J, pages);
+}
+
+static void doc_setNumPages(js_State *J)
+{
+	pdf_js *js = js_getcontext(J);
+	fz_warn(js->ctx, "Unexpected call to doc_setNumPages");
 }
 
 static void doc_resetForm(js_State *J)
 {
 	pdf_js *js = js_getcontext(J);
+	pdf_obj *field;
 	fz_context *ctx = js->ctx;
 	int i, n;
 
@@ -436,7 +328,9 @@ static void doc_resetForm(js_State *J)
 		for (i = 0; i < n; ++i)
 		{
 			js_getindex(J, 1, i);
-			reset_field(js, js_tostring(J, -1));
+			field = pdf_lookup_field(ctx, js->form, js_tostring(J, -1));
+			if (field)
+				pdf_field_reset(ctx, js->doc, field);
 			js_pop(J, 1);
 		}
 	}
@@ -482,6 +376,258 @@ static void doc_mailDoc(js_State *J)
 		rethrow(js);
 }
 
+static void doc_calculateNow(js_State *J)
+{
+	pdf_js *js = js_getcontext(J);
+	fz_try(js->ctx)
+		pdf_calculate_form(js->ctx, js->doc);
+	fz_catch(js->ctx)
+		rethrow(js);
+}
+
+static void console_println(js_State *J)
+{
+	int i, top = js_gettop(J);
+	for (i = 1; i < top; ++i) {
+		const char *s = js_tostring(J, i);
+		if (i > 1) putchar(' ');
+		fputs(s, stdout);
+	}
+	putchar('\n');
+	js_pushundefined(J);
+}
+
+static void util_printf_d(fz_context *ctx, fz_buffer *out, int ds, int sign, int pad, int w, int base, int value)
+{
+	static const char *digits = "0123456789abcdef";
+	char buf[50];
+	unsigned int a;
+	int i, m = 0;
+
+	if (value < 0)
+	{
+		sign = '-';
+		a = -value;
+	}
+	else
+	{
+		a = value;
+	}
+
+	i = 0;
+	do
+	{
+		buf[i++] = digits[a % base];
+		a /= base;
+		if (a > 0 && ++m == 3)
+		{
+			if (ds == 0) buf[i++] = ',';
+			if (ds == 2) buf[i++] = '.';
+			m = 0;
+		}
+	} while (a);
+
+	if (sign)
+	{
+		if (pad == '0')
+			while (i < w - 1)
+				buf[i++] = pad;
+		buf[i++] = sign;
+	}
+	while (i < w)
+		buf[i++] = pad;
+
+	while (i > 0)
+		fz_append_byte(ctx, out, buf[--i]);
+}
+
+static void util_printf_f(fz_context *ctx, fz_buffer *out, int ds, int sign, int pad, int special, int w, int p, double value)
+{
+	char buf[40], *point, *digits = buf;
+	int n = 0;
+	int m = 0;
+
+	fz_snprintf(buf, sizeof buf, "%.*f", p, value);
+
+	if (*digits == '-')
+	{
+		sign = '-';
+		++digits;
+	}
+
+	if (*digits != '.' && (*digits < '0' || *digits > '9'))
+	{
+		fz_append_string(ctx, out, "nan");
+		return;
+	}
+
+	n = strlen(digits);
+	if (sign)
+		++n;
+	point = strchr(digits, '.');
+	if (point)
+		m = 3 - (point - digits) % 3;
+	else
+	{
+		m = 3 - n % 3;
+		if (special)
+			++n;
+	}
+	if (m == 3)
+		m = 0;
+
+	if (pad == '0' && sign)
+		fz_append_byte(ctx, out, sign);
+	for (; n < w; ++n)
+		fz_append_byte(ctx, out, pad);
+	if (pad == ' ' && sign)
+		fz_append_byte(ctx, out, sign);
+
+	while (*digits && *digits != '.')
+	{
+		fz_append_byte(ctx, out, *digits++);
+		if (++m == 3 && *digits && *digits != '.')
+		{
+			if (ds == 0) fz_append_byte(ctx, out, ',');
+			if (ds == 2) fz_append_byte(ctx, out, '.');
+			m = 0;
+		}
+	}
+
+	if (*digits == '.' || special)
+	{
+		if (ds == 0 || ds == 1)
+			fz_append_byte(ctx, out, '.');
+		else
+			fz_append_byte(ctx, out, ',');
+	}
+
+	if (*digits == '.')
+	{
+		++digits;
+		while (*digits)
+			fz_append_byte(ctx, out, *digits++);
+	}
+}
+
+static void util_printf(js_State *J)
+{
+	pdf_js *js = js_getcontext(J);
+	fz_context *ctx = js->ctx;
+	const char *fmt = js_tostring(J, 1);
+	fz_buffer *out = NULL;
+	int ds, w, p, sign, pad, special;
+	int c, i = 1;
+	int failed = 0;
+	const char *str;
+
+	fz_var(out);
+	fz_try(ctx)
+	{
+		out = fz_new_buffer(ctx, 256);
+
+		while ((c = *fmt++) != 0)
+		{
+			if (c == '%')
+			{
+				c = *fmt++;
+
+				ds = 1;
+				if (c == ',')
+				{
+					c = *fmt++;
+					if (!c)
+						break;
+					ds = c - '0';
+				}
+
+				special = 0;
+				sign = 0;
+				pad = ' ';
+				while (c == ' ' || c == '+' || c == '0' || c == '#')
+				{
+					if (c == '+') sign = '+';
+					else if (c == ' ') sign = ' ';
+					else if (c == '0') pad = '0';
+					else if (c == '#') special = 1;
+					c = *fmt++;
+				}
+				if (!pad)
+					pad = ' ';
+				if (!c)
+					break;
+
+				w = 0;
+				while (c >= '0' && c <= '9')
+				{
+					w = w * 10 + (c - '0');
+					c = *fmt++;
+				}
+				if (!c)
+					break;
+
+				p = 0;
+				if (c == '.')
+				{
+					c = *fmt++;
+					while (c >= '0' && c <= '9')
+					{
+						p = p * 10 + (c - '0');
+						c = *fmt++;
+					}
+				}
+				else
+				{
+					special = 1;
+				}
+				if (!c)
+					break;
+
+				switch (c)
+				{
+				case '%':
+					fz_append_byte(ctx, out, '%');
+					break;
+				case 'x':
+					util_printf_d(ctx, out, ds, sign, pad, w, 16, js_tryinteger(J, ++i, 0));
+					break;
+				case 'd':
+					util_printf_d(ctx, out, ds, sign, pad, w, 10, js_tryinteger(J, ++i, 0));
+					break;
+				case 'f':
+					util_printf_f(ctx, out, ds, sign, pad, special, w, p, js_trynumber(J, ++i, 0));
+					break;
+				case 's':
+				default:
+					fz_append_string(ctx, out, js_trystring(J, ++i, ""));
+				}
+			}
+			else
+			{
+				fz_append_byte(ctx, out, c);
+			}
+		}
+
+		str = fz_string_from_buffer(ctx, out);
+		if (js_try(J))
+		{
+			failed = 1;
+		}
+		else
+		{
+			js_pushstring(J, str);
+			js_endtry(J);
+		}
+	}
+	fz_always(ctx)
+		fz_drop_buffer(ctx, out);
+	fz_catch(ctx)
+		rethrow(js);
+
+	if (failed)
+		js_throw(J);
+}
+
 static void addmethod(js_State *J, const char *name, js_CFunction fun, int n)
 {
 	const char *realname = strchr(name, '.');
@@ -507,6 +653,19 @@ static void declare_dom(pdf_js *js)
 	js_pushglobal(J);
 	js_defglobal(J, "global", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
 
+	/* Create the 'event' object */
+	js_newobject(J);
+	js_defglobal(J, "event", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
+
+	/* Create the 'util' object */
+	js_newobject(J);
+	{
+		// TODO: util.printd
+		// TODO: util.printx
+		addmethod(J, "util.printf", util_printf, 1);
+	}
+	js_defglobal(J, "util", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
+
 	/* Create the 'app' object */
 	js_newobject(J);
 	{
@@ -520,22 +679,11 @@ static void declare_dom(pdf_js *js)
 		js_defproperty(J, -2, "app.platform", JS_READONLY | JS_DONTENUM | JS_DONTCONF);
 
 		addmethod(J, "app.alert", app_alert, 4);
-		addmethod(J, "app.execDialog", app_execDialog, 0);
 		addmethod(J, "app.execMenuItem", app_execMenuItem, 1);
 		addmethod(J, "app.launchURL", app_launchURL, 2);
 	}
 	js_defglobal(J, "app", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
 
-	/* Create the 'event' object */
-	js_newobject(J);
-	{
-		addproperty(J, "event.target", event_getTarget, event_setTarget);
-		addproperty(J, "event.value", event_getValue, event_setValue);
-		addproperty(J, "event.willCommit", event_getWillCommit, event_setWillCommit);
-		addproperty(J, "event.rc", event_getRC, event_setRC);
-	}
-	js_defglobal(J, "event", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
-
 	/* Create the Field prototype object */
 	js_newobject(J);
 	{
@@ -549,18 +697,25 @@ static void declare_dom(pdf_js *js)
 	}
 	js_setregistry(J, "Field");
 
-	/* Create the Doc prototype object */
+	/* Create the console object */
 	js_newobject(J);
 	{
+		addmethod(J, "console.println", console_println, 1);
+	}
+	js_defglobal(J, "console", JS_READONLY | JS_DONTCONF | JS_DONTENUM);
+
+	/* Put all of the Doc methods in the global object, which is used as
+	 * the 'this' binding for regular non-strict function calls. */
+	js_pushglobal(J);
+	{
+		addproperty(J, "Doc.numPages", doc_getNumPages, doc_setNumPages);
 		addmethod(J, "Doc.getField", doc_getField, 1);
 		addmethod(J, "Doc.resetForm", doc_resetForm, 0);
+		addmethod(J, "Doc.calculateNow", doc_calculateNow, 0);
 		addmethod(J, "Doc.print", doc_print, 0);
 		addmethod(J, "Doc.mailDoc", doc_mailDoc, 6);
 	}
-	js_setregistry(J, "Doc");
-
-	js_getregistry(J, "Doc");
-	js_setglobal(J, "MuPDF_Doc"); /* for pdf-util.js use */
+	js_pop(J, 1);
 }
 
 static void preload_helpers(pdf_js *js)
@@ -591,7 +746,6 @@ void pdf_drop_js(fz_context *ctx, pdf_js *js)
 	if (js)
 	{
 		js_freestate(js->imp);
-		fz_free(ctx, js->event.value);
 		fz_free(ctx, js);
 	}
 }
@@ -659,7 +813,12 @@ static void pdf_js_load_document_level(pdf_js *js)
 			pdf_obj *fragment = pdf_dict_get_val(ctx, javascript, i);
 			pdf_obj *code = pdf_dict_get(ctx, fragment, PDF_NAME(JS));
 			char *codebuf = pdf_load_stream_or_string_as_utf8(ctx, code);
-			pdf_js_execute(js, codebuf);
+			char buf[100];
+			if (pdf_is_indirect(ctx, code))
+				fz_snprintf(buf, sizeof buf, "%d", pdf_to_num(ctx, code));
+			else
+				fz_snprintf(buf, sizeof buf, "Root/Names/JavaScript/Names/%d/JS", (i+1)*2);
+			pdf_js_execute(js, buf, codebuf);
 			fz_free(ctx, codebuf);
 		}
 	}
@@ -669,38 +828,112 @@ static void pdf_js_load_document_level(pdf_js *js)
 		fz_rethrow(ctx);
 }
 
-void pdf_js_setup_event(pdf_js *js, pdf_js_event *e)
+void pdf_js_event_init(pdf_js *js, pdf_obj *target, const char *value, int willCommit)
 {
 	if (js)
 	{
-		fz_context *ctx = js->ctx;
-		char *ev = e->value ? e->value : "";
-		char *v = fz_strdup(ctx, ev);
+		js_getglobal(js->imp, "event");
+		{
+			js_pushboolean(js->imp, 1);
+			js_setproperty(js->imp, -2, "rc");
 
-		fz_free(ctx, js->event.value);
-		js->event.value = v;
+			js_pushboolean(js->imp, willCommit);
+			js_setproperty(js->imp, -2, "willCommit");
 
-		js->event.target = e->target;
-		js->event.rc = 1;
+			js_getregistry(js->imp, "Field");
+			js_newuserdata(js->imp, "Field", pdf_keep_obj(js->ctx, target), field_finalize);
+			js_setproperty(js->imp, -2, "target");
+
+			js_pushstring(js->imp, value);
+			js_setproperty(js->imp, -2, "value");
+		}
+		js_pop(js->imp, 1);
 	}
 }
 
-pdf_js_event *pdf_js_get_event(pdf_js *js)
+int pdf_js_event_result(pdf_js *js)
 {
-	return js ? &js->event : NULL;
+	int rc = 1;
+	if (js)
+	{
+		js_getglobal(js->imp, "event");
+		js_getproperty(js->imp, -1, "rc");
+		rc = js_tryboolean(js->imp, -1, 1);
+		js_pop(js->imp, 2);
+	}
+	return rc;
 }
 
-void pdf_js_execute(pdf_js *js, char *source)
+void pdf_js_event_init_keystroke(pdf_js *js, pdf_obj *target, pdf_keystroke_event *event)
 {
 	if (js)
 	{
-		if (js_ploadstring(js->imp, "[pdf]", source))
+		pdf_js_event_init(js, target, event->value, event->willCommit);
+		js_getglobal(js->imp, "event");
+		{
+			js_pushstring(js->imp, event->change);
+			js_setproperty(js->imp, -2, "change");
+			js_pushnumber(js->imp, event->selStart);
+			js_setproperty(js->imp, -2, "selStart");
+			js_pushnumber(js->imp, event->selEnd);
+			js_setproperty(js->imp, -2, "selEnd");
+		}
+		js_pop(js->imp, 1);
+	}
+}
+
+int pdf_js_event_result_keystroke(pdf_js *js, pdf_keystroke_event *evt)
+{
+	int rc = 1;
+	if (js)
+	{
+		js_getglobal(js->imp, "event");
+		{
+			js_getproperty(js->imp, -1, "rc");
+			rc = js_tryboolean(js->imp, -1, 1);
+			js_pop(js->imp, 1);
+			if (rc)
+			{
+				js_getproperty(js->imp, -1, "change");
+				evt->newChange = fz_strdup(js->ctx, js_trystring(js->imp, -1, ""));
+				js_pop(js->imp, 1);
+				js_getproperty(js->imp, -1, "selStart");
+				evt->selStart = js_tryinteger(js->imp, -1, 0);
+				js_pop(js->imp, 1);
+				js_getproperty(js->imp, -1, "selEnd");
+				evt->selEnd = js_tryinteger(js->imp, -1, 0);
+				js_pop(js->imp, 1);
+			}
+		}
+		js_pop(js->imp, 1);
+	}
+	return rc;
+}
+
+char *pdf_js_event_value(pdf_js *js)
+{
+	char *value = NULL;
+	if (js)
+	{
+		js_getglobal(js->imp, "event");
+		js_getproperty(js->imp, -1, "value");
+		value = fz_strdup(js->ctx, js_trystring(js->imp, -1, "undefined"));
+		js_pop(js->imp, 2);
+	}
+	return value;
+}
+
+void pdf_js_execute(pdf_js *js, const char *name, const char *source)
+{
+	if (js)
+	{
+		if (js_ploadstring(js->imp, name, source))
 		{
 			fz_warn(js->ctx, "%s", js_trystring(js->imp, -1, "Error"));
 			js_pop(js->imp, 1);
 			return;
 		}
-		js_getregistry(js->imp, "Doc"); /* set 'this' to the Doc object */
+		js_pushundefined(js->imp);
 		if (js_pcall(js->imp, 0))
 		{
 			fz_warn(js->ctx, "%s", js_trystring(js->imp, -1, "Error"));
@@ -737,8 +970,11 @@ void pdf_drop_js(fz_context *ctx, pdf_js *js) { }
 void pdf_enable_js(fz_context *ctx, pdf_document *doc) { }
 void pdf_disable_js(fz_context *ctx, pdf_document *doc) { }
 int pdf_js_supported(fz_context *ctx, pdf_document *doc) { return 0; }
-void pdf_js_setup_event(pdf_js *js, pdf_js_event *e) { }
-pdf_js_event *pdf_js_get_event(pdf_js *js) { return NULL; }
-void pdf_js_execute(pdf_js *js, char *code) { }
+void pdf_js_event_init(pdf_js *js, pdf_obj *target, const char *value, int willCommit) { }
+void pdf_js_event_init_keystroke(pdf_js *js, pdf_obj *target, pdf_keystroke_event *event) { }
+int pdf_js_event_result_keystroke(pdf_js *js, pdf_keystroke_event *evt) { return 1; }
+int pdf_js_event_result(pdf_js *js) { return 1; }
+char *pdf_js_event_value(pdf_js *js) { return ""; }
+void pdf_js_execute(pdf_js *js, const char *name, const char *source) { }
 
 #endif /* FZ_ENABLE_JS */
