@@ -18,35 +18,17 @@ pdf_load_page_tree_imp(fz_context *ctx, pdf_document *doc, pdf_obj *node, int id
 	if (pdf_name_eq(ctx, type, PDF_NAME(Pages)))
 	{
 		pdf_obj *kids = pdf_dict_get(ctx, node, PDF_NAME(Kids));
-		int count = pdf_dict_get_int(ctx, node, PDF_NAME(Count));
 		int i, n = pdf_array_len(ctx, kids);
 
-		/* if Kids length is same as Count, all children must be page objects */
-		if (n == count)
-		{
+		if (pdf_mark_obj(ctx, node))
+			fz_throw(ctx, FZ_ERROR_GENERIC, "cycle in page tree");
+		fz_try(ctx)
 			for (i = 0; i < n; ++i)
-			{
-				if (idx >= doc->rev_page_count)
-					fz_throw(ctx, FZ_ERROR_GENERIC, "too many kids in page tree");
-				doc->rev_page_map[idx].page = idx;
-				doc->rev_page_map[idx].object = pdf_to_num(ctx, pdf_array_get(ctx, kids, i));
-				++idx;
-			}
-		}
-
-		/* else Kids may contain intermediate nodes */
-		else
-		{
-			if (pdf_mark_obj(ctx, node))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "cycle in page tree");
-			fz_try(ctx)
-				for (i = 0; i < n; ++i)
-					idx = pdf_load_page_tree_imp(ctx, doc, pdf_array_get(ctx, kids, i), idx);
-			fz_always(ctx)
-				pdf_unmark_obj(ctx, node);
-			fz_catch(ctx)
-				fz_rethrow(ctx);
-		}
+				idx = pdf_load_page_tree_imp(ctx, doc, pdf_array_get(ctx, kids, i), idx);
+		fz_always(ctx)
+			pdf_unmark_obj(ctx, node);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
 	}
 	else if (pdf_name_eq(ctx, type, PDF_NAME(Page)))
 	{
@@ -314,6 +296,12 @@ pdf_lookup_page_number(fz_context *ctx, pdf_document *doc, pdf_obj *page)
 		return pdf_lookup_page_number_slow(ctx, doc, page);
 }
 
+/*
+	Find the page number of a named destination.
+
+	For use with looking up the destination page of a fragment
+	identifier in hyperlinks: foo.pdf#bar or foo.pdf#page=5.
+*/
 int
 pdf_lookup_anchor(fz_context *ctx, pdf_document *doc, const char *name, float *xp, float *yp)
 {
@@ -351,6 +339,12 @@ pdf_flatten_inheritable_page_item(fz_context *ctx, pdf_obj *page, pdf_obj *key)
 		pdf_dict_put(ctx, page, key, val);
 }
 
+/*
+	Make page self sufficient.
+
+	Copy any inheritable page keys into the actual page object, removing
+	any dependencies on the page tree parents.
+*/
 void
 pdf_flatten_inheritable_page_items(fz_context *ctx, pdf_obj *page)
 {
@@ -362,6 +356,16 @@ pdf_flatten_inheritable_page_items(fz_context *ctx, pdf_obj *page)
 
 /* We need to know whether to install a page-level transparency group */
 
+/*
+ * Object memo flags - allows us to secretly remember "a memo" (a bool) in an
+ * object, and to read back whether there was a memo, and if so, what it was.
+ */
+enum
+{
+	PDF_FLAGS_MEMO_BM = 0,
+	PDF_FLAGS_MEMO_OP = 1
+};
+
 static int pdf_resources_use_blending(fz_context *ctx, pdf_obj *rdb);
 
 static int
@@ -584,6 +588,14 @@ pdf_page_presentation(fz_context *ctx, pdf_page *page, fz_transition *transition
 	return transition;
 }
 
+/*
+	Determine the size of a page.
+
+	Determine the page size in user space units, taking page rotation
+	into account. The page size is taken to be the crop box if it
+	exists (visible area after cropping), otherwise the media box will
+	be used (possibly including printing marks).
+*/
 fz_rect
 pdf_bound_page(fz_context *ctx, pdf_page *page)
 {
@@ -711,11 +723,20 @@ find_seps(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 				return; /* Got that one already */
 		}
 
-		cs = pdf_load_colorspace(ctx, obj);
-		if (!*seps)
-			*seps = fz_new_separations(ctx, 0);
-		fz_add_separation(ctx, *seps, name, cs, 0);
-		fz_drop_colorspace(ctx, cs);
+		fz_try(ctx)
+			cs = pdf_load_colorspace(ctx, obj);
+		fz_catch(ctx)
+			return; /* ignore broken colorspace */
+		fz_try(ctx)
+		{
+			if (!*seps)
+				*seps = fz_new_separations(ctx, 0);
+			fz_add_separation(ctx, *seps, name, cs, 0);
+		}
+		fz_always(ctx)
+			fz_drop_colorspace(ctx, cs);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
 	}
 	else if (pdf_name_eq(ctx, nameobj, PDF_NAME(Indexed)))
 	{
@@ -767,11 +788,20 @@ find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 
 		if (i == n)
 		{
-			cs = pdf_load_colorspace(ctx, obj);
-			if (!*seps)
-				*seps = fz_new_separations(ctx, 0);
-			fz_add_separation(ctx, *seps, name, cs, j);
-			fz_drop_colorspace(ctx, cs);
+			fz_try(ctx)
+				cs = pdf_load_colorspace(ctx, obj);
+			fz_catch(ctx)
+				continue; /* ignore broken colorspace */
+			fz_try(ctx)
+			{
+				if (!*seps)
+					*seps = fz_new_separations(ctx, 0);
+				fz_add_separation(ctx, *seps, name, cs, j);
+			}
+			fz_always(ctx)
+				fz_drop_colorspace(ctx, cs);
+			fz_catch(ctx)
+				fz_rethrow(ctx);
 		}
 	}
 }
@@ -779,94 +809,97 @@ find_devn(fz_context *ctx, fz_separations **seps, pdf_obj *obj)
 typedef void (res_finder_fn)(fz_context *ctx, fz_separations **seps, pdf_obj *obj);
 
 static void
-search_res(fz_context *ctx, fz_separations **seps, pdf_obj *res, res_finder_fn *fn)
+scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_fn *fn, pdf_obj *clearme)
 {
-	int i = 0;
-	int len = pdf_dict_len(ctx, res);
-
-	fz_var(i);
-
-	while (i < len)
-	{
-		fz_try(ctx)
-		{
-			do
-			{
-				fn(ctx, seps, pdf_dict_get_val(ctx, res, i++));
-			}
-			while (i < len);
-		}
-		fz_catch(ctx)
-		{
-			/* Don't die because a single separation failed to load */
-		}
-	}
-}
-
-static void
-scan_page_seps(fz_context *ctx, pdf_obj *res, fz_separations **seps, res_finder_fn *fn)
-{
-	pdf_obj *forms;
-	pdf_obj *sh;
-	pdf_obj *xo = NULL;
-	int i, len;
-
-	fz_var(xo);
+	pdf_obj *dict;
+	pdf_obj *obj;
+	int i, n;
 
 	if (pdf_mark_obj(ctx, res))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cycle in resources");
+		return; /* already been here */
 
-	fz_try(ctx)
+	/* remember to clear this resource dictionary at the end */
+	pdf_array_push(ctx, clearme, res);
+
+	dict = pdf_dict_get(ctx, res, PDF_NAME(ColorSpace));
+	n = pdf_dict_len(ctx, dict);
+	for (i = 0; i < n; i++)
 	{
-		search_res(ctx, seps, pdf_dict_get(ctx, res, PDF_NAME(ColorSpace)), fn);
-
-		sh = pdf_dict_get(ctx, res, PDF_NAME(Shading));
-		len = pdf_dict_len(ctx, sh);
-		for (i = 0; i < len; i++)
-			fn(ctx, seps, pdf_dict_get(ctx, pdf_dict_get_val(ctx, sh, i), PDF_NAME(ColorSpace)));
-
-		forms = pdf_dict_get(ctx, res, PDF_NAME(XObject));
-		len = pdf_dict_len(ctx, forms);
-
-		/* Recurse on the forms. Throw if we cycle */
-		for (i = 0; i < len; i++)
-		{
-			xo = pdf_dict_get_val(ctx, forms, i);
-
-			if (pdf_mark_obj(ctx, xo))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "cycle in forms");
-
-			scan_page_seps(ctx, pdf_dict_get(ctx, xo, PDF_NAME(Resources)), seps, fn);
-			fn(ctx, seps, pdf_dict_get(ctx, xo, PDF_NAME(ColorSpace)));
-			pdf_unmark_obj(ctx, xo);
-			xo = NULL;
-		}
+		obj = pdf_dict_get_val(ctx, dict, i);
+		fn(ctx, seps, obj);
 	}
-	fz_always(ctx)
+
+	dict = pdf_dict_get(ctx, res, PDF_NAME(Shading));
+	n = pdf_dict_len(ctx, dict);
+	for (i = 0; i < n; i++)
 	{
-		pdf_unmark_obj(ctx, xo);
-		pdf_unmark_obj(ctx, res);
+		obj = pdf_dict_get_val(ctx, dict, i);
+		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)));
+	}
+
+	dict = pdf_dict_get(ctx, res, PDF_NAME(XObject));
+	n = pdf_dict_len(ctx, dict);
+	for (i = 0; i < n; i++)
+	{
+		obj = pdf_dict_get_val(ctx, dict, i);
+		fn(ctx, seps, pdf_dict_get(ctx, obj, PDF_NAME(ColorSpace)));
+		/* Recurse on XObject forms. */
+		scan_page_seps(ctx, pdf_dict_get(ctx, obj, PDF_NAME(Resources)), seps, fn, clearme);
 	}
-	fz_catch(ctx)
-		fz_rethrow(ctx);
 }
 
+/*
+	Get the separation details for a page.
+*/
 fz_separations *
 pdf_page_separations(fz_context *ctx, pdf_page *page)
 {
 	pdf_obj *res = pdf_page_resources(ctx, page);
+	pdf_obj *clearme = NULL;
 	fz_separations *seps = NULL;
 
-	/* Run through and look for separations first. This is
-	 * because separations are simplest to deal with, and
-	 * because DeviceN may be implemented on top of separations.
-	 */
-	scan_page_seps(ctx, res, &seps, find_seps);
+	clearme = pdf_new_array(ctx, page->doc, 100);
+	fz_try(ctx)
+	{
+		/* Run through and look for separations first. This is
+		 * because separations are simplest to deal with, and
+		 * because DeviceN may be implemented on top of separations.
+		 */
+		scan_page_seps(ctx, res, &seps, find_seps, clearme);
+	}
+	fz_always(ctx)
+	{
+		int i, n = pdf_array_len(ctx, clearme);
+		for (i = 0; i < n; ++i)
+			pdf_unmark_obj(ctx, pdf_array_get(ctx, clearme, i));
+		pdf_drop_obj(ctx, clearme);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_separations(ctx, seps);
+		fz_rethrow(ctx);
+	}
 
-	/* Now run through again, and look for DeviceNs. These may
-	 * have spot colors in that aren't defined in terms of
-	 * separations. */
-	scan_page_seps(ctx, res, &seps, find_devn);
+	clearme = pdf_new_array(ctx, page->doc, 100);
+	fz_try(ctx)
+	{
+		/* Now run through again, and look for DeviceNs. These may
+		 * have spot colors in that aren't defined in terms of
+		 * separations. */
+		scan_page_seps(ctx, res, &seps, find_devn, clearme);
+	}
+	fz_always(ctx)
+	{
+		int i, n = pdf_array_len(ctx, clearme);
+		for (i = 0; i < n; ++i)
+			pdf_unmark_obj(ctx, pdf_array_get(ctx, clearme, i));
+		pdf_drop_obj(ctx, clearme);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_separations(ctx, seps);
+		fz_rethrow(ctx);
+	}
 
 	return seps;
 }
@@ -880,15 +913,9 @@ pdf_page_uses_overprint(fz_context *ctx, pdf_page *page)
 static void
 pdf_drop_page_imp(fz_context *ctx, pdf_page *page)
 {
-	pdf_document *doc = page->doc;
-
-	/* We are about to destroy the annotation records for this page and so,
-	 * if doc->focus refers to one of them, it must be NULLed */
-	if (doc->focus && doc->focus->page == page)
-		doc->focus = NULL;
-
 	fz_drop_link(ctx, page->links);
 	pdf_drop_annots(ctx, page->annots);
+	pdf_drop_widgets(ctx, page->widgets);
 
 	pdf_drop_obj(ctx, page->obj);
 
@@ -905,8 +932,9 @@ pdf_new_page(fz_context *ctx, pdf_document *doc)
 	page->super.drop_page = (fz_page_drop_page_fn*)pdf_drop_page_imp;
 	page->super.load_links = (fz_page_load_links_fn*)pdf_load_links;
 	page->super.bound_page = (fz_page_bound_page_fn*)pdf_bound_page;
-	page->super.first_annot = (fz_page_first_annot_fn*)pdf_first_annot;
-	page->super.run_page_contents = (fz_page_run_page_contents_fn*)pdf_run_page_contents;
+	page->super.run_page_contents = (fz_page_run_page_fn*)pdf_run_page_contents;
+	page->super.run_page_annots = (fz_page_run_page_fn*)pdf_run_page_annots;
+	page->super.run_page_widgets = (fz_page_run_page_fn*)pdf_run_page_widgets;
 	page->super.page_presentation = (fz_page_page_presentation_fn*)pdf_page_presentation;
 	page->super.separations = (fz_page_separations_fn *)pdf_page_separations;
 	page->super.overprint = (fz_page_uses_overprint_fn *)pdf_page_uses_overprint;
@@ -917,7 +945,8 @@ pdf_new_page(fz_context *ctx, pdf_document *doc)
 	page->links = NULL;
 	page->annots = NULL;
 	page->annot_tailp = &page->annots;
-	page->incomplete = 0;
+	page->widgets = NULL;
+	page->widget_tailp = &page->widgets;
 
 	return page;
 }
@@ -940,8 +969,7 @@ pdf_load_default_colorspaces_imp(fz_context *ctx, fz_default_colorspaces *defaul
 	}
 	fz_catch(ctx)
 	{
-		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
-			fz_warn(ctx, "Error while reading DefaultGray: %s", fz_caught_message(ctx));
+		fz_warn(ctx, "Error while reading DefaultGray: %s", fz_caught_message(ctx));
 	}
 
 	fz_try(ctx)
@@ -956,8 +984,7 @@ pdf_load_default_colorspaces_imp(fz_context *ctx, fz_default_colorspaces *defaul
 	}
 	fz_catch(ctx)
 	{
-		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
-			fz_warn(ctx, "Error while reading DefaultRGB: %s", fz_caught_message(ctx));
+		fz_warn(ctx, "Error while reading DefaultRGB: %s", fz_caught_message(ctx));
 	}
 
 	fz_try(ctx)
@@ -972,8 +999,7 @@ pdf_load_default_colorspaces_imp(fz_context *ctx, fz_default_colorspaces *defaul
 	}
 	fz_catch(ctx)
 	{
-		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
-			fz_warn(ctx, "Error while reading DefaultCMYK: %s", fz_caught_message(ctx));
+		fz_warn(ctx, "Error while reading DefaultCMYK: %s", fz_caught_message(ctx));
 	}
 }
 
@@ -986,18 +1012,30 @@ pdf_load_default_colorspaces(fz_context *ctx, pdf_document *doc, pdf_page *page)
 	fz_colorspace *oi;
 
 	default_cs = fz_new_default_colorspaces(ctx);
-	res = pdf_page_resources(ctx, page);
-	obj = pdf_dict_get(ctx, res, PDF_NAME(ColorSpace));
-	if (obj)
-		pdf_load_default_colorspaces_imp(ctx, default_cs, obj);
 
-	oi = pdf_document_output_intent(ctx, doc);
-	if (oi)
-		fz_set_default_output_intent(ctx, default_cs, oi);
+	fz_try(ctx)
+	{
+		res = pdf_page_resources(ctx, page);
+		obj = pdf_dict_get(ctx, res, PDF_NAME(ColorSpace));
+		if (obj)
+			pdf_load_default_colorspaces_imp(ctx, default_cs, obj);
+
+		oi = pdf_document_output_intent(ctx, doc);
+		if (oi)
+			fz_set_default_output_intent(ctx, default_cs, oi);
+	}
+	fz_catch(ctx)
+	{
+		fz_drop_default_colorspaces(ctx, default_cs);
+		fz_rethrow(ctx);
+	}
 
 	return default_cs;
 }
 
+/*
+	Update default colorspaces for an xobject.
+*/
 fz_default_colorspaces *
 pdf_update_default_colorspaces(fz_context *ctx, fz_default_colorspaces *old_cs, pdf_obj *res)
 {
@@ -1014,6 +1052,16 @@ pdf_update_default_colorspaces(fz_context *ctx, fz_default_colorspaces *old_cs,
 	return new_cs;
 }
 
+/*
+	Load a page and its resources.
+
+	Locates the page in the PDF document and loads the page and its
+	resources. After pdf_load_page is it possible to retrieve the size
+	of the page using pdf_bound_page, or to render the page using
+	pdf_run_page_*.
+
+	number: page number, where 0 is the first page of the document.
+*/
 pdf_page *
 pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 {
@@ -1021,14 +1069,7 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	pdf_annot *annot;
 	pdf_obj *pageobj, *obj;
 
-	if (doc->file_reading_linearly)
-	{
-		pageobj = pdf_progressive_advance(ctx, doc, number);
-		if (pageobj == NULL)
-			fz_throw(ctx, FZ_ERROR_TRYLATER, "page %d not available yet", number);
-	}
-	else
-		pageobj = pdf_lookup_page_obj(ctx, doc, number);
+	pageobj = pdf_lookup_page_obj(ctx, doc, number);
 
 	page = pdf_new_page(ctx, doc);
 	page->obj = pdf_keep_obj(ctx, pageobj);
@@ -1048,14 +1089,8 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	}
 	fz_catch(ctx)
 	{
-		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
-		{
-			fz_drop_page(ctx, &page->super);
-			fz_rethrow(ctx);
-		}
-		page->incomplete |= PDF_PAGE_INCOMPLETE_ANNOTS;
-		fz_drop_link(ctx, page->links);
-		page->links = NULL;
+		fz_drop_page(ctx, &page->super);
+		fz_rethrow(ctx);
 	}
 
 	/* Scan for transparency and overprint */
@@ -1077,17 +1112,22 @@ pdf_load_page(fz_context *ctx, pdf_document *doc, int number)
 	}
 	fz_catch(ctx)
 	{
-		if (fz_caught(ctx) != FZ_ERROR_TRYLATER)
-		{
-			fz_drop_page(ctx, &page->super);
-			fz_rethrow(ctx);
-		}
-		page->incomplete |= PDF_PAGE_INCOMPLETE_CONTENTS;
+		fz_drop_page(ctx, &page->super);
+		fz_rethrow(ctx);
 	}
 
 	return page;
 }
 
+/*
+	Delete a page from the page tree of
+	a document. This does not remove the page contents
+	or resources from the file.
+
+	doc: The document to operate on.
+
+	number: The page to remove (numbered from 0)
+*/
 void
 pdf_delete_page(fz_context *ctx, pdf_document *doc, int at)
 {
@@ -1106,6 +1146,18 @@ pdf_delete_page(fz_context *ctx, pdf_document *doc, int at)
 	}
 }
 
+/*
+	Delete a range of pages from the
+	page tree of a document. This does not remove the page
+	contents or resources from the file.
+
+	doc: The document to operate on.
+
+	start, end: The range of pages (numbered from 0)
+	(inclusive, exclusive) to remove. If end is negative or
+	greater than the number of pages in the document, it
+	will be taken to be the end of the document.
+*/
 void
 pdf_delete_page_range(fz_context *ctx, pdf_document *doc, int start, int end)
 {
@@ -1122,6 +1174,30 @@ pdf_delete_page_range(fz_context *ctx, pdf_document *doc, int start, int end)
 	}
 }
 
+/*
+	Create a pdf_obj within a document that
+	represents a page, from a previously created resources
+	dictionary and page content stream. This should then be
+	inserted into the document using pdf_insert_page.
+
+	After this call the page exists within the document
+	structure, but is not actually ever displayed as it is
+	not linked into the PDF page tree.
+
+	doc: The document to which to add the page.
+
+	mediabox: The mediabox for the page (should be identical
+	to that used when creating the resources/contents).
+
+	rotate: 0, 90, 180 or 270. The rotation to use for the
+	page.
+
+	resources: The resources dictionary for the new page
+	(typically created by pdf_page_write).
+
+	contents: The page contents for the new page (typically
+	create by pdf_page_write).
+*/
 pdf_obj *
 pdf_add_page(fz_context *ctx, pdf_document *doc, fz_rect mediabox, int rotate, pdf_obj *resources, fz_buffer *contents)
 {
@@ -1150,6 +1226,18 @@ pdf_add_page(fz_context *ctx, pdf_document *doc, fz_rect mediabox, int rotate, p
 	return pdf_add_object_drop(ctx, doc, page_obj);
 }
 
+/*
+	Insert a page previously created by
+	pdf_add_page into the pages tree of the document.
+
+	doc: The document to insert into.
+
+	at: The page number to insert at. 0 inserts at the start.
+	negative numbers, or INT_MAX insert at the end. Otherwise
+	n inserts after page n.
+
+	page: The page to insert.
+*/
 void
 pdf_insert_page(fz_context *ctx, pdf_document *doc, int at, pdf_obj *page_ref)
 {
