@@ -1,6 +1,5 @@
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
-#include "pdf-imp.h"
 
 #include <string.h>
 
@@ -57,7 +56,6 @@ pdf_repair_obj(fz_context *ctx, pdf_document *doc, pdf_lexbuf *buf, int64_t *stm
 		}
 		fz_catch(ctx)
 		{
-			fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 			/* Don't let a broken object at EOF overwrite a good one */
 			if (file->eof)
 				fz_rethrow(ctx);
@@ -105,16 +103,6 @@ pdf_repair_obj(fz_context *ctx, pdf_document *doc, pdf_lexbuf *buf, int64_t *stm
 		if (!pdf_is_indirect(ctx, obj) && pdf_is_int(ctx, obj))
 			stm_len = pdf_to_int(ctx, obj);
 
-		if (doc->file_reading_linearly && page)
-		{
-			obj = pdf_dict_get(ctx, dict, PDF_NAME(Type));
-			if (!pdf_is_indirect(ctx, obj) && pdf_name_eq(ctx, obj, PDF_NAME(Page)))
-			{
-				pdf_drop_obj(ctx, *page);
-				*page = pdf_keep_obj(ctx, dict);
-			}
-		}
-
 		pdf_drop_obj(ctx, dict);
 	}
 
@@ -152,7 +140,6 @@ pdf_repair_obj(fz_context *ctx, pdf_document *doc, pdf_lexbuf *buf, int64_t *stm
 			}
 			fz_catch(ctx)
 			{
-				fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 				fz_warn(ctx, "cannot find endstream token, falling back to scanning");
 			}
 			if (tok == PDF_TOK_ENDSTREAM)
@@ -378,12 +365,14 @@ pdf_repair_xref(fz_context *ctx, pdf_document *doc)
 				tok = pdf_lex_no_string(ctx, doc->file, buf);
 			fz_catch(ctx)
 			{
-				fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 				fz_warn(ctx, "skipping ahead to next token");
 				do
 					c = fz_read_byte(ctx, doc->file);
 				while (c != EOF && !is_white(c));
-				continue;
+				if (c == EOF)
+					tok = PDF_TOK_EOF;
+				else
+					continue;
 			}
 
 			/* If we have the next token already, then we'll jump
@@ -423,7 +412,6 @@ pdf_repair_xref(fz_context *ctx, pdf_document *doc)
 				}
 				fz_catch(ctx)
 				{
-					fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 					/* If we haven't seen a root yet, there is nothing
 					 * we can do, but give up. Otherwise, we'll make
 					 * do. */
@@ -465,13 +453,14 @@ pdf_repair_xref(fz_context *ctx, pdf_document *doc)
 			 * by a corrupt file. */
 			else if (tok == PDF_TOK_OPEN_DICT)
 			{
+				pdf_obj *dictobj;
+
 				fz_try(ctx)
 				{
 					dict = pdf_parse_dict(ctx, doc, doc->file, buf);
 				}
 				fz_catch(ctx)
 				{
-					fz_rethrow_if(ctx, FZ_ERROR_TRYLATER);
 					/* If this was the real trailer dict
 					 * it was broken, in which case we are
 					 * in trouble. Keep going though in
@@ -481,37 +470,35 @@ pdf_repair_xref(fz_context *ctx, pdf_document *doc)
 
 				fz_try(ctx)
 				{
-					obj = pdf_dict_get(ctx, dict, PDF_NAME(Encrypt));
-					if (obj)
+					dictobj = pdf_dict_get(ctx, dict, PDF_NAME(Encrypt));
+					if (dictobj)
 					{
 						pdf_drop_obj(ctx, encrypt);
-						encrypt = pdf_keep_obj(ctx, obj);
+						encrypt = pdf_keep_obj(ctx, dictobj);
 					}
 
-					obj = pdf_dict_get(ctx, dict, PDF_NAME(ID));
-					if (obj && (!id || !encrypt || pdf_dict_get(ctx, dict, PDF_NAME(Encrypt))))
+					dictobj = pdf_dict_get(ctx, dict, PDF_NAME(ID));
+					if (dictobj && (!id || !encrypt || pdf_dict_get(ctx, dict, PDF_NAME(Encrypt))))
 					{
 						pdf_drop_obj(ctx, id);
-						id = pdf_keep_obj(ctx, obj);
+						id = pdf_keep_obj(ctx, dictobj);
 					}
 
-					obj = pdf_dict_get(ctx, dict, PDF_NAME(Root));
-					if (obj)
-						add_root(ctx, obj, &roots, &num_roots, &max_roots);
+					dictobj = pdf_dict_get(ctx, dict, PDF_NAME(Root));
+					if (dictobj)
+						add_root(ctx, dictobj, &roots, &num_roots, &max_roots);
 
-					obj = pdf_dict_get(ctx, dict, PDF_NAME(Info));
-					if (obj)
+					dictobj = pdf_dict_get(ctx, dict, PDF_NAME(Info));
+					if (dictobj)
 					{
 						pdf_drop_obj(ctx, info);
-						info = pdf_keep_obj(ctx, obj);
+						info = pdf_keep_obj(ctx, dictobj);
 					}
 				}
 				fz_always(ctx)
 					pdf_drop_obj(ctx, dict);
 				fz_catch(ctx)
 					fz_rethrow(ctx);
-
-				obj = NULL;
 			}
 
 			else if (tok == PDF_TOK_EOF)
