@@ -14,17 +14,21 @@ pdf_obj_num_is_stream(fz_context *ctx, pdf_document *doc, int num)
 	if (num <= 0 || num >= pdf_xref_len(ctx, doc))
 		return 0;
 
-	entry = pdf_cache_object(ctx, doc, num);
+	fz_try(ctx)
+		entry = pdf_cache_object(ctx, doc, num);
+	fz_catch(ctx)
+		return 0;
 
 	return entry->stm_ofs != 0 || entry->stm_buf;
 }
 
 int
-pdf_is_stream(fz_context *ctx, pdf_obj *obj)
+pdf_is_stream(fz_context *ctx, pdf_obj *ref)
 {
-	pdf_document *doc = pdf_get_bound_document(ctx, obj);
-	int num = pdf_obj_parent_num(ctx, obj);
-	return pdf_obj_num_is_stream(ctx, doc, num);
+	pdf_document *doc = pdf_get_indirect_document(ctx, ref);
+	if (doc)
+		return pdf_obj_num_is_stream(ctx, doc, pdf_to_num(ctx, ref));
+	return 0;
 }
 
 /*
@@ -37,10 +41,10 @@ pdf_stream_has_crypt(fz_context *ctx, pdf_obj *stm)
 	pdf_obj *obj;
 	int i;
 
-	filters = pdf_dict_geta(ctx, stm, PDF_NAME_Filter, PDF_NAME_F);
+	filters = pdf_dict_geta(ctx, stm, PDF_NAME(Filter), PDF_NAME(F));
 	if (filters)
 	{
-		if (pdf_name_eq(ctx, filters, PDF_NAME_Crypt))
+		if (pdf_name_eq(ctx, filters, PDF_NAME(Crypt)))
 			return 1;
 		if (pdf_is_array(ctx, filters))
 		{
@@ -48,7 +52,7 @@ pdf_stream_has_crypt(fz_context *ctx, pdf_obj *stm)
 			for (i = 0; i < n; i++)
 			{
 				obj = pdf_array_get(ctx, filters, i);
-				if (pdf_name_eq(ctx, obj, PDF_NAME_Crypt))
+				if (pdf_name_eq(ctx, obj, PDF_NAME(Crypt)))
 					return 1;
 			}
 		}
@@ -57,7 +61,7 @@ pdf_stream_has_crypt(fz_context *ctx, pdf_obj *stm)
 }
 
 static fz_jbig2_globals *
-pdf_load_jbig2_globals(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
+pdf_load_jbig2_globals(fz_context *ctx, pdf_obj *dict)
 {
 	fz_jbig2_globals *globals;
 	fz_buffer *buf = NULL;
@@ -65,9 +69,10 @@ pdf_load_jbig2_globals(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
 	fz_var(buf);
 
 	if ((globals = pdf_find_item(ctx, fz_drop_jbig2_globals_imp, dict)) != NULL)
-	{
 		return globals;
-	}
+
+	if (pdf_mark_obj(ctx, dict))
+		fz_throw(ctx, FZ_ERROR_GENERIC, "cyclic reference when loading JBIG2 globals");
 
 	fz_try(ctx)
 	{
@@ -78,6 +83,7 @@ pdf_load_jbig2_globals(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
 	fz_always(ctx)
 	{
 		fz_drop_buffer(ctx, buf);
+		pdf_unmark_obj(ctx, dict);
 	}
 	fz_catch(ctx)
 	{
@@ -90,22 +96,22 @@ pdf_load_jbig2_globals(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
 static void
 build_compression_params(fz_context *ctx, pdf_obj *f, pdf_obj *p, fz_compression_params *params)
 {
-	int predictor = pdf_to_int(ctx, pdf_dict_get(ctx, p, PDF_NAME_Predictor));
-	pdf_obj *columns_obj = pdf_dict_get(ctx, p, PDF_NAME_Columns);
+	int predictor = pdf_dict_get_int(ctx, p, PDF_NAME(Predictor));
+	pdf_obj *columns_obj = pdf_dict_get(ctx, p, PDF_NAME(Columns));
 	int columns = pdf_to_int(ctx, columns_obj);
-	int colors = pdf_to_int(ctx, pdf_dict_get(ctx, p, PDF_NAME_Colors));
-	int bpc = pdf_to_int(ctx, pdf_dict_get(ctx, p, PDF_NAME_BitsPerComponent));
+	int colors = pdf_dict_get_int(ctx, p, PDF_NAME(Colors));
+	int bpc = pdf_dict_get_int(ctx, p, PDF_NAME(BitsPerComponent));
 
 	params->type = FZ_IMAGE_RAW;
 
-	if (pdf_name_eq(ctx, f, PDF_NAME_CCITTFaxDecode) || pdf_name_eq(ctx, f, PDF_NAME_CCF))
+	if (pdf_name_eq(ctx, f, PDF_NAME(CCITTFaxDecode)) || pdf_name_eq(ctx, f, PDF_NAME(CCF)))
 	{
-		pdf_obj *k = pdf_dict_get(ctx, p, PDF_NAME_K);
-		pdf_obj *eol = pdf_dict_get(ctx, p, PDF_NAME_EndOfLine);
-		pdf_obj *eba = pdf_dict_get(ctx, p, PDF_NAME_EncodedByteAlign);
-		pdf_obj *rows = pdf_dict_get(ctx, p, PDF_NAME_Rows);
-		pdf_obj *eob = pdf_dict_get(ctx, p, PDF_NAME_EndOfBlock);
-		pdf_obj *bi1 = pdf_dict_get(ctx, p, PDF_NAME_BlackIs1);
+		pdf_obj *k = pdf_dict_get(ctx, p, PDF_NAME(K));
+		pdf_obj *eol = pdf_dict_get(ctx, p, PDF_NAME(EndOfLine));
+		pdf_obj *eba = pdf_dict_get(ctx, p, PDF_NAME(EncodedByteAlign));
+		pdf_obj *rows = pdf_dict_get(ctx, p, PDF_NAME(Rows));
+		pdf_obj *eob = pdf_dict_get(ctx, p, PDF_NAME(EndOfBlock));
+		pdf_obj *bi1 = pdf_dict_get(ctx, p, PDF_NAME(BlackIs1));
 
 		params->type = FZ_IMAGE_FAX;
 		params->u.fax.k = (k ? pdf_to_int(ctx, k) : 0);
@@ -116,18 +122,18 @@ build_compression_params(fz_context *ctx, pdf_obj *f, pdf_obj *p, fz_compression
 		params->u.fax.end_of_block = (eob ? pdf_to_bool(ctx, eob) : 1);
 		params->u.fax.black_is_1 = (bi1 ? pdf_to_bool(ctx, bi1) : 0);
 	}
-	else if (pdf_name_eq(ctx, f, PDF_NAME_DCTDecode) || pdf_name_eq(ctx, f, PDF_NAME_DCT))
+	else if (pdf_name_eq(ctx, f, PDF_NAME(DCTDecode)) || pdf_name_eq(ctx, f, PDF_NAME(DCT)))
 	{
-		pdf_obj *ct = pdf_dict_get(ctx, p, PDF_NAME_ColorTransform);
+		pdf_obj *ct = pdf_dict_get(ctx, p, PDF_NAME(ColorTransform));
 
 		params->type = FZ_IMAGE_JPEG;
 		params->u.jpeg.color_transform = (ct ? pdf_to_int(ctx, ct) : -1);
 	}
-	else if (pdf_name_eq(ctx, f, PDF_NAME_RunLengthDecode) || pdf_name_eq(ctx, f, PDF_NAME_RL))
+	else if (pdf_name_eq(ctx, f, PDF_NAME(RunLengthDecode)) || pdf_name_eq(ctx, f, PDF_NAME(RL)))
 	{
 		params->type = FZ_IMAGE_RLD;
 	}
-	else if (pdf_name_eq(ctx, f, PDF_NAME_FlateDecode) || pdf_name_eq(ctx, f, PDF_NAME_Fl))
+	else if (pdf_name_eq(ctx, f, PDF_NAME(FlateDecode)) || pdf_name_eq(ctx, f, PDF_NAME(Fl)))
 	{
 		params->type = FZ_IMAGE_FLATE;
 		params->u.flate.predictor = predictor;
@@ -135,9 +141,9 @@ build_compression_params(fz_context *ctx, pdf_obj *f, pdf_obj *p, fz_compression
 		params->u.flate.colors = colors;
 		params->u.flate.bpc = bpc;
 	}
-	else if (pdf_name_eq(ctx, f, PDF_NAME_LZWDecode) || pdf_name_eq(ctx, f, PDF_NAME_LZW))
+	else if (pdf_name_eq(ctx, f, PDF_NAME(LZWDecode)) || pdf_name_eq(ctx, f, PDF_NAME(LZW)))
 	{
-		pdf_obj *ec = pdf_dict_get(ctx, p, PDF_NAME_EarlyChange);
+		pdf_obj *ec = pdf_dict_get(ctx, p, PDF_NAME(EarlyChange));
 
 		params->type = FZ_IMAGE_LZW;
 		params->u.lzw.predictor = predictor;
@@ -146,6 +152,20 @@ build_compression_params(fz_context *ctx, pdf_obj *f, pdf_obj *p, fz_compression
 		params->u.lzw.bpc = bpc;
 		params->u.lzw.early_change = (ec ? pdf_to_int(ctx, ec) : 1);
 	}
+	else if (pdf_name_eq(ctx, f, PDF_NAME(JBIG2Decode)))
+	{
+		pdf_obj *g = pdf_dict_get(ctx, p, PDF_NAME(JBIG2Globals));
+
+		params->type = FZ_IMAGE_JBIG2;
+		params->u.jbig2.globals = NULL;
+		if (g)
+		{
+			if (!pdf_is_stream(ctx, g))
+				fz_warn(ctx, "jbig2 globals is not a stream, skipping globals");
+			else
+				params->u.jbig2.globals = pdf_load_jbig2_globals(ctx, g);
+		}
+	}
 }
 
 /*
@@ -155,122 +175,105 @@ static fz_stream *
 build_filter(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj *f, pdf_obj *p, int num, int gen, fz_compression_params *params)
 {
 	fz_compression_params local_params;
-	fz_stream *tmp;
 
-	fz_var(chain);
+	local_params.u.jbig2.globals = NULL;
+	if (params == NULL)
+		params = &local_params;
 
-	fz_try(ctx)
+	build_compression_params(ctx, f, p, params);
+
+	/* If we were using params we were passed in, and we successfully
+	 * recognised the image type, we can use the existing filter and
+	 * shortstop here. */
+	if (params != &local_params && params->type != FZ_IMAGE_RAW)
+		return fz_keep_stream(ctx, chain); /* nothing to do */
+
+	else if (params->type == FZ_IMAGE_JBIG2)
 	{
-		if (params == NULL)
-			params = &local_params;
+		fz_stream *stm;
+		fz_try(ctx)
+			stm = fz_open_image_decomp_stream(ctx, chain, params, NULL);
+		fz_always(ctx)
+			fz_drop_jbig2_globals(ctx, local_params.u.jbig2.globals);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
+		return stm;
+	}
 
-		build_compression_params(ctx, f, p, params);
+	else if (params->type != FZ_IMAGE_RAW)
+		return fz_open_image_decomp_stream(ctx, chain, params, NULL);
 
-		/* If we were using params we were passed in, and we successfully
-		 * recognised the image type, we can use the existing filter and
-		 * shortstop here. */
-		if (params != &local_params && params->type != FZ_IMAGE_RAW)
-			break; /* nothing to do */
+	else if (pdf_name_eq(ctx, f, PDF_NAME(ASCIIHexDecode)) || pdf_name_eq(ctx, f, PDF_NAME(AHx)))
+		return fz_open_ahxd(ctx, chain);
 
-		else if (params->type != FZ_IMAGE_RAW)
-		{
-			tmp = chain;
-			chain = NULL;
-			chain = fz_open_image_decomp_stream(ctx, tmp, params, NULL);
-		}
+	else if (pdf_name_eq(ctx, f, PDF_NAME(ASCII85Decode)) || pdf_name_eq(ctx, f, PDF_NAME(A85)))
+		return fz_open_a85d(ctx, chain);
 
-		else if (pdf_name_eq(ctx, f, PDF_NAME_ASCIIHexDecode) || pdf_name_eq(ctx, f, PDF_NAME_AHx))
-		{
-			tmp = chain;
-			chain = NULL;
-			chain = fz_open_ahxd(ctx, tmp);
-		}
+	else if (pdf_name_eq(ctx, f, PDF_NAME(JPXDecode)))
+		return fz_keep_stream(ctx, chain); /* JPX decoding is special cased in the image loading code */
 
-		else if (pdf_name_eq(ctx, f, PDF_NAME_ASCII85Decode) || pdf_name_eq(ctx, f, PDF_NAME_A85))
-		{
-			tmp = chain;
-			chain = NULL;
-			chain = fz_open_a85d(ctx, tmp);
-		}
-
-		else if (pdf_name_eq(ctx, f, PDF_NAME_JBIG2Decode))
-		{
-			fz_jbig2_globals *globals = NULL;
-			pdf_obj *obj = pdf_dict_get(ctx, p, PDF_NAME_JBIG2Globals);
-			if (pdf_is_indirect(ctx, obj))
-				globals = pdf_load_jbig2_globals(ctx, doc, obj);
-			tmp = chain;
-			chain = NULL;
-			chain = fz_open_jbig2d(ctx, tmp, globals);
-		}
-
-		else if (pdf_name_eq(ctx, f, PDF_NAME_JPXDecode))
-			break; /* JPX decoding is special cased in the image loading code */
-
-		else if (pdf_name_eq(ctx, f, PDF_NAME_Crypt))
-		{
-			if (!doc->crypt)
-				fz_warn(ctx, "crypt filter in unencrypted document");
-			else
-			{
-				pdf_obj *name = pdf_dict_get(ctx, p, PDF_NAME_Name);
-				if (pdf_is_name(ctx, name))
-				{
-					tmp = chain;
-					chain = NULL;
-					chain = pdf_open_crypt_with_filter(ctx, tmp, doc->crypt, name, num, gen);
-				}
-			}
-		}
+	else if (pdf_name_eq(ctx, f, PDF_NAME(Crypt)))
+	{
+		if (!doc->crypt)
+			fz_warn(ctx, "crypt filter in unencrypted document");
 		else
-			fz_warn(ctx, "unknown filter name (%s)", pdf_to_name(ctx, f));
-	}
-	fz_catch(ctx)
-	{
-		fz_drop_stream(ctx, chain);
-		fz_rethrow(ctx);
+		{
+			pdf_obj *name = pdf_dict_get(ctx, p, PDF_NAME(Name));
+			if (pdf_is_name(ctx, name))
+				return pdf_open_crypt_with_filter(ctx, chain, doc->crypt, name, num, gen);
+		}
 	}
 
-	return chain;
+	else
+		fz_warn(ctx, "unknown filter name (%s)", pdf_to_name(ctx, f));
+
+	return fz_keep_stream(ctx, chain);
+}
+
+/* Build filter, and assume ownership of chain */
+static fz_stream *
+build_filter_drop(fz_context *ctx, fz_stream *tail, pdf_document *doc, pdf_obj *f, pdf_obj *p, int num, int gen, fz_compression_params *params)
+{
+	fz_stream *head;
+	fz_try(ctx)
+		head = build_filter(ctx, tail, doc, f, p, num, gen, params);
+	fz_always(ctx)
+		fz_drop_stream(ctx, tail);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return head;
 }
 
 /*
  * Build a chain of filters given filter names and param dicts.
- * If head is given, start filter chain with it.
- * Assume ownership of head.
+ * If chain is given, start filter chain with it.
+ * Assume ownership of chain.
  */
 static fz_stream *
-build_filter_chain(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj *fs, pdf_obj *ps, int num, int gen, fz_compression_params *params)
+build_filter_chain_drop(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj *fs, pdf_obj *ps, int num, int gen, fz_compression_params *params)
 {
-	pdf_obj *f;
-	pdf_obj *p;
-	int i, n;
-
 	fz_var(chain);
-
 	fz_try(ctx)
 	{
-		n = pdf_array_len(ctx, fs);
+		int i, n = pdf_array_len(ctx, fs);
 		for (i = 0; i < n; i++)
 		{
-			fz_stream *chain2;
-
-			f = pdf_array_get(ctx, fs, i);
-			p = pdf_array_get(ctx, ps, i);
-			chain2 = chain;
-			chain = NULL;
-			chain = build_filter(ctx, chain2, doc, f, p, num, gen, (i == n-1 ? params : NULL));
+			pdf_obj *f = pdf_array_get(ctx, fs, i);
+			pdf_obj *p = pdf_array_get(ctx, ps, i);
+			chain = build_filter_drop(ctx, chain, doc, f, p, num, gen, (i == n-1 ? params : NULL));
 		}
 	}
 	fz_catch(ctx)
-	{
-		fz_drop_stream(ctx, chain);
 		fz_rethrow(ctx);
-	}
-
 	return chain;
 }
 
+static fz_stream *
+build_filter_chain(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj *fs, pdf_obj *ps, int num, int gen, fz_compression_params *params)
+{
+	return build_filter_chain_drop(ctx, fz_keep_stream(ctx, chain), doc, fs, ps, num, gen, params);
+}
+
 /*
  * Build a filter for reading raw stream data.
  * This is a null filter to constrain reading to the stream length (and to
@@ -280,10 +283,10 @@ build_filter_chain(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj
  * orig_num and orig_gen are used purely to seed the encryption.
  */
 static fz_stream *
-pdf_open_raw_filter(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_obj *stmobj, int num, int *orig_num, int *orig_gen, int64_t offset)
+pdf_open_raw_filter(fz_context *ctx, fz_stream *file_stm, pdf_document *doc, pdf_obj *stmobj, int num, int *orig_num, int *orig_gen, int64_t offset)
 {
 	pdf_xref_entry *x = NULL;
-	fz_stream *chain2;
+	fz_stream *null_stm, *crypt_stm;
 	int hascrypt;
 	int len;
 
@@ -303,32 +306,20 @@ pdf_open_raw_filter(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_ob
 		*orig_gen = 0;
 	}
 
-	fz_var(chain);
-
-	fz_try(ctx)
+	hascrypt = pdf_stream_has_crypt(ctx, stmobj);
+	len = pdf_dict_get_int(ctx, stmobj, PDF_NAME(Length));
+	null_stm = fz_open_endstream_filter(ctx, file_stm, len, offset);
+	if (doc->crypt && !hascrypt)
 	{
-		len = pdf_to_int(ctx, pdf_dict_get(ctx, stmobj, PDF_NAME_Length));
-
-		/* don't close chain when we close this filter */
-		chain2 = fz_keep_stream(ctx, chain);
-		chain = NULL;
-		chain = fz_open_null(ctx, chain2, len, offset);
-
-		hascrypt = pdf_stream_has_crypt(ctx, stmobj);
-		if (doc->crypt && !hascrypt)
-		{
-			chain2 = chain;
-			chain = NULL;
-			chain = pdf_open_crypt(ctx, chain2, doc->crypt, *orig_num, *orig_gen);
-		}
+		fz_try(ctx)
+			crypt_stm = pdf_open_crypt(ctx, null_stm, doc->crypt, *orig_num, *orig_gen);
+		fz_always(ctx)
+			fz_drop_stream(ctx, null_stm);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
+		return crypt_stm;
 	}
-	fz_catch(ctx)
-	{
-		fz_drop_stream(ctx, chain);
-		fz_rethrow(ctx);
-	}
-
-	return chain;
+	return null_stm;
 }
 
 /*
@@ -336,41 +327,29 @@ pdf_open_raw_filter(fz_context *ctx, fz_stream *chain, pdf_document *doc, pdf_ob
  * to stream length and decrypting.
  */
 static fz_stream *
-pdf_open_filter(fz_context *ctx, pdf_document *doc, fz_stream *chain, pdf_obj *stmobj, int num, int64_t offset, fz_compression_params *imparams)
+pdf_open_filter(fz_context *ctx, pdf_document *doc, fz_stream *file_stm, pdf_obj *stmobj, int num, int64_t offset, fz_compression_params *imparams)
 {
-	pdf_obj *filters;
-	pdf_obj *params;
+	pdf_obj *filters = pdf_dict_geta(ctx, stmobj, PDF_NAME(Filter), PDF_NAME(F));
+	pdf_obj *params = pdf_dict_geta(ctx, stmobj, PDF_NAME(DecodeParms), PDF_NAME(DP));
 	int orig_num, orig_gen;
+	fz_stream *rstm, *fstm;
 
-	filters = pdf_dict_geta(ctx, stmobj, PDF_NAME_Filter, PDF_NAME_F);
-	params = pdf_dict_geta(ctx, stmobj, PDF_NAME_DecodeParms, PDF_NAME_DP);
-
-	chain = pdf_open_raw_filter(ctx, chain, doc, stmobj, num, &orig_num, &orig_gen, offset);
-
-	fz_var(chain);
-
+	rstm = pdf_open_raw_filter(ctx, file_stm, doc, stmobj, num, &orig_num, &orig_gen, offset);
 	fz_try(ctx)
 	{
 		if (pdf_is_name(ctx, filters))
-		{
-			fz_stream *chain2 = chain;
-			chain = NULL;
-			chain = build_filter(ctx, chain2, doc, filters, params, orig_num, orig_gen, imparams);
-		}
+			fstm = build_filter(ctx, rstm, doc, filters, params, orig_num, orig_gen, imparams);
 		else if (pdf_array_len(ctx, filters) > 0)
-		{
-			fz_stream *chain2 = chain;
-			chain = NULL;
-			chain = build_filter_chain(ctx, chain2, doc, filters, params, orig_num, orig_gen, imparams);
-		}
+			fstm = build_filter_chain(ctx, rstm, doc, filters, params, orig_num, orig_gen, imparams);
+		else
+			fstm = fz_keep_stream(ctx, rstm);
 	}
+	fz_always(ctx)
+		fz_drop_stream(ctx, rstm);
 	fz_catch(ctx)
-	{
-		fz_drop_stream(ctx, chain);
 		fz_rethrow(ctx);
-	}
 
-	return chain;
+	return fstm;
 }
 
 /*
@@ -378,54 +357,52 @@ pdf_open_filter(fz_context *ctx, pdf_document *doc, fz_stream *chain, pdf_obj *s
  * constraining to stream length, and without decryption.
  */
 fz_stream *
-pdf_open_inline_stream(fz_context *ctx, pdf_document *doc, pdf_obj *stmobj, int length, fz_stream *chain, fz_compression_params *imparams)
+pdf_open_inline_stream(fz_context *ctx, pdf_document *doc, pdf_obj *stmobj, int length, fz_stream *file_stm, fz_compression_params *imparams)
 {
-	pdf_obj *filters;
-	pdf_obj *params;
-	int64_t offset;
-
-	filters = pdf_dict_geta(ctx, stmobj, PDF_NAME_Filter, PDF_NAME_F);
-	params = pdf_dict_geta(ctx, stmobj, PDF_NAME_DecodeParms, PDF_NAME_DP);
-
-	/* don't close chain when we close this filter */
-	fz_keep_stream(ctx, chain);
+	pdf_obj *filters = pdf_dict_geta(ctx, stmobj, PDF_NAME(Filter), PDF_NAME(F));
+	pdf_obj *params = pdf_dict_geta(ctx, stmobj, PDF_NAME(DecodeParms), PDF_NAME(DP));
 
 	if (pdf_is_name(ctx, filters))
-		return build_filter(ctx, chain, doc, filters, params, 0, 0, imparams);
-	if (pdf_array_len(ctx, filters) > 0)
-		return build_filter_chain(ctx, chain, doc, filters, params, 0, 0, imparams);
+		return build_filter(ctx, file_stm, doc, filters, params, 0, 0, imparams);
+	else if (pdf_array_len(ctx, filters) > 0)
+		return build_filter_chain(ctx, file_stm, doc, filters, params, 0, 0, imparams);
 
 	if (imparams)
 		imparams->type = FZ_IMAGE_RAW;
-
-	fz_try(ctx)
-		offset = fz_tell(ctx, chain);
-	fz_catch(ctx)
-	{
-		fz_drop_stream(ctx, chain);
-		fz_rethrow(ctx);
-	}
-
-	return fz_open_null(ctx, chain, length, offset);
+	return fz_open_null_filter(ctx, file_stm, length, fz_tell(ctx, file_stm));
 }
 
 void
-pdf_load_compressed_inline_image(fz_context *ctx, pdf_document *doc, pdf_obj *dict, int length, fz_stream *stm, int indexed, fz_compressed_image *image)
+pdf_load_compressed_inline_image(fz_context *ctx, pdf_document *doc, pdf_obj *dict, int length, fz_stream *file_stm, int indexed, fz_compressed_image *image)
 {
-	fz_compressed_buffer *bc = fz_malloc_struct(ctx, fz_compressed_buffer);
+	fz_stream *istm = NULL, *leech = NULL, *decomp = NULL;
+	fz_pixmap *pixmap = NULL;
+	fz_compressed_buffer *bc;
+	int dummy_l2factor = 0;
 
+	fz_var(istm);
+	fz_var(leech);
+	fz_var(decomp);
+	fz_var(pixmap);
+
+	bc = fz_malloc_struct(ctx, fz_compressed_buffer);
 	fz_try(ctx)
 	{
-		int dummy_l2factor = 0;
 		bc->buffer = fz_new_buffer(ctx, 1024);
-
-		stm = pdf_open_inline_stream(ctx, doc, dict, length, stm, &bc->params);
-		stm = fz_open_leecher(ctx, stm, bc->buffer);
-		stm = fz_open_image_decomp_stream(ctx, stm, &bc->params, &dummy_l2factor);
-
-		fz_set_compressed_image_tile(ctx, image, fz_decomp_image_from_stream(ctx, stm, image, NULL, indexed, 0));
+		istm = pdf_open_inline_stream(ctx, doc, dict, length, file_stm, &bc->params);
+		leech = fz_open_leecher(ctx, istm, bc->buffer);
+		decomp = fz_open_image_decomp_stream(ctx, leech, &bc->params, &dummy_l2factor);
+		pixmap = fz_decomp_image_from_stream(ctx, decomp, image, NULL, indexed, 0);
+		fz_set_compressed_image_tile(ctx, image, pixmap);
 		fz_set_compressed_image_buffer(ctx, image, bc);
 	}
+	fz_always(ctx)
+	{
+		fz_drop_stream(ctx, istm);
+		fz_drop_stream(ctx, leech);
+		fz_drop_stream(ctx, decomp);
+		fz_drop_pixmap(ctx, pixmap);
+	}
 	fz_catch(ctx)
 	{
 		fz_drop_compressed_buffer(ctx, bc);
@@ -507,9 +484,12 @@ pdf_load_raw_stream_number(fz_context *ctx, pdf_document *doc, int num)
 
 	dict = pdf_load_object(ctx, doc, num);
 
-	len = pdf_to_int(ctx, pdf_dict_get(ctx, dict, PDF_NAME_Length));
-
-	pdf_drop_obj(ctx, dict);
+	fz_try(ctx)
+		len = pdf_dict_get_int(ctx, dict, PDF_NAME(Length));
+	fz_always(ctx)
+		pdf_drop_obj(ctx, dict);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 
 	stm = pdf_open_raw_stream_number(ctx, doc, num);
 
@@ -558,12 +538,12 @@ can_reuse_buffer(fz_context *ctx, pdf_xref_entry *entry, fz_compression_params *
 	if (params)
 		params->type = FZ_IMAGE_RAW;
 
-	f = pdf_dict_geta(ctx, entry->obj, PDF_NAME_Filter, PDF_NAME_F);
+	f = pdf_dict_geta(ctx, entry->obj, PDF_NAME(Filter), PDF_NAME(F));
 	/* If there are no filters, it's uncompressed, and we can use it */
 	if (!f)
 		return 1;
 
-	p = pdf_dict_geta(ctx, entry->obj, PDF_NAME_DecodeParms, PDF_NAME_DP);
+	p = pdf_dict_geta(ctx, entry->obj, PDF_NAME(DecodeParms), PDF_NAME(DP));
 	if (pdf_is_array(ctx, f))
 	{
 		int len = pdf_array_len(ctx, f);
@@ -611,15 +591,23 @@ pdf_load_image_stream(fz_context *ctx, pdf_document *doc, int num, fz_compressio
 	}
 
 	dict = pdf_load_object(ctx, doc, num);
-
-	len = pdf_to_int(ctx, pdf_dict_get(ctx, dict, PDF_NAME_Length));
-	obj = pdf_dict_get(ctx, dict, PDF_NAME_Filter);
-	len = pdf_guess_filter_length(len, pdf_to_name(ctx, obj));
-	n = pdf_array_len(ctx, obj);
-	for (i = 0; i < n; i++)
-		len = pdf_guess_filter_length(len, pdf_to_name(ctx, pdf_array_get(ctx, obj, i)));
-
-	pdf_drop_obj(ctx, dict);
+	fz_try(ctx)
+	{
+		len = pdf_dict_get_int(ctx, dict, PDF_NAME(Length));
+		obj = pdf_dict_get(ctx, dict, PDF_NAME(Filter));
+		len = pdf_guess_filter_length(len, pdf_to_name(ctx, obj));
+		n = pdf_array_len(ctx, obj);
+		for (i = 0; i < n; i++)
+			len = pdf_guess_filter_length(len, pdf_to_name(ctx, pdf_array_get(ctx, obj, i)));
+	}
+	fz_always(ctx)
+	{
+		pdf_drop_obj(ctx, dict);
+	}
+	fz_catch(ctx)
+	{
+		fz_rethrow(ctx);
+	}
 
 	stm = pdf_open_image_stream(ctx, doc, num, params);
 
@@ -651,12 +639,6 @@ pdf_load_stream_number(fz_context *ctx, pdf_document *doc, int num)
 	return pdf_load_image_stream(ctx, doc, num, NULL, NULL);
 }
 
-fz_buffer *
-pdf_load_stream_truncated(fz_context *ctx, pdf_document *doc, int num, int *truncated)
-{
-	return pdf_load_image_stream(ctx, doc, num, NULL, truncated);
-}
-
 fz_compressed_buffer *
 pdf_load_compressed_stream(fz_context *ctx, pdf_document *doc, int num)
 {
@@ -689,14 +671,7 @@ pdf_open_object_array(fz_context *ctx, pdf_document *doc, pdf_obj *list)
 		fz_try(ctx)
 			fz_concat_push_drop(ctx, stm, pdf_open_stream(ctx, obj));
 		fz_catch(ctx)
-		{
-			if (fz_caught(ctx) == FZ_ERROR_TRYLATER)
-			{
-				fz_drop_stream(ctx, stm);
-				fz_rethrow(ctx);
-			}
 			fz_warn(ctx, "cannot load content stream part %d/%d", i + 1, n);
-		}
 	}
 
 	return stm;
@@ -714,7 +689,8 @@ pdf_open_contents_stream(fz_context *ctx, pdf_document *doc, pdf_obj *obj)
 	if (pdf_is_stream(ctx, obj))
 		return pdf_open_image_stream(ctx, doc, num, NULL);
 
-	fz_throw(ctx, FZ_ERROR_GENERIC, "pdf object stream missing (%d 0 R)", num);
+	fz_warn(ctx, "content stream is not a stream (%d 0 R)", num);
+	return fz_open_memory(ctx, (unsigned char *)"", 0);
 }
 
 fz_buffer *pdf_load_raw_stream(fz_context *ctx, pdf_obj *ref)
