@@ -35,7 +35,7 @@ int gettimeofday(struct timeval *tv, struct timezone *tz);
 
 enum {
 	OUT_NONE,
-	OUT_PNG, OUT_TGA, OUT_PNM, OUT_PGM, OUT_PPM, OUT_PAM,
+	OUT_PNG, OUT_PNM, OUT_PGM, OUT_PPM, OUT_PAM,
 	OUT_PBM, OUT_PKM, OUT_PWG, OUT_PCL, OUT_PS, OUT_PSD,
 	OUT_TEXT, OUT_HTML, OUT_XHTML, OUT_STEXT, OUT_PCLM,
 	OUT_TRACE, OUT_SVG,
@@ -74,7 +74,6 @@ static const suffix_t suffix_table[] =
 #endif
 	{ ".psd", OUT_PSD, 1 },
 	{ ".ps", OUT_PS, 0 },
-	{ ".tga", OUT_TGA, 0 },
 
 	{ ".txt", OUT_TEXT, 0 },
 	{ ".text", OUT_TEXT, 0 },
@@ -131,7 +130,6 @@ static const format_cs_table_t format_cs_table[] =
 	{ OUT_PCLM, CS_RGB, { CS_RGB, CS_GRAY } },
 	{ OUT_PS, CS_RGB, { CS_GRAY, CS_RGB, CS_CMYK } },
 	{ OUT_PSD, CS_CMYK, { CS_GRAY, CS_GRAY_ALPHA, CS_RGB, CS_RGB_ALPHA, CS_CMYK, CS_CMYK_ALPHA, CS_ICC } },
-	{ OUT_TGA, CS_RGB, { CS_GRAY, CS_GRAY_ALPHA, CS_RGB, CS_RGB_ALPHA } },
 
 	{ OUT_TRACE, CS_RGB, { CS_RGB } },
 	{ OUT_SVG, CS_RGB, { CS_RGB } },
@@ -217,7 +215,6 @@ typedef struct worker_t {
 static char *output = NULL;
 static fz_output *out = NULL;
 static int output_pagenum = 0;
-static int output_append = 0;
 static int output_file_per_page = 0;
 
 static char *format = NULL;
@@ -260,10 +257,11 @@ static int invert = 0;
 static int band_height = 0;
 static int lowmemory = 0;
 
+static int quiet = 0;
 static int errored = 0;
-static fz_colorspace *colorspace;
+static fz_colorspace *colorspace = NULL;
 static fz_colorspace *oi = NULL;
-#ifdef FZ_ENABLE_SPOT_RENDERING
+#if FZ_ENABLE_SPOT_RENDERING
 static int spots = SPOTS_OVERPRINT_SIM;
 #else
 static int spots = SPOTS_NONE;
@@ -275,10 +273,10 @@ static int num_workers = 0;
 static worker_t *workers;
 static fz_band_writer *bander = NULL;
 
-#ifdef NO_ICC
-static fz_cmm_engine *icc_engine = NULL;
-#else
+#if FZ_ENABLE_ICC
 static fz_cmm_engine *icc_engine = &fz_cmm_engine_lcms;
+#else
+static fz_cmm_engine *icc_engine = NULL;
 #endif
 
 static const char *layer_config = NULL;
@@ -318,10 +316,11 @@ static void usage(void)
 		"\n"
 		"\t-o -\toutput file name (%%d for page number)\n"
 		"\t-F -\toutput format (default inferred from output file name)\n"
-		"\t\traster: png, tga, pnm, pam, pbm, pkm, pwg, pcl, ps\n"
+		"\t\traster: png, pnm, pam, pbm, pkm, pwg, pcl, ps\n"
 		"\t\tvector: svg, pdf, trace\n"
 		"\t\ttext: txt, html, stext\n"
 		"\n"
+		"\t-q\tbe quiet (don't print progress messages)\n"
 		"\t-s -\tshow extra information:\n"
 		"\t\tm - show memory use\n"
 		"\t\tt - show timings\n"
@@ -364,8 +363,7 @@ static void usage(void)
 #endif
 		"\t-N\tdisable ICC workflow (\"N\"o color management)\n"
 		"\t-O -\tControl spot/overprint rendering\n"
-		"\t\t 0 = No spot rendering\n"
-#ifdef FZ_ENABLE_SPOT_RENDERING
+#if FZ_ENABLE_SPOT_RENDERING
 		"\t\t 0 = No spot rendering\n"
 		"\t\t 1 = Overprint simulation (default)\n"
 		"\t\t 2 = Full spot rendering\n"
@@ -441,7 +439,6 @@ file_level_headers(fz_context *ctx)
 		fz_parse_pclm_options(ctx, &opts, "compression=flate");
 		bander = fz_new_pclm_band_writer(ctx, out, &opts);
 	}
-
 }
 
 static void
@@ -460,10 +457,9 @@ file_level_trailers(fz_context *ctx)
 
 	if (output_format == OUT_PCLM)
 		fz_drop_band_writer(ctx, bander);
-
 }
 
-static void drawband(fz_context *ctx, fz_page *page, fz_display_list *list, const fz_matrix *ctm, const fz_rect *tbounds, fz_cookie *cookie, int band_start, fz_pixmap *pix, fz_bitmap **bit)
+static void drawband(fz_context *ctx, fz_page *page, fz_display_list *list, fz_matrix ctm, fz_rect tbounds, fz_cookie *cookie, int band_start, fz_pixmap *pix, fz_bitmap **bit)
 {
 	fz_device *dev = NULL;
 
@@ -478,7 +474,7 @@ static void drawband(fz_context *ctx, fz_page *page, fz_display_list *list, cons
 		else
 			fz_clear_pixmap_with_value(ctx, pix, 255);
 
-		dev = fz_new_draw_device_with_proof(ctx, NULL, pix, proof_cs);
+		dev = fz_new_draw_device_with_proof(ctx, fz_identity, pix, proof_cs);
 		if (lowmemory)
 			fz_enable_device_hints(ctx, dev, FZ_NO_CACHE);
 		if (alphabits_graphics == 0)
@@ -519,12 +515,13 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 	fz_try(ctx)
 	{
 		if (list)
-			fz_bound_display_list(ctx, list, &mediabox);
+			mediabox = fz_bound_display_list(ctx, list);
 		else
-			fz_bound_page(ctx, page, &mediabox);
+			mediabox = fz_bound_page(ctx, page);
 	}
 	fz_catch(ctx)
 	{
+		fz_drop_display_list(ctx, list);
 		fz_drop_separations(ctx, seps);
 		fz_drop_page(ctx, page);
 		fz_rethrow(ctx);
@@ -540,9 +537,9 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 			if (lowmemory)
 				fz_enable_device_hints(ctx, dev, FZ_NO_CACHE);
 			if (list)
-				fz_run_display_list(ctx, list, dev, &fz_identity, &fz_infinite_rect, cookie);
+				fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, cookie);
 			else
-				fz_run_page(ctx, page, dev, &fz_identity, cookie);
+				fz_run_page(ctx, page, dev, fz_identity, cookie);
 			fz_write_printf(ctx, out, "</page>\n");
 			fz_close_device(ctx, dev);
 		}
@@ -566,7 +563,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		fz_matrix ctm;
 
 		zoom = resolution / 72;
-		fz_pre_scale(fz_rotate(&ctm, rotation), zoom, zoom);
+		ctm = fz_pre_scale(fz_rotate(rotation), zoom, zoom);
 
 		fz_var(text);
 
@@ -575,14 +572,14 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 			fz_stext_options stext_options;
 
 			stext_options.flags = (output_format == OUT_HTML || output_format == OUT_XHTML) ? FZ_STEXT_PRESERVE_IMAGES : 0;
-			text = fz_new_stext_page(ctx, &mediabox);
+			text = fz_new_stext_page(ctx, mediabox);
 			dev = fz_new_stext_device(ctx,  text, &stext_options);
 			if (lowmemory)
 				fz_enable_device_hints(ctx, dev, FZ_NO_CACHE);
 			if (list)
-				fz_run_display_list(ctx, list, dev, &ctm, &fz_infinite_rect, cookie);
+				fz_run_display_list(ctx, list, dev, ctm, fz_infinite_rect, cookie);
 			else
-				fz_run_page(ctx, page, dev, &ctm, cookie);
+				fz_run_page(ctx, page, dev, ctm, cookie);
 			fz_close_device(ctx, dev);
 			fz_drop_device(ctx, dev);
 			dev = NULL;
@@ -631,16 +628,16 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		{
 			pdf_obj *page_obj;
 
-			dev = pdf_page_write(ctx, pdfout, &mediabox, &resources, &contents);
+			dev = pdf_page_write(ctx, pdfout, mediabox, &resources, &contents);
 			if (list)
-				fz_run_display_list(ctx, list, dev, &fz_identity, NULL, cookie);
+				fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, cookie);
 			else
-				fz_run_page(ctx, page, dev, &fz_identity, cookie);
+				fz_run_page(ctx, page, dev, fz_identity, cookie);
 			fz_close_device(ctx, dev);
 			fz_drop_device(ctx, dev);
 			dev = NULL;
 
-			page_obj = pdf_add_page(ctx, pdfout, &mediabox, rotation, resources, contents);
+			page_obj = pdf_add_page(ctx, pdfout, mediabox, rotation, resources, contents);
 			pdf_insert_page(ctx, pdfout, -1, page_obj);
 			pdf_drop_obj(ctx, page_obj);
 		}
@@ -671,9 +668,8 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		fz_var(out);
 
 		zoom = resolution / 72;
-		fz_pre_rotate(fz_scale(&ctm, zoom, zoom), rotation);
-		tbounds = mediabox;
-		fz_transform_rect(&tbounds, &ctm);
+		ctm = fz_pre_rotate(fz_scale(zoom, zoom), rotation);
+		tbounds = fz_transform_rect(mediabox, ctm);
 
 		fz_try(ctx)
 		{
@@ -689,9 +685,9 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 			if (lowmemory)
 				fz_enable_device_hints(ctx, dev, FZ_NO_CACHE);
 			if (list)
-				fz_run_display_list(ctx, list, dev, &ctm, &tbounds, cookie);
+				fz_run_display_list(ctx, list, dev, ctm, tbounds, cookie);
 			else
-				fz_run_page(ctx, page, dev, &ctm, cookie);
+				fz_run_page(ctx, page, dev, ctm, cookie);
 			fz_close_device(ctx, dev);
 			fz_close_output(ctx, out);
 		}
@@ -723,15 +719,10 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		fz_var(bit);
 
 		zoom = resolution / 72;
-		fz_pre_scale(fz_rotate(&ctm, rotation), zoom, zoom);
+		ctm = fz_pre_scale(fz_rotate(rotation), zoom, zoom);
 
-		if (output_format == OUT_TGA)
-		{
-			fz_pre_scale(fz_pre_translate(&ctm, 0, -height), 1, -1);
-		}
-
-		tbounds = mediabox;
-		fz_round_rect(&ibounds, fz_transform_rect(&tbounds, &ctm));
+		tbounds = fz_transform_rect(mediabox, ctm);
+		ibounds = fz_round_rect(tbounds);
 
 		/* Make local copies of our width/height */
 		w = width;
@@ -778,13 +769,12 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 				else
 					scaley = scalex;
 			}
-			fz_scale(&scale_mat, scalex, scaley);
-			fz_concat(&ctm, &ctm, &scale_mat);
-			tbounds = mediabox;
-			fz_transform_rect(&tbounds, &ctm);
+			scale_mat = fz_scale(scalex, scaley);
+			ctm = fz_concat(ctm, scale_mat);
+			tbounds = fz_transform_rect(mediabox, ctm);
 		}
-		fz_round_rect(&ibounds, &tbounds);
-		fz_rect_from_irect(&tbounds, &ibounds);
+		ibounds = fz_round_rect(tbounds);
+		tbounds = fz_rect_from_irect(ibounds);
 
 		fz_try(ctx)
 		{
@@ -814,7 +804,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 					workers[band].tbounds = tbounds;
 					memset(&workers[band].cookie, 0, sizeof(fz_cookie));
 					workers[band].list = list;
-					workers[band].pix = fz_new_pixmap_with_bbox(ctx, colorspace, &band_ibounds, seps, alpha);
+					workers[band].pix = fz_new_pixmap_with_bbox(ctx, colorspace, band_ibounds, seps, alpha);
 					fz_set_pixmap_resolution(ctx, workers[band].pix, resolution, resolution);
 #ifndef DISABLE_MUTHREADS
 					DEBUG_THREADS(("Worker %d, Pre-triggering band %d\n", band, band));
@@ -826,7 +816,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 			}
 			else
 			{
-				pix = fz_new_pixmap_with_bbox(ctx, colorspace, &band_ibounds, seps, alpha);
+				pix = fz_new_pixmap_with_bbox(ctx, colorspace, band_ibounds, seps, alpha);
 				fz_set_pixmap_resolution(ctx, pix, resolution, resolution);
 			}
 
@@ -847,8 +837,6 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 					bander = fz_new_ps_band_writer(ctx, out);
 				else if (output_format == OUT_PSD)
 					bander = fz_new_psd_band_writer(ctx, out);
-				else if (output_format == OUT_TGA)
-					bander = fz_new_tga_band_writer(ctx, out, colorspace == fz_device_bgr(ctx));
 				else if (output_format == OUT_PWG)
 				{
 					if (out_cs == CS_MONO)
@@ -884,7 +872,7 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 					cookie->errors += w->cookie.errors;
 				}
 				else
-					drawband(ctx, page, list, &ctm, &tbounds, cookie, band * band_height, pix, &bit);
+					drawband(ctx, page, list, ctm, tbounds, cookie, band * band_height, pix, &bit);
 
 				if (output)
 				{
@@ -1000,7 +988,8 @@ static void dodrawpage(fz_context *ctx, fz_page *page, fz_display_list *list, in
 		}
 	}
 
-	fprintf(stderr, "\n");
+	if (!quiet || showfeatures || showtime || showmd5)
+		fprintf(stderr, "\n");
 
 	if (lowmemory)
 		fz_empty_store(ctx);
@@ -1032,8 +1021,8 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 	fz_device *dev = NULL;
 	int start;
 	fz_cookie cookie = { 0 };
-	fz_rect bounds;
 	fz_separations *seps = NULL;
+	const char *features = "";
 
 	fz_var(list);
 	fz_var(dev);
@@ -1084,11 +1073,11 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 	{
 		fz_try(ctx)
 		{
-			list = fz_new_display_list(ctx, fz_bound_page(ctx, page, &bounds));
+			list = fz_new_display_list(ctx, fz_bound_page(ctx, page));
 			dev = fz_new_list_device(ctx, list);
 			if (lowmemory)
 				fz_enable_device_hints(ctx, dev, FZ_NO_CACHE);
-			fz_run_page(ctx, page, dev, &fz_identity, &cookie);
+			fz_run_page(ctx, page, dev, fz_identity, &cookie);
 			fz_close_device(ctx, dev);
 		}
 		fz_always(ctx)
@@ -1120,9 +1109,9 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 		fz_try(ctx)
 		{
 			if (list)
-				fz_run_display_list(ctx, list, dev, &fz_identity, &fz_infinite_rect, NULL);
+				fz_run_display_list(ctx, list, dev, fz_identity, fz_infinite_rect, NULL);
 			else
-				fz_run_page(ctx, page, dev, &fz_identity, &cookie);
+				fz_run_page(ctx, page, dev, fz_identity, &cookie);
 			fz_close_device(ctx, dev);
 		}
 		fz_always(ctx)
@@ -1137,7 +1126,7 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 			fz_drop_page(ctx, page);
 			fz_rethrow(ctx);
 		}
-		fprintf(stderr, " %s", iscolor ? "color" : "grayscale");
+		features = iscolor ? " color" : " grayscale";
 	}
 
 	if (output_file_per_page)
@@ -1145,11 +1134,13 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 		char text_buffer[512];
 
 		bgprint_flush();
-		fz_close_output(ctx, out);
-		fz_drop_output(ctx, out);
+		if (out)
+		{
+			fz_close_output(ctx, out);
+			fz_drop_output(ctx, out);
+		}
 		fz_snprintf(text_buffer, sizeof(text_buffer), output, pagenum);
-		out = fz_new_output_with_path(ctx, text_buffer, output_append);
-		output_append = 1;
+		out = fz_new_output_with_path(ctx, text_buffer, 0);
 	}
 
 	if (bgprint.active)
@@ -1157,7 +1148,8 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 		bgprint_flush();
 		if (bgprint.active)
 		{
-			fprintf(stderr, "page %s %d", filename, pagenum);
+			if (!quiet || showfeatures || showtime || showmd5)
+				fprintf(stderr, "page %s %d%s", filename, pagenum, features);
 		}
 
 		bgprint.started = 1;
@@ -1176,7 +1168,8 @@ static void drawpage(fz_context *ctx, fz_document *doc, int pagenum)
 	}
 	else
 	{
-		fprintf(stderr, "page %s %d", filename, pagenum);
+		if (!quiet || showfeatures || showtime || showmd5)
+			fprintf(stderr, "page %s %d%s", filename, pagenum, features);
 		dodrawpage(ctx, page, list, pagenum, &cookie, start, 0, filename, 0, seps);
 	}
 }
@@ -1317,7 +1310,7 @@ static void worker_thread(void *arg)
 		mu_wait_semaphore(&me->start);
 		DEBUG_THREADS(("Worker %d woken for band %d\n", me->num, me->band));
 		if (me->band >= 0)
-			drawband(me->ctx, NULL, me->list, &me->ctm, &me->tbounds, &me->cookie, me->band * band_height, me->pix, &me->bit);
+			drawband(me->ctx, NULL, me->list, me->ctm, me->tbounds, &me->cookie, me->band * band_height, me->pix, &me->bit);
 		DEBUG_THREADS(("Worker %d completed band %d\n", me->num, me->band));
 		mu_trigger_semaphore(&me->stop);
 	}
@@ -1473,12 +1466,14 @@ int mudraw_main(int argc, char **argv)
 
 	fz_var(doc);
 
-	while ((c = fz_getopt(argc, argv, "p:o:F:R:r:w:h:fB:c:e:G:Is:A:DiW:H:S:T:U:XLvPl:y:NO:")) != -1)
+	while ((c = fz_getopt(argc, argv, "qp:o:F:R:r:w:h:fB:c:e:G:Is:A:DiW:H:S:T:U:XLvPl:y:NO:")) != -1)
 	{
 		switch (c)
 		{
 		default: usage(); break;
 
+		case 'q': quiet = 1; break;
+
 		case 'p': password = fz_optarg; break;
 
 		case 'o': output = fz_optarg; break;
@@ -1595,443 +1590,459 @@ int mudraw_main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (proof_filename)
-		proof_cs = fz_new_icc_colorspace_from_file(ctx, NULL, proof_filename);
+	fz_try(ctx)
+	{
+		if (proof_filename)
+			proof_cs = fz_new_icc_colorspace_from_file(ctx, FZ_COLORSPACE_NONE, proof_filename);
 
-	fz_set_text_aa_level(ctx, alphabits_text);
-	fz_set_graphics_aa_level(ctx, alphabits_graphics);
-	fz_set_graphics_min_line_width(ctx, min_line_width);
-	fz_set_cmm_engine(ctx, icc_engine);
+		fz_set_text_aa_level(ctx, alphabits_text);
+		fz_set_graphics_aa_level(ctx, alphabits_graphics);
+		fz_set_graphics_min_line_width(ctx, min_line_width);
+		fz_set_cmm_engine(ctx, icc_engine);
 
 #ifndef DISABLE_MUTHREADS
-	if (bgprint.active)
-	{
-		int fail = 0;
-		bgprint.ctx = fz_clone_context(ctx);
-		fail |= mu_create_semaphore(&bgprint.start);
-		fail |= mu_create_semaphore(&bgprint.stop);
-		fail |= mu_create_thread(&bgprint.thread, bgprint_worker, NULL);
-		if (fail)
+		if (bgprint.active)
 		{
-			fprintf(stderr, "bgprint startup failed\n");
-			exit(1);
+			int fail = 0;
+			bgprint.ctx = fz_clone_context(ctx);
+			fail |= mu_create_semaphore(&bgprint.start);
+			fail |= mu_create_semaphore(&bgprint.stop);
+			fail |= mu_create_thread(&bgprint.thread, bgprint_worker, NULL);
+			if (fail)
+			{
+				fprintf(stderr, "bgprint startup failed\n");
+				exit(1);
+			}
 		}
-	}
 
-	if (num_workers > 0)
-	{
-		int i;
-		int fail = 0;
-		workers = fz_calloc(ctx, num_workers, sizeof(*workers));
-		for (i = 0; i < num_workers; i++)
+		if (num_workers > 0)
 		{
-			workers[i].ctx = fz_clone_context(ctx);
-			workers[i].num = i;
-			fail |= mu_create_semaphore(&workers[i].start);
-			fail |= mu_create_semaphore(&workers[i].stop);
-			fail |= mu_create_thread(&workers[i].thread, worker_thread, &workers[i]);
+			int i;
+			int fail = 0;
+			workers = fz_calloc(ctx, num_workers, sizeof(*workers));
+			for (i = 0; i < num_workers; i++)
+			{
+				workers[i].ctx = fz_clone_context(ctx);
+				workers[i].num = i;
+				fail |= mu_create_semaphore(&workers[i].start);
+				fail |= mu_create_semaphore(&workers[i].stop);
+				fail |= mu_create_thread(&workers[i].thread, worker_thread, &workers[i]);
+			}
+			if (fail)
+			{
+				fprintf(stderr, "worker startup failed\n");
+				exit(1);
+			}
 		}
-		if (fail)
-		{
-			fprintf(stderr, "worker startup failed\n");
-			exit(1);
-		}
-	}
 #endif /* DISABLE_MUTHREADS */
 
-	if (layout_css)
-	{
-		fz_buffer *buf = fz_read_file(ctx, layout_css);
-		fz_set_user_css(ctx, fz_string_from_buffer(ctx, buf));
-		fz_drop_buffer(ctx, buf);
-	}
-
-	fz_set_use_document_css(ctx, layout_use_doc_css);
-
-	/* Determine output type */
-	if (band_height < 0)
-	{
-		fprintf(stderr, "Bandheight must be > 0\n");
-		exit(1);
-	}
-
-	output_format = OUT_PNG;
-	if (format)
-	{
-		int i;
-
-		for (i = 0; i < nelem(suffix_table); i++)
+		if (layout_css)
 		{
-			if (!strcmp(format, suffix_table[i].suffix+1))
+			fz_buffer *buf = fz_read_file(ctx, layout_css);
+			fz_set_user_css(ctx, fz_string_from_buffer(ctx, buf));
+			fz_drop_buffer(ctx, buf);
+		}
+
+		fz_set_use_document_css(ctx, layout_use_doc_css);
+
+		/* Determine output type */
+		if (band_height < 0)
+		{
+			fprintf(stderr, "Bandheight must be > 0\n");
+			exit(1);
+		}
+
+		output_format = OUT_PNG;
+		if (format)
+		{
+			int i;
+
+			for (i = 0; i < nelem(suffix_table); i++)
 			{
-				output_format = suffix_table[i].format;
-				if (spots == SPOTS_FULL && suffix_table[i].spots == 0)
+				if (!strcmp(format, suffix_table[i].suffix+1))
 				{
-					fprintf(stderr, "Output format '%s' does not support spot rendering.\nDoing overprint simulation instead.\n", suffix_table[i].suffix+1);
-					spots = SPOTS_OVERPRINT_SIM;
-				}
-				break;
-			}
-		}
-		if (i == nelem(suffix_table))
-		{
-			fprintf(stderr, "Unknown output format '%s'\n", format);
-			exit(1);
-		}
-	}
-	else if (output)
-	{
-		char *suffix = output;
-		int i;
-
-		for (i = 0; i < nelem(suffix_table); i++)
-		{
-			char *s = strstr(suffix, suffix_table[i].suffix);
-
-			if (s != NULL)
-			{
-				suffix = s+1;
-				output_format = suffix_table[i].format;
-				if (spots == SPOTS_FULL && suffix_table[i].spots == 0)
-				{
-					fprintf(stderr, "Output format '%s' does not support spot rendering.\nDoing overprint simulation instead.\n", suffix_table[i].suffix+1);
-					spots = SPOTS_OVERPRINT_SIM;
-				}
-				i = 0;
-			}
-		}
-	}
-
-	if (band_height)
-	{
-		if (output_format != OUT_PAM && output_format != OUT_PGM && output_format != OUT_PPM && output_format != OUT_PNM && output_format != OUT_PNG && output_format != OUT_PBM && output_format != OUT_PKM && output_format != OUT_PCL && output_format != OUT_PCLM && output_format != OUT_PS && output_format != OUT_PSD && output_format != OUT_TGA)
-		{
-			fprintf(stderr, "Banded operation only possible with PxM, PCL, PCLM, PS, PSD, PNG and TGA outputs\n");
-			exit(1);
-		}
-		if (showmd5)
-		{
-			fprintf(stderr, "Banded operation not compatible with MD5\n");
-			exit(1);
-		}
-	}
-
-	{
-		int i, j;
-
-		for (i = 0; i < nelem(format_cs_table); i++)
-		{
-			if (format_cs_table[i].format == output_format)
-			{
-				if (out_cs == CS_UNSET)
-					out_cs = format_cs_table[i].default_cs;
-				for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
-				{
-					if (format_cs_table[i].permitted_cs[j] == out_cs)
-						break;
-				}
-				if (j == nelem(format_cs_table[i].permitted_cs))
-				{
-					fprintf(stderr, "Unsupported colorspace for this format\n");
-					exit(1);
-				}
-			}
-		}
-	}
-
-	alpha = 1;
-	switch (out_cs)
-	{
-	case CS_MONO:
-	case CS_GRAY:
-	case CS_GRAY_ALPHA:
-		colorspace = fz_device_gray(ctx);
-		alpha = (out_cs == CS_GRAY_ALPHA);
-		break;
-	case CS_RGB:
-	case CS_RGB_ALPHA:
-		colorspace = fz_device_rgb(ctx);
-		alpha = (out_cs == CS_RGB_ALPHA);
-		break;
-	case CS_CMYK:
-	case CS_CMYK_ALPHA:
-		colorspace = fz_device_cmyk(ctx);
-		alpha = (out_cs == CS_CMYK_ALPHA);
-		break;
-	case CS_ICC:
-		fz_try(ctx)
-			colorspace = fz_new_icc_colorspace_from_file(ctx, NULL, icc_filename);
-		fz_catch(ctx)
-		{
-			fprintf(stderr, "Invalid ICC destination color space\n");
-			exit(1);
-		}
-		if (colorspace == NULL)
-		{
-			fprintf(stderr, "Invalid ICC destination color space\n");
-			exit(1);
-		}
-		alpha = 0;
-		break;
-	default:
-		fprintf(stderr, "Unknown colorspace!\n");
-		exit(1);
-		break;
-	}
-
-	if (out_cs != CS_ICC)
-		colorspace = fz_keep_colorspace(ctx, colorspace);
-	else
-	{
-		int i, j, okay;
-
-		/* Check to make sure this icc profile is ok with the output format */
-		okay = 0;
-		for (i = 0; i < nelem(format_cs_table); i++)
-		{
-			if (format_cs_table[i].format == output_format)
-			{
-				for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
-				{
-					switch (format_cs_table[i].permitted_cs[j])
+					output_format = suffix_table[i].format;
+					if (spots == SPOTS_FULL && suffix_table[i].spots == 0)
 					{
-					case CS_MONO:
-					case CS_GRAY:
-					case CS_GRAY_ALPHA:
-						if (fz_colorspace_n(ctx, colorspace) == 1)
-							okay = 1;
-						break;
-					case CS_RGB:
-					case CS_RGB_ALPHA:
-						if (fz_colorspace_n(ctx, colorspace) == 3)
-							okay = 1;
-						break;
-					case CS_CMYK:
-					case CS_CMYK_ALPHA:
-						if (fz_colorspace_n(ctx, colorspace) == 4)
-							okay = 1;
-						break;
+						fprintf(stderr, "Output format '%s' does not support spot rendering.\nDoing overprint simulation instead.\n", suffix_table[i].suffix+1);
+						spots = SPOTS_OVERPRINT_SIM;
+					}
+					break;
+				}
+			}
+			if (i == nelem(suffix_table))
+			{
+				fprintf(stderr, "Unknown output format '%s'\n", format);
+				exit(1);
+			}
+		}
+		else if (output)
+		{
+			char *suffix = output;
+			int i;
+
+			for (i = 0; i < nelem(suffix_table); i++)
+			{
+				char *s = strstr(suffix, suffix_table[i].suffix);
+
+				if (s != NULL)
+				{
+					suffix = s+1;
+					output_format = suffix_table[i].format;
+					if (spots == SPOTS_FULL && suffix_table[i].spots == 0)
+					{
+						fprintf(stderr, "Output format '%s' does not support spot rendering.\nDoing overprint simulation instead.\n", suffix_table[i].suffix+1);
+						spots = SPOTS_OVERPRINT_SIM;
+					}
+					i = 0;
+				}
+			}
+		}
+
+		if (band_height)
+		{
+			if (output_format != OUT_PAM && output_format != OUT_PGM && output_format != OUT_PPM && output_format != OUT_PNM && output_format != OUT_PNG && output_format != OUT_PBM && output_format != OUT_PKM && output_format != OUT_PCL && output_format != OUT_PCLM && output_format != OUT_PS && output_format != OUT_PSD)
+			{
+				fprintf(stderr, "Banded operation only possible with PxM, PCL, PCLM, PS, PSD, and PNG outputs\n");
+				exit(1);
+			}
+			if (showmd5)
+			{
+				fprintf(stderr, "Banded operation not compatible with MD5\n");
+				exit(1);
+			}
+		}
+
+		{
+			int i, j;
+
+			for (i = 0; i < nelem(format_cs_table); i++)
+			{
+				if (format_cs_table[i].format == output_format)
+				{
+					if (out_cs == CS_UNSET)
+						out_cs = format_cs_table[i].default_cs;
+					for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
+					{
+						if (format_cs_table[i].permitted_cs[j] == out_cs)
+							break;
+					}
+					if (j == nelem(format_cs_table[i].permitted_cs))
+					{
+						fprintf(stderr, "Unsupported colorspace for this format\n");
+						exit(1);
 					}
 				}
 			}
 		}
 
-		if (!okay)
+		alpha = 1;
+		switch (out_cs)
 		{
-			fprintf(stderr, "ICC profile uses a colorspace that cannot be used for this format\n");
-			exit(1);
-		}
-	}
-
-#if FZ_ENABLE_PDF
-	if (output_format == OUT_PDF)
-	{
-		pdfout = pdf_create_document(ctx);
-	}
-	else
-#endif
-	if (output_format == OUT_GPROOF || output_format == OUT_SVG)
-	{
-		/* GPROOF files are saved direct. Do not open "output". */
-		/* SVG files are always opened for each page. Do not open "output". */
-	}
-	else if (output && (output[0] != '-' || output[1] != 0) && *output != 0)
-	{
-		if (has_percent_d(output))
-			output_file_per_page = 1;
-		else
-			out = fz_new_output_with_path(ctx, output, 0);
-	}
-	else
-	{
-#ifdef _WIN32
-		/* Windows specific code to make stdout binary. */
-		if (output_format != OUT_TEXT && output_format != OUT_STEXT && output_format != OUT_HTML && output_format != OUT_XHTML && output_format != OUT_TRACE)
-			setmode(fileno(stdout), O_BINARY);
-#endif
-		out = fz_stdout(ctx);
-	}
-
-	if (!output_file_per_page)
-		file_level_headers(ctx);
-
-	timing.count = 0;
-	timing.total = 0;
-	timing.min = 1 << 30;
-	timing.max = 0;
-	timing.mininterp = 1 << 30;
-	timing.maxinterp = 0;
-	timing.minpage = 0;
-	timing.maxpage = 0;
-	timing.minfilename = "";
-	timing.maxfilename = "";
-	if (showtime && bgprint.active)
-		timing.total = gettime();
-
-	fz_try(ctx)
-	{
-		fz_register_document_handlers(ctx);
-
-		while (fz_optind < argc)
-		{
-			fz_try(ctx)
-			{
-				filename = argv[fz_optind++];
-				files++;
-
-				doc = fz_open_document(ctx, filename);
-
-				/* Once document is open check for output intent colorspace */
-				oi = fz_document_output_intent(ctx, doc);
-				if (oi)
+			case CS_MONO:
+			case CS_GRAY:
+			case CS_GRAY_ALPHA:
+				colorspace = fz_device_gray(ctx);
+				alpha = (out_cs == CS_GRAY_ALPHA);
+				break;
+			case CS_RGB:
+			case CS_RGB_ALPHA:
+				colorspace = fz_device_rgb(ctx);
+				alpha = (out_cs == CS_RGB_ALPHA);
+				break;
+			case CS_CMYK:
+			case CS_CMYK_ALPHA:
+				colorspace = fz_device_cmyk(ctx);
+				alpha = (out_cs == CS_CMYK_ALPHA);
+				break;
+			case CS_ICC:
+				fz_try(ctx)
+					colorspace = fz_new_icc_colorspace_from_file(ctx, FZ_COLORSPACE_NONE, icc_filename);
+				fz_catch(ctx)
 				{
-					/* See if we had explicitly set a profile to render */
-					if (out_cs != CS_ICC)
+					fprintf(stderr, "Invalid ICC destination color space\n");
+					exit(1);
+				}
+				if (colorspace == NULL)
+				{
+					fprintf(stderr, "Invalid ICC destination color space\n");
+					exit(1);
+				}
+				alpha = 0;
+				break;
+			default:
+				fprintf(stderr, "Unknown colorspace!\n");
+				exit(1);
+				break;
+		}
+
+		if (out_cs != CS_ICC)
+			colorspace = fz_keep_colorspace(ctx, colorspace);
+		else
+		{
+			int i, j, okay;
+
+			/* Check to make sure this icc profile is ok with the output format */
+			okay = 0;
+			for (i = 0; i < nelem(format_cs_table); i++)
+			{
+				if (format_cs_table[i].format == output_format)
+				{
+					for (j = 0; j < nelem(format_cs_table[i].permitted_cs); j++)
 					{
-						/* In this case, we want to render to the output intent
-						 * color space if the number of channels is the same */
-						if (fz_colorspace_n(ctx, oi) == fz_colorspace_n(ctx, colorspace))
+						switch (format_cs_table[i].permitted_cs[j])
 						{
-							fz_drop_colorspace(ctx, colorspace);
-							colorspace = fz_keep_colorspace(ctx, oi);
+							case CS_MONO:
+							case CS_GRAY:
+							case CS_GRAY_ALPHA:
+								if (fz_colorspace_is_gray(ctx, colorspace))
+									okay = 1;
+								break;
+							case CS_RGB:
+							case CS_RGB_ALPHA:
+								if (fz_colorspace_is_rgb(ctx, colorspace))
+									okay = 1;
+								break;
+							case CS_CMYK:
+							case CS_CMYK_ALPHA:
+								if (fz_colorspace_is_cmyk(ctx, colorspace))
+									okay = 1;
+								break;
 						}
 					}
 				}
-
-				if (fz_needs_password(ctx, doc))
-				{
-					if (!fz_authenticate_password(ctx, doc, password))
-						fz_throw(ctx, FZ_ERROR_GENERIC, "cannot authenticate password: %s", filename);
-				}
-
-				fz_layout_document(ctx, doc, layout_w, layout_h, layout_em);
-
-				if (layer_config)
-					apply_layer_config(ctx, doc, layer_config);
-
-				if (output_format == OUT_GPROOF)
-				{
-					fz_save_gproof(ctx, filename, doc, output, resolution, "", "");
-				}
-				else
-				{
-					if (fz_optind == argc || !fz_is_page_range(ctx, argv[fz_optind]))
-						drawrange(ctx, doc, "1-N");
-					if (fz_optind < argc && fz_is_page_range(ctx, argv[fz_optind]))
-						drawrange(ctx, doc, argv[fz_optind++]);
-				}
-
-				bgprint_flush();
-				fz_drop_document(ctx, doc);
-				doc = NULL;
 			}
-			fz_catch(ctx)
+
+			if (!okay)
 			{
-				fz_drop_document(ctx, doc);
-				doc = NULL;
-
-				if (!ignore_errors)
-					fz_rethrow(ctx);
-
-				bgprint_flush();
-				fz_warn(ctx, "ignoring error in '%s'", filename);
+				fprintf(stderr, "ICC profile uses a colorspace that cannot be used for this format\n");
+				exit(1);
 			}
 		}
-	}
-	fz_catch(ctx)
-	{
-		bgprint_flush();
-		fz_drop_document(ctx, doc);
-		fprintf(stderr, "error: cannot draw '%s'\n", filename);
-		errored = 1;
-	}
-
-	if (!output_file_per_page)
-		file_level_trailers(ctx);
 
 #if FZ_ENABLE_PDF
-	if (output_format == OUT_PDF)
-	{
-		if (!output)
-			output = "out.pdf";
-		pdf_save_document(ctx, pdfout, output, NULL);
-		pdf_drop_document(ctx, pdfout);
-	}
-	else
-#endif
-	if (output_format == OUT_GPROOF)
-	{
-		/* No output file to close */
-	}
-	else
-	{
-		fz_close_output(ctx, out);
-		fz_drop_output(ctx, out);
-		out = NULL;
-	}
-
-	if (showtime && timing.count > 0)
-	{
-		if (bgprint.active)
-			timing.total = gettime() - timing.total;
-
-		if (files == 1)
+		if (output_format == OUT_PDF)
 		{
-			fprintf(stderr, "total %dms / %d pages for an average of %dms\n",
-				timing.total, timing.count, timing.total / timing.count);
-			if (bgprint.active)
+			pdfout = pdf_create_document(ctx);
+		}
+		else
+#endif
+			if (output_format == OUT_GPROOF)
 			{
-				fprintf(stderr, "fastest page %d: %dms (interpretation) %dms (rendering) %dms(total)\n",
-					timing.minpage, timing.mininterp, timing.min - timing.mininterp, timing.min);
-				fprintf(stderr, "slowest page %d: %dms (interpretation) %dms (rendering) %dms(total)\n",
-					timing.maxpage, timing.maxinterp, timing.max - timing.maxinterp, timing.max);
+				/* GPROOF files are saved direct. Do not open "output". */
+				if (!output)
+					fz_throw(ctx, FZ_ERROR_GENERIC, "output filename required when saving GProof file");
+			}
+			else if (output_format == OUT_SVG)
+			{
+				/* SVG files are always opened for each page. Do not open "output". */
+			}
+			else if (output && (output[0] != '-' || output[1] != 0) && *output != 0)
+			{
+				if (has_percent_d(output))
+					output_file_per_page = 1;
+				else
+					out = fz_new_output_with_path(ctx, output, 0);
 			}
 			else
 			{
-				fprintf(stderr, "fastest page %d: %dms\n", timing.minpage, timing.min);
-				fprintf(stderr, "slowest page %d: %dms\n", timing.maxpage, timing.max);
+				quiet = 1; /* automatically be quiet if printing to stdout */
+#ifdef _WIN32
+				/* Windows specific code to make stdout binary. */
+				if (output_format != OUT_TEXT && output_format != OUT_STEXT && output_format != OUT_HTML && output_format != OUT_XHTML && output_format != OUT_TRACE)
+					setmode(fileno(stdout), O_BINARY);
+#endif
+				out = fz_stdout(ctx);
+			}
+
+		filename = argv[fz_optind];
+		if (!output_file_per_page)
+			file_level_headers(ctx);
+
+		timing.count = 0;
+		timing.total = 0;
+		timing.min = 1 << 30;
+		timing.max = 0;
+		timing.mininterp = 1 << 30;
+		timing.maxinterp = 0;
+		timing.minpage = 0;
+		timing.maxpage = 0;
+		timing.minfilename = "";
+		timing.maxfilename = "";
+		if (showtime && bgprint.active)
+			timing.total = gettime();
+
+		fz_try(ctx)
+		{
+			fz_register_document_handlers(ctx);
+
+			while (fz_optind < argc)
+			{
+				fz_try(ctx)
+				{
+					filename = argv[fz_optind++];
+					files++;
+
+					doc = fz_open_document(ctx, filename);
+
+					if (fz_needs_password(ctx, doc))
+					{
+						if (!fz_authenticate_password(ctx, doc, password))
+							fz_throw(ctx, FZ_ERROR_GENERIC, "cannot authenticate password: %s", filename);
+					}
+
+					/* Once document is open check for output intent colorspace */
+					oi = fz_document_output_intent(ctx, doc);
+					if (oi)
+					{
+						/* See if we had explicitly set a profile to render */
+						if (out_cs != CS_ICC)
+						{
+							/* In this case, we want to render to the output intent
+							 * color space if the number of channels is the same */
+							if (fz_colorspace_n(ctx, oi) == fz_colorspace_n(ctx, colorspace))
+							{
+								fz_drop_colorspace(ctx, colorspace);
+								colorspace = fz_keep_colorspace(ctx, oi);
+							}
+						}
+					}
+
+					fz_layout_document(ctx, doc, layout_w, layout_h, layout_em);
+
+					if (layer_config)
+						apply_layer_config(ctx, doc, layer_config);
+
+					if (output_format == OUT_GPROOF)
+					{
+						fz_save_gproof(ctx, filename, doc, output, resolution, "", "");
+					}
+					else
+					{
+						if (fz_optind == argc || !fz_is_page_range(ctx, argv[fz_optind]))
+							drawrange(ctx, doc, "1-N");
+						if (fz_optind < argc && fz_is_page_range(ctx, argv[fz_optind]))
+							drawrange(ctx, doc, argv[fz_optind++]);
+					}
+
+					bgprint_flush();
+				}
+				fz_always(ctx)
+				{
+					fz_drop_document(ctx, doc);
+					doc = NULL;
+				}
+				fz_catch(ctx)
+				{
+					if (!ignore_errors)
+						fz_rethrow(ctx);
+
+					bgprint_flush();
+					fz_warn(ctx, "ignoring error in '%s'", filename);
+				}
 			}
 		}
-		else
+		fz_catch(ctx)
 		{
-			fprintf(stderr, "total %dms / %d pages for an average of %dms in %d files\n",
-				timing.total, timing.count, timing.total / timing.count, files);
-			fprintf(stderr, "fastest page %d: %dms (%s)\n", timing.minpage, timing.min, timing.minfilename);
-			fprintf(stderr, "slowest page %d: %dms (%s)\n", timing.maxpage, timing.max, timing.maxfilename);
+			bgprint_flush();
+			fz_drop_document(ctx, doc);
+			fprintf(stderr, "error: cannot draw '%s'\n", filename);
+			errored = 1;
+		}
+
+		if (!output_file_per_page)
+			file_level_trailers(ctx);
+
+#if FZ_ENABLE_PDF
+		if (output_format == OUT_PDF)
+		{
+			if (!output)
+				output = "out.pdf";
+			pdf_save_document(ctx, pdfout, output, NULL);
+			pdf_drop_document(ctx, pdfout);
+		}
+		else
+#endif
+			if (output_format == OUT_GPROOF || output_format == OUT_SVG)
+			{
+				/* No output file to close */
+			}
+			else
+			{
+				fz_close_output(ctx, out);
+				fz_drop_output(ctx, out);
+				out = NULL;
+			}
+
+		if (showtime && timing.count > 0)
+		{
+			if (bgprint.active)
+				timing.total = gettime() - timing.total;
+
+			if (files == 1)
+			{
+				fprintf(stderr, "total %dms / %d pages for an average of %dms\n",
+						timing.total, timing.count, timing.total / timing.count);
+				if (bgprint.active)
+				{
+					fprintf(stderr, "fastest page %d: %dms (interpretation) %dms (rendering) %dms(total)\n",
+							timing.minpage, timing.mininterp, timing.min - timing.mininterp, timing.min);
+					fprintf(stderr, "slowest page %d: %dms (interpretation) %dms (rendering) %dms(total)\n",
+							timing.maxpage, timing.maxinterp, timing.max - timing.maxinterp, timing.max);
+				}
+				else
+				{
+					fprintf(stderr, "fastest page %d: %dms\n", timing.minpage, timing.min);
+					fprintf(stderr, "slowest page %d: %dms\n", timing.maxpage, timing.max);
+				}
+			}
+			else
+			{
+				fprintf(stderr, "total %dms / %d pages for an average of %dms in %d files\n",
+						timing.total, timing.count, timing.total / timing.count, files);
+				fprintf(stderr, "fastest page %d: %dms (%s)\n", timing.minpage, timing.min, timing.minfilename);
+				fprintf(stderr, "slowest page %d: %dms (%s)\n", timing.maxpage, timing.max, timing.maxfilename);
+			}
 		}
-	}
 
 #ifndef DISABLE_MUTHREADS
-	if (num_workers > 0)
-	{
-		int i;
-		for (i = 0; i < num_workers; i++)
+		if (num_workers > 0)
 		{
-			workers[i].band = -1;
-			mu_trigger_semaphore(&workers[i].start);
-			mu_wait_semaphore(&workers[i].stop);
-			mu_destroy_semaphore(&workers[i].start);
-			mu_destroy_semaphore(&workers[i].stop);
-			mu_destroy_thread(&workers[i].thread);
-			fz_drop_context(workers[i].ctx);
+			int i;
+			for (i = 0; i < num_workers; i++)
+			{
+				workers[i].band = -1;
+				mu_trigger_semaphore(&workers[i].start);
+				mu_wait_semaphore(&workers[i].stop);
+				mu_destroy_semaphore(&workers[i].start);
+				mu_destroy_semaphore(&workers[i].stop);
+				mu_destroy_thread(&workers[i].thread);
+				fz_drop_context(workers[i].ctx);
+			}
+			fz_free(ctx, workers);
 		}
-		fz_free(ctx, workers);
-	}
 
-	if (bgprint.active)
-	{
-		bgprint.pagenum = -1;
-		mu_trigger_semaphore(&bgprint.start);
-		mu_wait_semaphore(&bgprint.stop);
-		mu_destroy_semaphore(&bgprint.start);
-		mu_destroy_semaphore(&bgprint.stop);
-		mu_destroy_thread(&bgprint.thread);
-		fz_drop_context(bgprint.ctx);
-	}
+		if (bgprint.active)
+		{
+			bgprint.pagenum = -1;
+			mu_trigger_semaphore(&bgprint.start);
+			mu_wait_semaphore(&bgprint.stop);
+			mu_destroy_semaphore(&bgprint.start);
+			mu_destroy_semaphore(&bgprint.stop);
+			mu_destroy_thread(&bgprint.thread);
+			fz_drop_context(bgprint.ctx);
+		}
 #endif /* DISABLE_MUTHREADS */
+	}
+	fz_always(ctx)
+	{
+		fz_drop_colorspace(ctx, colorspace);
+		fz_drop_colorspace(ctx, proof_cs);
+	}
+	fz_catch(ctx)
+	{
+	}
 
-	fz_drop_colorspace(ctx, colorspace);
-	fz_drop_colorspace(ctx, proof_cs);
 	fz_drop_context(ctx);
 
 #ifndef DISABLE_MUTHREADS
