@@ -26,12 +26,16 @@ static void usage(void)
 		"\t-ggg\tin addition to -gg merge duplicate objects\n"
 		"\t-gggg\tin addition to -ggg check streams for duplication\n"
 		"\t-l\tlinearize PDF\n"
+		"\t-D\tsave file without encryption\n"
 		"\t-a\tascii hex encode binary streams\n"
 		"\t-d\tdecompress streams\n"
 		"\t-z\tdeflate uncompressed streams\n"
 		"\t-f\tcompress font streams\n"
 		"\t-i\tcompress image streams\n"
-		"\t-s\tclean content streams\n"
+		"\t-c\tclean content streams\n"
+		"\t-s\tsanitize content streams\n"
+		"\t-A\tcreate appearance streams for annotations\n"
+		"\t-AA\trecreate appearance streams for annotations\n"
 		"\tpages\tcomma separated list of page numbers and ranges\n"
 		);
 	exit(1);
@@ -47,10 +51,7 @@ int pdfclean_main(int argc, char **argv)
 	int errors = 0;
 	fz_context *ctx;
 
-	opts.continue_on_error = 1;
-	opts.errors = &errors;
-
-	while ((c = fz_getopt(argc, argv, "adfgilp:sz")) != -1)
+	while ((c = fz_getopt(argc, argv, "adfgilp:sczDA")) != -1)
 	{
 		switch (c)
 		{
@@ -63,7 +64,10 @@ int pdfclean_main(int argc, char **argv)
 		case 'a': opts.do_ascii += 1; break;
 		case 'g': opts.do_garbage += 1; break;
 		case 'l': opts.do_linear += 1; break;
-		case 's': opts.do_clean += 1; break;
+		case 'c': opts.do_clean += 1; break;
+		case 's': opts.do_sanitize += 1; break;
+		case 'D': opts.do_decrypt += 1; break;
+		case 'A': opts.do_appearance += 1; break;
 		default: usage(); break;
 		}
 	}
